@@ -8,7 +8,7 @@
 
 (* Certification of Imperative Programs / Jean-Christophe Fillitre *)
 
-(* $Id: pmisc.mli,v 1.4 2001/11/05 16:46:59 barras Exp $ *)
+(* $Id: pmisc.mli,v 1.6 2002/02/21 10:19:59 filliatr Exp $ *)
 
 open Names
 open Term
@@ -70,6 +70,6 @@ val abstract : (identifier * constr) list -> constr -> constr
 
 (* for debugging purposes *)
 
-val debug : bool ref
 val deb_mess : Pp.std_ppcmds -> unit
+val deb_print : ('a -> Pp.std_ppcmds) -> 'a -> unit
 
