@@ -8,7 +8,7 @@
 
 (* Certification of Imperative Programs / Jean-Christophe Fillitre *)
 
-(* $Id: putil.ml,v 1.7 2001/12/13 09:03:01 filliatr Exp $ *)
+(* $Id: putil.ml,v 1.8 2002/02/08 09:05:45 filliatr Exp $ *)
 
 open Util
 open Names
@@ -232,7 +232,8 @@ and c_of_constr c =
 
 open Pp
 open Util
-open Printer
+
+let prterm x = Printer.prterm_env (Global.env()) x
 
 let pp_pre = function
     [] -> (mt ())
