@@ -6,23 +6,29 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: common.mli,v 1.5 2001/12/10 14:54:35 letouzey Exp $ i*)
+(*i $Id: common.mli,v 1.9 2002/04/08 14:33:14 letouzey Exp $ i*)
 
+open Pp
 open Miniml
 open Mlutil
 open Names
 open Nametab
 
 module ToplevelPp : Mlpp
-
-val sp_of_r : global_reference -> section_path
-
-val long_module : global_reference -> dir_path
+module OcamlMonoPp : Mlpp
+module HaskellMonoPp : Mlpp
 
 val is_long_module : dir_path -> global_reference -> bool
 
 val short_module : global_reference -> identifier
 
+val pp_logical_ind : global_reference -> std_ppcmds
+
+val pp_singleton_ind : global_reference -> std_ppcmds
+
+val set_globals : unit -> unit
+
 val extract_to_file : 
-  string -> extraction_params -> ml_decl list -> unit
+  string option -> extraction_params -> ml_decl list -> unit
+
 
