@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: extraction.mli,v 1.16 2001/11/14 13:36:50 letouzey Exp $ i*)
+(*i $Id: extraction.mli,v 1.21 2002/03/28 12:41:15 letouzey Exp $ i*)
 
 (*s Extraction from Coq terms to Miniml. *)
 
@@ -18,13 +18,7 @@ open Nametab
 
 (*s Result of an extraction. *)
 
-type info = Logic | Info
-
-type arity = Arity | NotArity
-
-type type_var = info * arity
-
-type signature = type_var list
+type signature = bool list
 
 type extraction_result =
   | Emltype of ml_type * signature * identifier list
@@ -37,3 +31,14 @@ val extract_constr : env -> constr -> extraction_result
 (*s ML declaration corresponding to a Coq reference. *)
 
 val extract_declaration : global_reference -> ml_decl
+
+(*s Check whether a global reference corresponds to a logical inductive. *)
+
+val decl_is_logical_ind : global_reference -> bool
+
+(*s Check if a global reference corresponds to the constructor of 
+  a singleton inductive. *)
+
+val decl_is_singleton : global_reference -> bool
+
+val extract_type : env -> constr -> constr list -> int list -> ml_type
