@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: haskell.mli,v 1.6 2001/11/05 16:47:03 barras Exp $ i*)
+(*i $Id: haskell.mli,v 1.10 2002/04/08 12:25:11 letouzey Exp $ i*)
 
 open Pp
 open Names
@@ -15,6 +15,6 @@ open Miniml
 
 val keywords : Idset.t
 
-val preamble : extraction_params -> std_ppcmds
+val preamble : extraction_params -> Idset.t -> bool -> std_ppcmds
 
 module Make : functor(P : Mlpp_param) -> Mlpp
