@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: ocaml.ml,v 1.50 2001/12/21 12:27:29 letouzey Exp $ i*)
+(*i $Id: ocaml.ml,v 1.68 2002/04/08 14:33:14 letouzey Exp $ i*)
 
 (*s Production of Ocaml syntax. *)
 
@@ -23,6 +23,8 @@ open Nametab
 
 let current_module = ref None
 
+let cons_cofix = ref Refset.empty
+
 (*s Some utility functions. *)
 
 let rec collapse_type_app = function
@@ -82,14 +84,23 @@ type env = identifier list * Idset.t
 let rec rename_vars avoid = function
   | [] -> 
       [], avoid
-  | id :: idl when id == prop_name ->
-      (* we don't rename propositions binders *)
+  | id :: idl when id == dummy_name ->
+      (* we don't rename dummy binders *)
       let (idl', avoid') = rename_vars avoid idl in
       (id :: idl', avoid')
   | id :: idl ->
-      let id' = rename_id (lowercase_id id) avoid in
-      let (idl', avoid') = rename_vars (Idset.add id' avoid) idl in
-      (id' :: idl', avoid')
+      let (idl, avoid) = rename_vars avoid idl in
+      let id = rename_id (lowercase_id id) avoid in
+      (id :: idl, Idset.add id avoid)
+
+let rename_tvars avoid l = 
+  let rec rename avoid = function 
+    | [] -> [],avoid 
+    | id :: idl -> 
+	let id = rename_id (lowercase_id id) avoid in 
+	let idl, avoid = rename (Idset.add id avoid) idl in 
+	(id :: idl, avoid) in
+  fst (rename avoid l)
 
 let push_vars ids (db,avoid) =
   let ids',avoid' = rename_vars avoid ids in
@@ -108,50 +119,51 @@ let keywords =
     "module"; "mutable"; "new"; "object"; "of"; "open"; "or";
     "parser"; "private"; "rec"; "sig"; "struct"; "then"; "to"; "true";
     "try"; "type"; "val"; "virtual"; "when"; "while"; "with"; "mod";
-    "land"; "lor"; "lxor"; "lsl"; "lsr"; "asr" ; "prop" ; "arity" ] 
+    "land"; "lor"; "lxor"; "lsl"; "lsr"; "asr" ; "unit" ; "_" ; "__" ] 
   Idset.empty
 
-let preamble _ =
-  (str "type prop = unit" ++ fnl () ++
-     str "let prop = ()" ++ fnl () ++ fnl () ++
-     str "type arity = unit" ++ fnl () ++
-     str "let arity = ()" ++ fnl () ++ fnl ())
+let preamble _ used_modules print_dummy = 
+  Idset.fold (fun m s -> s ++ str "open " ++ pr_id (uppercase_id m) ++ fnl())
+    used_modules (mt ())
+  ++ 
+  (if Idset.is_empty used_modules  then mt () else fnl ())
+  ++
+  (if print_dummy then 
+     str "let (__:unit) = let rec f _ = Obj.magic f in Obj.magic f" 
+     ++ fnl () ++ fnl()
+   else mt ())
 
 (*s The pretty-printing functor. *)
 
 module Make = functor(P : Mlpp_param) -> struct
 
-let pp_type_global = P.pp_type_global 
-let pp_global = P.pp_global
-let rename_global = P.rename_global
+let pp_type_global r = P.pp_global r false (Some (P.globals()))
+let pp_global r env = P.pp_global r false (Some (snd env))
+let pp_global' r = P.pp_global r false None
+let rename_global r = P.rename_global r false 
 
 let empty_env () = [], P.globals()
 
 (*s Pretty-printing of types. [par] is a boolean indicating whether parentheses
     are needed or not. *)
 
-let rec pp_type par t =
+let rec pp_type par vl t =
   let rec pp_rec par = function
-    | Tvar id -> 
-	pp_tvar id
+    | Tvar i -> (try pp_tvar (List.nth vl (pred i)) 
+		 with _ -> (str "'a" ++ int i))
     | Tapp l ->
 	(match collapse_type_app l with
 	   | [] -> assert false
 	   | [t] -> pp_rec par t
-	   | t::l -> (pp_tuple (pp_rec false) l ++ 
-			sec_space_if (l <>[]) ++ 
-			pp_rec false t))
+	   | [t;u] -> pp_rec true u ++ spc () ++ pp_rec false t 
+	   | t::l -> (pp_tuple (pp_rec false) l ++ spc () ++  
+		      pp_rec false t))
     | Tarr (t1,t2) ->
 	(open_par par ++ pp_rec true t1 ++ spc () ++ str "->" ++ spc () ++ 
 	   pp_rec false t2 ++ close_par par)
-    | Tglob r -> 
-	pp_type_global r
-    | Texn s -> 
-	str ("unit (* " ^ s ^ " *)")
-    | Tprop ->
-	str "prop"
-    | Tarity ->
-	str "arity"
+    | Tglob r -> pp_type_global r
+    | Tdummy -> str "unit"
+    | Tunknown -> str "Obj.t"
   in 
   hov 0 (pp_rec par t)
 
@@ -162,6 +174,7 @@ let rec pp_type par t =
 
 let expr_needs_par = function
   | MLlam _  -> true
+  | MLcase (_,[|_|]) -> false 
   | MLcase _ -> true
   | _        -> false 
 
@@ -175,9 +188,7 @@ let rec pp_expr par env args =
   in
   function
     | MLrel n -> 
-	let id = get_db_name n env in 
-	apply (if string_of_id id = "_" then str "prop" else pr_id id)
-	  (* HACK, should disappear soon *)
+	let id = get_db_name n env in apply (pr_id id)
     | MLapp (f,args') ->
 	let stl = List.map (pp_expr true env []) args' in
         pp_expr par env (stl @ args) f
@@ -185,69 +196,96 @@ let rec pp_expr par env args =
       	let fl,a' = collect_lams a in
 	let fl,env' = push_vars fl env in
 	let st = (pp_abst (List.rev fl) ++ pp_expr false env' [] a') in
-	(open_par par' ++ st ++ close_par par')
+	apply (open_par par' ++ st ++ close_par par')
     | MLletin (id,a1,a2) ->
-	let id',env' = push_vars [id] env in
+	let i,env' = push_vars [id] env in
 	let par2 = not par' && expr_needs_par a2 in
 	apply 
-	  (hov 0 (open_par par' ++
-		    hov 2 (str "let " ++ pr_id (List.hd id') ++ str " =" ++ spc () ++
-			     pp_expr false env [] a1 ++ spc () ++ str "in") ++
-		    spc () ++
-		    pp_expr par2 env' [] a2 ++
-		    close_par par'))
+	  (hv 0 
+	     (hv 0 (open_par par' ++
+		    hov 2 
+		      (str "let " ++ pr_id (List.hd i) ++ str " =" ++ spc ()
+			++ pp_expr false env [] a1) ++ 
+		    spc () ++ str "in") ++ 
+	      spc () ++ hov 0 (pp_expr par2 env' [] a2) ++ close_par par'))
     | MLglob r -> 
-	apply (pp_global r)
+	apply (pp_global r env )
     | MLcons (r,[]) ->
-	pp_global r
+	assert (args=[]);
+	if Refset.mem r !cons_cofix then 
+	  (open_par par ++ str "lazy " ++ pp_global r env ++ close_par par)
+	else pp_global r env
     | MLcons (r,[a]) ->
-	(open_par par ++ pp_global r ++ spc () ++
+	assert (args=[]);
+	if Refset.mem r !cons_cofix then 
+	  (open_par par ++ str "lazy (" ++
+	   pp_global r env ++ spc () ++
+	   pp_expr true env [] a ++ str ")" ++ close_par par)
+	else
+	  (open_par par ++ pp_global r env ++ spc () ++
 	   pp_expr true env [] a ++ close_par par)
     | MLcons (r,args') ->
-	(open_par par ++ pp_global r ++ spc () ++
+	assert (args=[]);
+	if Refset.mem r !cons_cofix then 
+	  (open_par par ++ str "lazy (" ++ pp_global r env ++ spc () ++
+	   pp_tuple (pp_expr true env []) args' ++ str ")" ++ close_par par)
+	else
+	  (open_par par ++ pp_global r env ++ spc () ++
 	   pp_tuple (pp_expr true env []) args' ++ close_par par)
-    | MLcase (t,[|x|])->
+    | MLcase (t,[|(r,_,_) as x|])->
+	let expr = if Refset.mem r !cons_cofix then 
+	  (str "Lazy.force" ++ spc () ++ pp_expr true env [] t)
+	else 
+	  (pp_expr false env [] t) 
+	in 
+	let s1,s2 = pp_one_pat env x in 
 	apply 
-	  (hov 0 (open_par par' ++ str "let " ++  
-		    pp_one_pat 
-		      (str " =" ++ spc () ++
-			 pp_expr false env [] t ++ spc () ++ str "in") 
-		      env x ++
-		    close_par par'))
+	  (hv 0 
+	     (hv 0 (open_par par' ++ 
+		    hov 2 (str "let " ++ s1 ++ str " =" ++ spc () ++ expr) ++ 
+		    spc () ++ str "in") ++ 
+	      spc () ++ hov 0 s2 ++ close_par par'))
     | MLcase (t, pv) ->
-      	apply
-      	  (open_par par' ++
-      	     v 0 (str "match " ++ pp_expr false env [] t ++ str " with" ++
-		    fnl () ++ str "  " ++ pp_pat env pv) ++
-	     close_par par')
+	let r,_,_ = pv.(0) in 
+	let expr = if Refset.mem r !cons_cofix then 
+	  (str "Lazy.force" ++ spc () ++ pp_expr true env [] t)
+	else 
+	  (pp_expr false env [] t) 	
+	in apply
+      	     (open_par par' ++
+      	      v 0 (str "match " ++ expr ++ str " with" ++
+		   fnl () ++ str "  | " ++ pp_pat env pv) ++
+	      close_par par')
     | MLfix (i,ids,defs) ->
 	let ids',env' = push_vars (List.rev (Array.to_list ids)) env in
       	pp_fix par env' (Some i) (Array.of_list (List.rev ids'),defs) args
     | MLexn s -> 
+	(* An [MLexn] may be applied, but I don't really care. *)
 	(open_par par ++ str "assert false" ++ spc () ++ 
-	   str ("(* "^s^" *)") ++ close_par par)
-    | MLprop ->
-	str "prop"
-    | MLarity ->
-	str "arity"
+	 str ("(* "^s^" *)") ++ close_par par)
+    | MLdummy ->
+	str "()" (* An [MLdummy] may be applied, but I don't really care. *)
+    | MLdummy' -> 
+	str "__" (* idem *)
     | MLcast (a,t) ->
-	(open_par true ++ pp_expr false env args a ++ spc () ++ str ":" ++ spc () ++ 
-	   pp_type false t ++ close_par true)
+	(open_par true ++ pp_expr false env args a ++ spc () ++ str ":" ++ 
+	 spc () ++ pp_type false [] t ++ close_par true)
     | MLmagic a ->
 	(open_par true ++ str "Obj.magic" ++ spc () ++ 
-	   pp_expr false env args a ++ close_par true)
+	 pp_expr false env args a ++ close_par true)
 
-and pp_one_pat s env (r,ids,t) = 
+and pp_one_pat env (r,ids,t) = 
   let ids,env' = push_vars (List.rev ids) env in
   let par = expr_needs_par t in
   let args = 
     if ids = [] then (mt ()) 
     else str " " ++ pp_boxed_tuple pr_id (List.rev ids) in 
-  pp_global r ++ args ++ s ++ spc () ++ pp_expr par env' [] t
+  (pp_global r env ++ args), (pp_expr par env' [] t)
   
 and pp_pat env pv = 
-  prvect_with_sep (fun () -> (fnl () ++ str "| ")) 
-    (fun x -> hov 2 (pp_one_pat (str " ->") env x)) pv
+  prvect_with_sep (fun () -> (fnl () ++ str "  | ")) 
+    (fun x -> let s1,s2 = pp_one_pat env x in 
+     hov 2 (s1 ++ str " ->" ++ spc () ++ s2)) pv
 
 (*s names of the functions ([ids]) are already pushed in [env],
     and passed here just for convenience. *)
@@ -280,17 +318,19 @@ and pp_function env f t =
     let ktl = array_map_to_list (fun (_,l,t0) -> (List.length l,t0)) pv in
     not (List.exists (fun (k,t0) -> Mlutil.occurs (k+1) t0) ktl)
   in
+  let is_not_cofix pv = let (r,_,_) = pv.(0) in not (Refset.mem r !cons_cofix) 
+  in					  
   match t' with 
-    | MLcase(MLrel 1,pv) ->
+    | MLcase(MLrel 1,pv) when is_not_cofix pv ->
 	if is_function pv then
 	  (f ++ pr_binding (List.rev (List.tl bl)) ++
        	     str " = function" ++ fnl () ++
-	     v 0 (str "  " ++ pp_pat env' pv))
+	     v 0 (str "  | " ++ pp_pat env' pv))
 	else
           (f ++ pr_binding (List.rev bl) ++ 
              str " = match " ++
 	     pr_id (List.hd bl) ++ str " with" ++ fnl () ++
-	     v 0 (str "  " ++ pp_pat env' pv))
+	     v 0 (str "  | " ++ pp_pat env' pv))
 	  
     | _ -> (f ++ pr_binding (List.rev bl) ++
 	      str " =" ++ fnl () ++ str "  " ++
@@ -301,61 +341,73 @@ let pp_ast a = hov 0 (pp_expr false (empty_env ()) [] a)
 (*s Pretty-printing of inductive types declaration. *)
 
 let pp_parameters l = 
-  (pp_tuple pp_tvar l ++ space_if (l<>[]))
+  (pp_boxed_tuple pp_tvar l ++ space_if (l<>[]))
 
-let pp_one_inductive (pl,name,cl) =
+let pp_one_ind prefix (pl,name,cl) =
+  let pl = rename_tvars keywords pl in
   let pp_constructor (id,l) =
-    (pp_global id ++
-       match l with
-         | [] -> (mt ()) 
-	 | _  -> (str " of " ++
-      	       	    prlist_with_sep 
-		      (fun () -> (spc () ++ str "* ")) (pp_type true) l))
+    (pp_global' id ++
+     match l with
+       | [] -> (mt ()) 
+       | _  -> (str " of " ++
+      		prlist_with_sep 
+		  (fun () -> (spc () ++ str "* ")) 
+		  (pp_type true (List.rev pl)) l))
   in
-  (pp_parameters pl ++ pp_type_global name ++ str " =" ++ 
-     (fnl () ++
-	v 0 (str "    " ++
-	       prlist_with_sep (fun () -> (fnl () ++ str "  | "))
-                 (fun c -> hov 2 (pp_constructor c)) cl)))
-  
-let pp_inductive il =
+  (pp_parameters pl ++ str prefix ++ pp_type_global name ++ str " =" ++ 
+   if cl = [] then str " unit (* empty inductive *)" 
+   else (fnl () ++
+	 v 0 (str "  | " ++
+	      prlist_with_sep (fun () -> (fnl () ++ str "  | "))
+                (fun c -> hov 2 (pp_constructor c)) cl)))
+
+let pp_ind il =
   (str "type " ++
-     prlist_with_sep (fun () -> (fnl () ++ str "and ")) pp_one_inductive il ++
-     fnl ())
+   prlist_with_sep (fun () -> (fnl () ++ str "and ")) (pp_one_ind "") il
+   ++ fnl ())
+
+let pp_coind_preamble (pl,name,_) = 
+  let pl = rename_tvars keywords pl in
+  (pp_parameters pl ++ pp_type_global name ++ str " = " ++ 
+    pp_parameters pl ++ str "__" ++ pp_type_global name ++ str " Lazy.t")
+
+let pp_coind il = 
+  (str "type " ++ 
+   prlist_with_sep (fun () -> (fnl () ++ str "and ")) pp_coind_preamble il ++
+   fnl () ++ str "and " ++ 
+   prlist_with_sep (fun () -> (fnl () ++ str "and ")) (pp_one_ind "__") il ++
+   fnl ())
 
 (*s Pretty-printing of a declaration. *)
 
-let warning_coinductive r = 
-  warn (hov 0 
-	  (str "You are trying to extract the CoInductive definition" ++ spc () ++
-	     Printer.pr_global r ++ spc () ++ str "in Ocaml." ++ spc () ++ 
-	     str "This is in general NOT a good idea," ++ spc () ++ 
-	     str "since Ocaml is not lazy." ++ spc () ++
-	     str "You should consider using Haskell instead."))
-
 let pp_decl = function
-  | Dtype ([], _) -> 
-      if P.toplevel then hov 0 (str " prop (* Logic inductive *)" ++ fnl ())
-      else (mt ()) 
-  | Dtype ((_,r,_)::_ as i, cofix) -> 
-      if cofix && (not P.toplevel) then if_verbose warning_coinductive r; 
-      hov 0 (pp_inductive i)
+  | Dtype ([], _) -> mt ()
+  | Dtype (i, cofix) -> 
+      if cofix then begin
+	List.iter   
+	  (fun (_,_,l) -> 
+	     List.iter (fun (r,_) -> 
+			  cons_cofix := Refset.add r !cons_cofix)  l) i; 
+	hov 0 (pp_coind i)
+      end else 
+	hov 0 (pp_ind i)
   | Dabbrev (r, l, t) ->
+      let l = rename_tvars keywords l in 
       hov 0 (str "type" ++ spc () ++ pp_parameters l ++ 
 	       pp_type_global r ++ spc () ++ str "=" ++ spc () ++ 
-	       pp_type false  t ++ fnl ())
-  | Dglob (r, MLfix (_,[|_|],[|def|])) ->
-      let id = rename_global r in
-      let env' = [id], P.globals() in
-      (hov 2 (pp_fix false env' None ([|id|],[|def|]) []))
+	       pp_type false (List.rev l) t ++ fnl ())
+  | Dfix (rv, defs) ->
+      let ids = Array.map rename_global rv in 
+      let env = List.rev (Array.to_list ids), P.globals() in
+      (hov 2 (pp_fix false env None (ids,defs) []))
   | Dglob (r, a) ->
       hov 0 (str "let " ++ 
-	       pp_function (empty_env ()) (pp_global r) a ++ fnl ())
+	       pp_function (empty_env ()) (pp_global' r) a ++ fnl ())
   | Dcustom (r,s) -> 
-      hov 0 (str "let " ++ pp_global r ++ 
+      hov 0 (str "let " ++ pp_global' r ++ 
 	       str " =" ++ spc () ++ str s ++ fnl ())
 
-let pp_type = pp_type false
+let pp_type = pp_type false []
 
 end
 
