@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: table.mli,v 1.3 2001/11/05 16:47:05 barras Exp $ i*)
+(*i $Id: table.mli,v 1.4 2002/03/04 15:48:51 letouzey Exp $ i*)
 
 open Vernacinterp
 open Names
@@ -30,6 +30,12 @@ val check_constant : global_reference -> global_reference
 
 val refs_of_vargl : vernac_arg list -> global_reference list
 
+(*s Target language. *)
+
+type lang = Ocaml | Haskell | Toplevel
+
+val lang : unit -> lang 
+
 (*s Table for custom inlining *) 
 
 val to_inline : global_reference -> bool
