@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: fourierR.ml,v 1.4 2001/11/05 16:47:05 barras Exp $ *)
+(* $Id: fourierR.ml,v 1.5 2002/04/19 08:12:51 filliatr Exp $ *)
 
 
 
@@ -436,7 +436,7 @@ let rec fourier gl=
     let res=fourier_lineq (!lineq) in
     let tac=ref tclIDTAC in
     if res=[]
-    then (print_string "Tactic Fourier fails.";
+    then (print_string "Tactic Fourier fails.\n";
 		       flush stdout)
     (* l'algorithme de Fourier a russi: on va en tirer une preuve Coq *)
     else (match res with
