@@ -58,14 +58,21 @@ let ctf_FileErrorMessage reqid pps =
 (*In the code for CoqV6.2, the require_module call is encapsulated in
   a function "without_mes_ambig".  Here I have supposed that this
   function has no effect on parsing *)
-let try_require_module import specif name fname =
+let try_require_module import specif names =
  try Library.require_module 
    (if specif = "UNSPECIFIED" then None
     else Some (specif = "SPECIFICATION"))
-   (Nametab.make_short_qualid (Names.id_of_string name))
-   fname (import = "IMPORT")
+   (List.map (fun name -> Nametab.make_short_qualid (Names.id_of_string name))
+      names)
+   (import = "IMPORT")
  with
-   | e -> msgnl (str "Reinterning of " ++ str name ++ str " failed");;
+   | e -> msgnl (str "Reinterning of " ++ prlist str names ++ str " failed");;
+
+let try_require_module_from_file import specif name fname =
+ try Library.require_module_from_file (if specif = "UNSPECIFIED" then None
+  else Some (specif = "SPECIFICATION")) (Nametab.make_short_qualid (Names.id_of_string name)) fname (import = "IMPORT")
+ with
+ | e -> msgnl (str "Reinterning of " ++ str name ++ str " failed");;
 
 let execute_when_necessary ast =
  (match ast with
@@ -74,13 +81,16 @@ let execute_when_necessary ast =
   | Node (_, "TOKEN", ((Str (_, s)) :: [])) -> Metasyntax.add_token_obj s
   | Node (_, "Require",
             ((Str (_, import)) ::
-              ((Str (_, specif)) :: ((Nvar (_, mname)) :: [])))) ->
-   try_require_module import specif mname None
+              ((Str (_, specif)) :: l))) ->
+   let mnames = List.map (function
+     | (Nvar (_, m)) -> m
+     | _ -> error "parse_string_action : bad require expression") l in
+   try_require_module import specif mnames
   | Node (_, "RequireFrom",
             ((Str (_, import)) ::
               ((Str (_, specif)) ::
                 ((Nvar (_, mname)) :: ((Str (_, file_name)) :: []))))) ->
-   try_require_module import specif mname (Some file_name)
+   try_require_module_from_file import specif mname file_name
   | _ -> ()); ast;;
 
 let parse_to_dot =
@@ -161,7 +171,7 @@ let parse_command_list reqid stream string_list =
         with
         | (Stdpp.Exc_located(l, Stream.Error txt)) as e -> 
           begin
-             msgnl (ctf_SyntaxWarningMessage reqid (Errors.explain_exn e));
+             msgnl (ctf_SyntaxWarningMessage reqid (Cerrors.explain_exn e));
              try
                 discard_to_dot stream;
                 msgnl (str "debug" ++ fnl () ++ int this_pos ++ fnl () ++
@@ -234,11 +244,11 @@ let parse_string_action reqid phylum char_stream string_list =
  | Stdpp.Exc_located(l,Match_failure(_,_,_)) ->
     flush_until_end_of_stream char_stream;
     msgnl (ctf_SyntaxErrorMessage reqid
-              (Errors.explain_exn 
+              (Cerrors.explain_exn 
                  (Stdpp.Exc_located(l,Stream.Error "match failure"))))
  | e ->
     flush_until_end_of_stream char_stream;
-    msgnl (ctf_SyntaxErrorMessage reqid (Errors.explain_exn e));;
+    msgnl (ctf_SyntaxErrorMessage reqid (Cerrors.explain_exn e));;
 
 
 let quiet_parse_string_action char_stream =
@@ -272,7 +282,7 @@ let parse_file_action reqid file_name =
                    msgnl (ctf_SyntaxWarningMessage reqid
 			    (str "Error with file" ++ spc () ++
 			     str file_name ++ fnl () ++
-			     Errors.explain_exn 
+			     Cerrors.explain_exn 
 				 (Stdpp.Exc_located(l,Stream.Error txt))));
 		   (try 
 		      begin
@@ -318,13 +328,13 @@ let parse_file_action reqid file_name =
 	       (ctf_SyntaxErrorMessage reqid
 		  (str "Error with file" ++ spc () ++ str file_name ++ 
 		   fnl () ++
-		     Errors.explain_exn
+		     Cerrors.explain_exn
 		       (Stdpp.Exc_located(l,Stream.Error "match failure"))))
 	 | e ->
 	     msgnl
 	       (ctf_SyntaxErrorMessage reqid
 		  (str "Error with file" ++ spc () ++ str file_name ++
-		   fnl () ++ Errors.explain_exn e));;
+		   fnl () ++ Cerrors.explain_exn e));;
 
 (* This function is taken from Mltop.add_path *)
 
@@ -363,7 +373,7 @@ let add_path_action reqid string_arg =
 
 let print_version_action () =
   msgnl (mt ());
-  msgnl (str "$Id: parse.ml,v 1.8 2001/12/18 14:02:29 bertot Exp $");;
+  msgnl (str "$Id: parse.ml,v 1.10 2002/02/20 11:06:05 ddr Exp $");;
 
 let load_syntax_action reqid module_name =
  msg (str "loading " ++ str module_name ++ str "... ");
@@ -371,8 +381,7 @@ let load_syntax_action reqid module_name =
       (let qid = Nametab.make_short_qualid (Names.id_of_string module_name) in
       read_module qid;
       msg (str "opening... ");
-      let fullname = Nametab.locate_loaded_library qid in 
-      import_module fullname; 
+      import_module false qid; 
       msgnl (str "done" ++ fnl ());
       ())
  with
@@ -433,7 +442,7 @@ Libobject.relax true;
    with
      | End_of_file -> ()
      | e ->
-	 (msgnl (Errors.explain_exn e);
+	 (msgnl (Cerrors.explain_exn e);
 	  msgnl (str "could not load the VERNACRC file"));
 	 try
 	   msgnl (str vernacrc)
@@ -450,11 +459,11 @@ Libobject.relax true;
  with
    | End_of_file -> ()
    | e ->
-       msgnl (Errors.explain_exn e);
+       msgnl (Cerrors.explain_exn e);
        msgnl (str "error in your .vernacrc file"));
 msgnl (str "Starting Centaur Specialized Parser Loop");
 try
   coqparser_loop stdin
 with 
   | End_of_file -> ()
-  | e -> msgnl(Errors.explain_exn e))
+  | e -> msgnl(Cerrors.explain_exn e))
