@@ -1,6 +1,4 @@
 (* A proof by pointing algorithm. *)
-
-
 open Util;;
 open Names;;
 open Term;;
@@ -9,7 +7,7 @@ open Tacticals;;
 open Hipattern;;
 open Pattern;;
 open Reduction;;
-open Coqast;;
+open Ctast;;
 open Rawterm;;
 open Environ;;
 
@@ -20,14 +18,21 @@ open Typing;;
 open Pp;;
 
 (* get_hyp_by_name : goal sigma -> string -> constr,
-   looks up for an hypothesis, from its name *)
+   looks up for an hypothesis (or a global constant), from its name *)
 let get_hyp_by_name g name =
   let evd = project g in
   let env = pf_env g in
-  let judgment = 
-    Pretyping.understand_judgment 
-      evd env (RVar(dummy_loc, id_of_string name)) in
-    judgment.uj_type;;
+  try (let judgment = 
+         Pretyping.understand_judgment 
+          evd env (RVar(dummy_loc, id_of_string name)) in
+       ("hyp",judgment.uj_type))
+(* je sais, c'est pas beau, mais je ne sais pas trop me servir de look_up...
+   Loc *)
+  with _ -> (let parse_ast = Pcoq.parse_string Pcoq.Constr.constr in
+             let parse s = Astterm.interp_constr Evd.empty (Global.env())
+	          (parse_ast s) in
+             ("cste",type_of (Global.env()) Evd.empty (parse name)))
+;;
 
 type pbp_rule = (identifier list *
                     string list *
@@ -38,8 +43,8 @@ type pbp_rule = (identifier list *
                     (identifier list ->
                      string list ->
                      bool ->
-                       string option -> (types, constr) kind_of_term -> int list -> Coqast.t)) ->
-                    Coqast.t option;;
+                       string option -> (types, constr) kind_of_term -> int list -> Ctast.t)) ->
+                    Ctast.t option;;
 
 let zz = (0,0);;
 
@@ -62,8 +67,9 @@ let get_name_from_intro = function
 let make_clears =  function
     [] -> Node(zz, "Idtac",[])
   | str_list ->
-      Node(zz,"Clear",
-	   [Node(zz, "CLAUSE", List.map (function s -> Nvar(zz,s)) str_list)]);;
+      Node(zz, "TRY", [Node(zz,"Clear",
+	   [Node(zz, "CLAUSE", List.map (function s -> Nvar(zz,s)) str_list)])
+	       ]);;
 
 let add_clear_names_if_necessary tactic clear_names =
     match clear_names with
@@ -75,7 +81,7 @@ let make_final_cmd f optname clear_names constr path =
     add_clear_names_if_necessary (f optname constr path) clear_names;;
 
 let (rem_cast:pbp_rule) = function
-    (a,c,cf,o, IsCast(f,_), p, func) ->
+    (a,c,cf,o, Cast(f,_), p, func) ->
       Some(func a c cf o (kind_of_term f) p)
   | _ -> None;;
 
@@ -84,7 +90,7 @@ let (forall_intro: pbp_rule) = function
    clear_names,
    clear_flag,
    None,
-   IsProd(Name x, _, body),
+   Prod(Name x, _, body),
    (2::path),
    f) ->
      let x' = next_global_ident_away x avoid in
@@ -95,7 +101,7 @@ let (forall_intro: pbp_rule) = function
 
 let (imply_intro2: pbp_rule) = function
   avoid, clear_names,
-  clear_flag, None, IsProd(Anonymous, _, body), 2::path, f ->
+  clear_flag, None, Prod(Anonymous, _, body), 2::path, f ->
    let h' = next_global_ident_away (id_of_string "H") avoid in
      Some(Node(zz, "TACTICLIST",
             [make_named_intro (string_of_id h');
@@ -105,7 +111,7 @@ let (imply_intro2: pbp_rule) = function
 
 let (imply_intro1: pbp_rule) = function
   avoid, clear_names,
-  clear_flag, None, IsProd(Anonymous, prem, body), 1::path, f ->
+  clear_flag, None, Prod(Anonymous, prem, body), 1::path, f ->
    let h' = next_global_ident_away (id_of_string "H") avoid in
    let str_h' = (string_of_id h') in
      Some(Node(zz, "TACTICLIST",
@@ -117,7 +123,7 @@ let (imply_intro1: pbp_rule) = function
 
 let (forall_elim: pbp_rule) = function
   avoid, clear_names, clear_flag, 
-  Some h, IsProd(Name x, _, body), 2::path, f ->
+  Some h, Prod(Name x, _, body), 2::path, f ->
   let h' = next_global_ident_away (id_of_string "H") avoid in
   let clear_names' = if clear_flag then h::clear_names else clear_names in
   let str_h' = (string_of_id h') in
@@ -135,7 +141,7 @@ let (forall_elim: pbp_rule) = function
           
 let (imply_elim1: pbp_rule) = function
   avoid, clear_names, clear_flag,
-  Some h, IsProd(Anonymous, prem, body), 1::path, f ->
+  Some h, Prod(Anonymous, prem, body), 1::path, f ->
   let clear_names' = if clear_flag then h::clear_names else clear_names in
   let h' = next_global_ident_away (id_of_string "H") avoid in
   let str_h' = (string_of_id h') in
@@ -156,7 +162,7 @@ let (imply_elim1: pbp_rule) = function
 
 let (imply_elim2: pbp_rule) = function
   avoid, clear_names, clear_flag,
-  Some h, IsProd(Anonymous, prem, body), 2::path, f ->
+  Some h, Prod(Anonymous, prem, body), 2::path, f ->
   let clear_names' = if clear_flag then h::clear_names else clear_names in
   let h' = next_global_ident_away (id_of_string "H") avoid in
   let str_h' = (string_of_id h') in
@@ -176,7 +182,8 @@ let (imply_elim2: pbp_rule) = function
   | _ -> None;;
 
 let reference dir s =
-  let dir = "Coq"::"Init"::[dir] in
+  let dir = make_dirpath
+              (List.map id_of_string (List.rev ("Coq"::"Init"::[dir]))) in
   let id = id_of_string s in
   try 
     Nametab.locate_in_absolute_module dir id
@@ -184,9 +191,7 @@ let reference dir s =
     anomaly ("Coqlib: cannot find "^
 	     (Nametab.string_of_qualid (Nametab.make_qualid dir id)))
 
-let constant dir s =
-  Declare.constr_of_reference Evd.empty (Global.env()) (reference dir s);;
-
+let constant dir s =  Declare.constr_of_reference (reference dir s);;
 
 let andconstr: unit -> constr = Coqlib.build_coq_and;;
 let prodconstr () = constant "Datatypes" "prod";;
@@ -204,7 +209,7 @@ let is_matching_local a b = is_matching (pattern_of_constr a) b;;
 
 let (and_intro: pbp_rule) = function
     avoid, clear_names, clear_flag,
-    None, IsApp(and_oper, [|c1; c2|]), 2::a::path, f 
+    None, App(and_oper, [|c1; c2|]), 2::a::path, f 
       ->
       if ((is_matching_local (andconstr()) and_oper) or
           (is_matching_local (prodconstr ()) and_oper)) & (a = 1 or a = 2) then
@@ -229,12 +234,12 @@ let (and_intro: pbp_rule) = function
 
 let (ex_intro: pbp_rule) = function
     avoid, clear_names, clear_flag, None,
-    IsApp(oper, [| c1; c2|]), 2::2::2::path, f
+    App(oper, [| c1; c2|]), 2::2::2::path, f
       when (is_matching_local (exconstr ()) oper) or (is_matching_local (exTconstr ()) oper)
  	or (is_matching_local (sigconstr ()) oper)
 	      or (is_matching_local (sigTconstr ()) oper) ->
 		(match kind_of_term c2 with
-		  IsLambda(Name x, _, body) ->
+		  Lambda(Name x, _, body) ->
 		    Some(Node(zz, "Split",
 			      [Node(zz, "BINDINGS",
 				    [Node(zz, "BINDING",
@@ -250,7 +255,7 @@ let (ex_intro: pbp_rule) = function
 
 let (or_intro: pbp_rule) = function
     avoid, clear_names, clear_flag, None,
-    IsApp(or_oper, [|c1; c2 |]), 2::a::path, f ->
+    App(or_oper, [|c1; c2 |]), 2::a::path, f ->
       if ((is_matching_local (orconstr ()) or_oper) or
         (is_matching_local (sumboolconstr ()) or_oper) or 
 	(is_matching_local (sumconstr ()) or_oper))
@@ -270,7 +275,7 @@ let dummy_id = id_of_string "Dummy";;
 
 let (not_intro: pbp_rule) = function
     avoid, clear_names, clear_flag, None,
-    IsApp(not_oper, [|c1|]), 2::1::path, f ->
+    App(not_oper, [|c1|]), 2::1::path, f ->
       if(is_matching_local (notconstr ()) not_oper) or 
 	(is_matching_local (notTconstr ()) not_oper) then
 	let h' = next_global_ident_away (id_of_string "H") avoid in
@@ -336,11 +341,11 @@ let rec down_prods: (types, constr) kind_of_term * (int list) * int ->
            string list * (int list) * int * (types, constr) kind_of_term *
   (int list) = 
    function
-     IsProd(Name x, _, body), 2::path, k ->
+     Prod(Name x, _, body), 2::path, k ->
 	let res_sl, res_il, res_i, res_cstr, res_p 
 	    = down_prods (kind_of_term body, path, k+1) in
 	(string_of_id x)::res_sl, (k::res_il), res_i, res_cstr, res_p
-   | IsProd(Anonymous, _, body), 2::path, k ->
+   | Prod(Anonymous, _, body), 2::path, k ->
         let res_sl, res_il, res_i, res_cstr, res_p 
 	    = down_prods (kind_of_term body, path, k+1) in
 	res_sl, res_il, res_i+1, res_cstr, res_p
@@ -361,14 +366,14 @@ let (check_apply: (types, constr) kind_of_term -> (int list) -> bool) =
      | [] -> []
      | p::tl -> if n = p then tl else p::(delete n tl) in
    let rec check_rec l = function
-   | IsApp(f, array) ->
+   | App(f, array) ->
        Array.fold_left (fun l c -> check_rec l (kind_of_term c))
 	 (check_rec l (kind_of_term f)) array
-   | IsConst _ -> l
-   | IsMutInd _ -> l
-   | IsMutConstruct _ -> l
-   | IsVar _ -> l
-   | IsRel p ->
+   | Const _ -> l
+   | Ind _ -> l
+   | Construct _ -> l
+   | Var _ -> l
+   | Rel p ->
        let result = delete p l in
        if result = [] then
 	 raise (Pbp_internal [])
@@ -399,7 +404,7 @@ let (head_tactic_patt: pbp_rule) = function
     avoid, clear_names, clear_flag, Some h, cstr, path, f ->
     (match down_prods (cstr, path, 0) with
     | (str_list, _, nprems, 
-       IsApp(oper,[|c1|]), 2::1::path) 
+       App(oper,[|c1|]), 2::1::path) 
       	when
  	  (is_matching_local (notconstr ()) oper) or
  	  (is_matching_local (notTconstr ()) oper) ->
@@ -407,7 +412,7 @@ let (head_tactic_patt: pbp_rule) = function
 		   [elim_with_bindings h str_list;
 		     f avoid clear_names false None (kind_of_term c1) path]))
     | (str_list, _, nprems, 
-       IsApp(oper, [|c1; c2|]), 2::a::path) 
+       App(oper, [|c1; c2|]), 2::a::path) 
       when ((is_matching_local (andconstr()) oper) or
 	    (is_matching_local (prodconstr()) oper)) & (a = 1 or a = 2) ->
 	let h1 = next_global_ident_away (id_of_string "H") avoid in
@@ -431,18 +436,18 @@ let (head_tactic_patt: pbp_rule) = function
 			   cont_tac::(auxiliary_goals
 					clear_names clear_flag
 					h nprems))]))
-    | (str_list, _, nprems, IsApp(oper,[|c1; c2|]), 2::a::path)
+    | (str_list, _, nprems, App(oper,[|c1; c2|]), 2::a::path)
       when ((is_matching_local (exconstr ()) oper) or
 	    (is_matching_local (exTconstr ()) oper) or
 	    (is_matching_local (sigconstr ()) oper) or
 	    (is_matching_local (sigTconstr()) oper)) & a = 2 ->
 		   (match (kind_of_term c2),path with
-		     IsLambda(Name x, _,body), (2::path) ->
+		     Lambda(Name x, _,body), (2::path) ->
 		       Some(Node(zz,"TACTICLIST",
 		    		 [elim_with_bindings h str_list;
 				   let x' = next_global_ident_away x avoid in
 				   let cont_body =
-				     IsProd(Name x', c1,
+				     Prod(Name x', c1,
 					    mkProd(Anonymous, body, 
 						   mkVar(dummy_id))) in
 				   let cont_tac 
@@ -456,7 +461,7 @@ let (head_tactic_patt: pbp_rule) = function
 						       clear_names clear_flag
 						       h nprems))]))
 		   | _ -> None)
-    | (str_list, _, nprems, IsApp(oper,[|c1; c2|]), 2::a::path)
+    | (str_list, _, nprems, App(oper,[|c1; c2|]), 2::a::path)
 	when ((is_matching_local (orconstr ()) oper) or
               (is_matching_local (sumboolconstr ()) oper) or
 	      (is_matching_local (sumconstr ()) oper)) &
@@ -491,7 +496,7 @@ let (head_tactic_patt: pbp_rule) = function
 				   false "dummy" nprems))]))
     | (str_list, int_list, nprems, c, []) 
         when  (check_apply c (mk_db_indices int_list nprems)) &
-              (match c with IsProd(_,_,_) -> false
+              (match c with Prod(_,_,_) -> false
               |  _ -> true) &
               (List.length int_list) + nprems > 0 ->
           Some(add_clear_names_if_necessary
@@ -543,6 +548,9 @@ let default_ast optname constr path =
                    [Node(zz, "Exact",[Node(zz,"COMMAND",[Nvar(zz,a)])])]);;
 
 (* This is the main proof by pointing function. *)
+(* avoid: les noms a ne pas utiliser *)
+(* final_cmd: la fonction appelee par defaut *)
+(* opt_name: eventuellement le nom de l'hypothese sur laquelle on agit *)
 
 let rec pbpt final_cmd avoid clear_names clear_flag opt_name constr path =
   let rec try_all_rules rl =
@@ -644,7 +652,7 @@ let cleanup_clears empty_allowed names str_list other =
 		[]
 	      else [Node(zz,"Idtac",[])]
           | _ -> other)
-  | l -> Node(zz, "Clear", [Node(zz,"CLAUSE", l)])::other;;
+  | l -> Node(zz, "TRY", [Node(zz, "Clear", [Node(zz,"CLAUSE", l)])])::other;;
 
 
 (* This function takes care of compacting instanciations of universal
@@ -660,7 +668,7 @@ let rec optim3 str_list = function
 	     | Some s -> optim3_aux true (s::str_list)
                             (Node(a, "Generalize",
                                 [merge_ast_in_command com1 com2])::others))
-	|( Node(a,"Clear", [Node(_,"CLAUSE", names)]))::other ->
+	|( Node(zz, "TRY", [Node(a,"Clear", [Node(_,"CLAUSE", names)])]))::other ->
 	    cleanup_clears empty_allowed names str_list other
 	| [Node(a,"TACLIST",branches)] ->
 	    [Node(a,"TACLIST",List.map (optim3 str_list) branches)]
@@ -685,9 +693,16 @@ let pbp_tac display_function = function
             (Identifier a)::l -> 
                  (function g ->
                     let str = (string_of_id a) in
+		    let (ou,tstr) = (get_hyp_by_name g str) in
 		    let exp_ast =
-		      pbpt default_ast  (pf_ids_of_hyps g)
-                        [] false (Some str) (kind_of_term (get_hyp_by_name g str))
+		      pbpt default_ast
+		        (match ou with
+			       "hyp" ->(pf_ids_of_hyps g)
+			       |_ -> (a::(pf_ids_of_hyps g)))
+                        []
+			false
+			(Some str)
+			(kind_of_term tstr)
                         (tactic_args_to_ints l) in
                     (display_function (optim exp_ast);
                         tclIDTAC g))
@@ -703,3 +718,5 @@ let pbp_tac display_function = function
                      (display_function (default_ast None (pf_concl g) []);
                       tclIDTAC g))
           |  _ -> failwith "expecting other arguments";;
+
+
