@@ -13,7 +13,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: coq_omega.ml,v 1.33 2001/12/13 09:03:03 filliatr Exp $ *)
+(* $Id: coq_omega.ml,v 1.36 2002/04/11 13:25:07 herbelin Exp $ *)
 
 open Util
 open Pp
@@ -70,91 +70,12 @@ let new_identifier_var =
 let rec mk_then = 
   function [t] -> t | (t::l) -> (tclTHEN (t) (mk_then l)) | [] -> tclIDTAC
 
-let collect_com lbind = 
-  map_succeed (function (Com,c)->c | _ -> failwith "Com") lbind
-
-(*** EST-CE QUE CES FONCTIONS NE SONT PAS AILLEURS DANS LE CODE ?? *)
-
-let make_clenv_binding_apply wc (c,t) lbind = 
-  let largs = collect_com lbind in 
-  let lcomargs = List.length largs in 
-  if lcomargs = List.length lbind then 
-    let clause = mk_clenv_from wc (c,t) in
-    clenv_constrain_missing_args largs clause
-  else if lcomargs = 0 then 
-    let clause = mk_clenv_rename_from wc (c,t) in
-    clenv_match_args lbind clause
-  else 
-    errorlabstrm "make_clenv_bindings"
-      (str "Cannot mix bindings and free associations")
-
-let resolve_with_bindings_tac  (c,lbind) gl = 
-  let (wc,kONT) = startWalk gl in
-  let t = w_hnf_constr wc (w_type_of wc c) in 
-  let clause = make_clenv_binding_apply wc (c,t) lbind in 
-  res_pf kONT clause gl
-
-(*
-let pf_one_step_reduce = pf_reduce Tacred.one_step_reduce
-
-let reduce_to_mind gl t = 
-  let rec elimrec t l =
-    let c, args = whd_stack t in
-    match kind_of_term c, args with
-    | (Ind ind,_) -> (ind,Environ.it_mkProd_or_LetIn t l)
-    | (Const _,_) -> 
-	(try 
-	   let t' = pf_nf_betaiota gl (pf_one_step_reduce gl t) in elimrec t' l
-         with e when catchable_exception e -> 
-	   errorlabstrm "tactics__reduce_to_mind"
-             (str"Not an inductive product"))
-    | (Case _,_) ->
-	(try 
-	   let t' = pf_nf_betaiota gl (pf_one_step_reduce gl t) in elimrec t' l
-         with e when catchable_exception e -> 
-	   errorlabstrm "tactics__reduce_to_mind"
-             (str"Not an inductive product"))
-    | (Cast (c,_),[]) -> elimrec c l
-    | (Prod (n,ty,t'),[]) -> 
-	let ty' = Retyping.get_assumption_of (Global.env()) Evd.empty ty in
-	elimrec t' ((n,None,ty')::l)
-    | (LetIn (n,b,ty,t'),[]) ->
-	let ty' = Retyping.get_assumption_of (Global.env()) Evd.empty ty in
-	elimrec t' ((n,Some b,ty')::l)
-    | _ -> error "Not an inductive product"
-  in 
-  elimrec t []
-*)
-
-let reduce_to_mind = pf_reduce_to_quantified_ind
-
-let constructor_tac nconstropt i lbind gl = 
-  let cl = pf_concl gl in
-  let (mind, redcl) = reduce_to_mind gl cl in
-  let (mib,mip) = Global.lookup_inductive mind in
-  let nconstr = Array.length mip.mind_consnames
-  and sigma = project gl in
-  (match nconstropt with 
-     | Some expnconstr -> 
-	 if expnconstr <> nconstr then
-           error "Not the expected number of constructors"
-     | _ -> ());
-  if i > nconstr then error "Not enough Constructors";
-  let c = mkConstruct (ith_constructor_of_inductive mind i) in 
-  let resolve_tac = resolve_with_bindings_tac (c,lbind) in
-  (tclTHEN (tclTHEN (change_in_concl redcl) intros) resolve_tac) gl
-
-let exists_tac c= constructor_tac (Some 1) 1 [Com,c]
-		    
+let exists_tac c = constructor_tac (Some 1) 1 [Com,c]
 let generalize_tac t = generalize_time (generalize t)
 let elim t = elim_time (simplest_elim t)
 let exact t = exact_time (Tactics.refine t)
-
 let unfold s = Tactics.unfold_in_concl [[], Lazy.force s]
  
-(*** fin de EST-CE QUE CES FONCTIONS NE SONT PAS AILLEURS DANS LE CODE ?? *)
-(****************************************************************)
-
 let rev_assoc k =
   let rec loop = function
     | [] -> raise Not_found | (v,k')::_ when k = k' -> v | _ :: l -> loop l 
@@ -1161,7 +1082,7 @@ let replay_history tactic_normalisation =
 	  let kk = mk_integer k in
 	  let state_eq = mk_eq eq1 (mk_times eq2 kk) in
 	  if e1.kind = DISE then
-            let tac = scalar_norm [P_APP 1] e2.body in
+            let tac = scalar_norm [P_APP 3] e2.body in
             tclTHENS 
 	      (cut state_eq) 
 	      [tclTHEN 
@@ -1681,13 +1602,14 @@ let rec decidability gl t =
 let destructure_hyps gl =
   let rec loop evbd = function
     | [] -> (tclTHEN nat_inject coq_omega)
-    | (i,t)::lit ->
+    | (i,body,t)::lit ->
 	begin try match destructurate t with
           | Kapp(("Zle"|"Zge"|"Zgt"|"Zlt"|"Zne"),[t1;t2]) -> loop evbd lit
           | Kapp("or",[t1;t2]) ->
               (tclTHENS ((tclTHEN ((tclTHEN (elim_id i) (clear [i]))) 
 			    (intros_using [i]))) 
-		 ([ loop evbd ((i,t1)::lit); loop evbd ((i,t2)::lit) ]))
+		 ([ loop evbd ((i,None,t1)::lit);
+                    loop evbd ((i,None,t2)::lit) ]))
           | Kapp("and",[t1;t2]) ->
               let i1 = id_of_string (string_of_id i ^ "_left") in
               let i2 = id_of_string (string_of_id i ^ "_right") in
@@ -1695,7 +1617,7 @@ let destructure_hyps gl =
 		 (tclTHEN 
 		    (tclTHEN (elim_id i) (clear [i]))
 		    (intros_using [i1;i2]))
-		 (loop (i1::i2::evbd) ((i1,t1)::(i2,t2)::lit)))
+		 (loop (i1::i2::evbd) ((i1,None,t1)::(i2,None,t2)::lit)))
           | Kimp(t1,t2) ->
               if isprop (pf_type_of gl t1) & closed0 t2 then begin
 		(tclTHEN 
@@ -1707,7 +1629,7 @@ let destructure_hyps gl =
 						    mkVar i|])]) 
 			 (clear [i])) 
 		      (intros_using [i])) 
-		   (loop evbd ((i,mk_or (mk_not t1) t2)::lit)))
+		   (loop evbd ((i,None,mk_or (mk_not t1) t2)::lit)))
               end else loop evbd lit
           | Kapp("not",[t]) -> 
               begin match destructurate t with 
@@ -1719,7 +1641,7 @@ let destructure_hyps gl =
 							t1; t2; mkVar i |])])
 			     (clear [i])) 
 			  (intros_using [i])) 
-                       (loop evbd ((i,mk_and (mk_not t1) (mk_not t2)):: lit)))
+                       (loop evbd ((i,None,mk_and (mk_not t1) (mk_not t2)):: lit)))
 		| Kapp("and",[t1;t2]) -> 
                     (tclTHEN 
 		       (tclTHEN 
@@ -1729,7 +1651,7 @@ let destructure_hyps gl =
 					   decidability gl t1;mkVar i|])])
 			     (clear [i]))
 			  (intros_using [i]))
-                       (loop evbd ((i,mk_or (mk_not t1) (mk_not t2))::lit)))
+                       (loop evbd ((i,None,mk_or (mk_not t1) (mk_not t2))::lit)))
 		| Kimp(t1,t2) ->
                     (tclTHEN 
 		       (tclTHEN 
@@ -1739,7 +1661,7 @@ let destructure_hyps gl =
 					   decidability gl t1;mkVar i |])])
 			     (clear [i])) 
 			  (intros_using [i]))
-                       (loop evbd ((i,mk_and t1 (mk_not t2)) :: lit)))
+                       (loop evbd ((i,None,mk_and t1 (mk_not t2)) :: lit)))
 		| Kapp("not",[t]) ->
                     (tclTHEN 
 		       (tclTHEN 
@@ -1749,7 +1671,7 @@ let destructure_hyps gl =
 					    decidability gl t; mkVar i |])])
 			     (clear [i])) 
 			  (intros_using [i])) 
-                       (loop evbd ((i,t)::lit)))
+                       (loop evbd ((i,None,t)::lit)))
 		| Kapp("Zle", [t1;t2]) ->
                     (tclTHEN 
 		       (tclTHEN 
@@ -1852,13 +1774,15 @@ let destructure_hyps gl =
 		      match destructurate (pf_nf gl typ) with
 			| Kapp("nat",_) -> 
                             (tclTHEN 
-			       (convert_hyp i 
-				  (mkApp (Lazy.force coq_neq, [| t1;t2|])))
+			       (convert_hyp
+                                  (i,body,
+				  (mkApp (Lazy.force coq_neq, [| t1;t2|]))))
 			       (loop evbd lit))
 			| Kapp("Z",_) ->
                             (tclTHEN 
-			       (convert_hyp i 
-				  (mkApp (Lazy.force coq_Zne, [| t1;t2|])))
+			       (convert_hyp
+                                  (i,body,
+				  (mkApp (Lazy.force coq_Zne, [| t1;t2|]))))
 			       (loop evbd lit))
 			| _ -> loop evbd lit
                     end
@@ -1868,7 +1792,7 @@ let destructure_hyps gl =
 	with e when catchable_exception e -> loop evbd lit
 	end 
   in
-  loop (pf_ids_of_hyps gl) (pf_hyps_types gl) gl
+  loop (pf_ids_of_hyps gl) (pf_hyps gl) gl
 
 let destructure_goal gl =
   let concl = pf_concl gl in
