@@ -13,7 +13,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: omega.ml,v 1.3 2001/03/15 13:38:19 filliatr Exp $ *) 
+(* $Id: omega.ml,v 1.6 2002/01/25 10:28:59 filliatr Exp $ *) 
 
 open Util
 open Hashtbl
@@ -279,7 +279,7 @@ let normalize ({id=id; kind=eq_flag; body=e; constant =x} as eq) =
       let d = x - c * gcd in
       let new_eq = {id=id; kind=eq_flag; constant=c; 
                     body=map_eq_linear (fun c -> c / gcd) e} in
-      add_event (if eq_flag=EQUA then EXACT_DIVIDE(eq,gcd)
+      add_event (if eq_flag=EQUA or eq_flag = DISE then EXACT_DIVIDE(eq,gcd)
                  else DIVIDE_AND_APPROX(eq,new_eq,gcd,d));
       [new_eq]
     end else [eq]
@@ -406,7 +406,7 @@ let redundancy_elimination system =
   let accu_ineq = ref [] in
   iter
     (fun p0 p1 -> match (p0,p1) with 
-       | (e, (Some x, Some y)) when x.constant = -y.constant -> 
+       | (e, (Some x, Some y)) when x.constant = y.constant ->
            let id=new_id () in
            add_event (MERGE_EQ(id,x,y.id));
            push {id=id; kind=EQUA; body=x.body; constant=x.constant} accu_eq
