@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: quote.ml,v 1.23 2001/11/05 16:47:12 barras Exp $ *)
+(* $Id: quote.ml,v 1.24 2002/03/04 13:29:33 barras Exp $ *)
 
 (* The `Quote' tactic *)
 
@@ -268,7 +268,7 @@ let compute_ivs gl f cs =
 
 	      (* The Cases predicate is a lambda; we assume no dependency *)
 	      let p = match kind_of_term p with
-		| Lambda (_,_,p) -> pop p
+		| Lambda (_,_,p) -> Termops.pop p
 		| _ -> p
 	      in
               
