@@ -6,14 +6,13 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: closure.ml,v 1.45 2001/12/13 09:03:03 filliatr Exp $ *)
+(* $Id: closure.ml,v 1.47 2002/03/12 10:18:06 courant Exp $ *)
 
 open Util
 open Pp
 open Term
 open Names
 open Environ
-open Univ
 open Esubst
 
 
@@ -54,7 +53,7 @@ let with_stats c =
 
 type evaluable_global_reference =
   | EvalVarRef of identifier
-  | EvalConstRef of section_path
+  | EvalConstRef of constant
 
 type transparent_state = Idpred.t * Sppred.t
 
@@ -68,7 +67,7 @@ module type RedFlagsSig = sig
   val fDELTA : red_kind
   val fIOTA : red_kind
   val fZETA : red_kind
-  val fCONST : section_path -> red_kind
+  val fCONST : constant -> red_kind
   val fVAR : identifier -> red_kind
   val no_red : reds
   val red_add : reds -> red_kind -> reds
