@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: closure.mli,v 1.36 2001/11/29 09:21:08 barras Exp $ i*)
+(*i $Id: closure.mli,v 1.37 2002/01/30 13:47:58 herbelin Exp $ i*)
 
 (*i*)
 open Pp
@@ -50,7 +50,7 @@ module type RedFlagsSig = sig
   val fDELTA : red_kind
   val fIOTA : red_kind
   val fZETA : red_kind
-  val fCONST : section_path -> red_kind
+  val fCONST : constant -> red_kind
   val fVAR : identifier -> red_kind
 
   (* No reduction at all *)
