@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: inductive.ml,v 1.60 2001/12/20 08:24:41 herbelin Exp $ *)
+(* $Id: inductive.ml,v 1.68 2002/04/03 12:10:05 barras Exp $ *)
 
 open Util
 open Names
@@ -18,69 +18,45 @@ open Environ
 open Reduction
 open Type_errors
 
-exception Induc
-
-(* raise Induc if not an inductive type *)
+(* raise Not_found if not an inductive type *)
 let lookup_mind_specif env (sp,tyi) =
-  let mib =
-    try Environ.lookup_mind sp env
-    with Not_found -> raise Induc in
+  let mib = Environ.lookup_mind sp env in
   if tyi >= Array.length mib.mind_packets then
     error "Inductive.lookup_mind_specif: invalid inductive index";
   (mib, mib.mind_packets.(tyi))
 
-let lookup_recargs env ind =
-  let (mib,mip) = lookup_mind_specif env ind in
-  Array.map (fun mip -> mip.mind_listrec) mib.mind_packets
-
-let lookup_subterms env (_,i as ind) =
-  (lookup_recargs env ind).(i)
-
 let find_rectype env c =
   let (t, l) = decompose_app (whd_betadeltaiota env c) in
   match kind_of_term t with
     | Ind ind -> (ind, l)
-    | _ -> raise Induc
+    | _ -> raise Not_found
 
 let find_inductive env c =
   let (t, l) = decompose_app (whd_betadeltaiota env c) in
   match kind_of_term t with
     | Ind ind
         when (fst (lookup_mind_specif env ind)).mind_finite -> (ind, l)
-    | _ -> raise Induc
+    | _ -> raise Not_found
 
 let find_coinductive env c =
   let (t, l) = decompose_app (whd_betadeltaiota env c) in
   match kind_of_term t with
     | Ind ind
         when not (fst (lookup_mind_specif env ind)).mind_finite -> (ind, l)
-    | _ -> raise Induc
+    | _ -> raise Not_found
 
 (***********************************************************************)
 
-type inductive_instance = {
-  mis_sp : section_path;
-  mis_mib : mutual_inductive_body;
-  mis_tyi : int;
-  mis_mip : one_inductive_body }
-
-let mis_nconstr mis = Array.length (mis.mis_mip.mind_consnames)
-let mis_inductive mis = (mis.mis_sp,mis.mis_tyi)
-
-let lookup_mind_instance (sp,tyi) env =
-  let (mib,mip) = lookup_mind_specif env (sp,tyi) in
-  { mis_sp = sp; mis_mib = mib; mis_tyi = tyi; mis_mip = mip }
-
 (* Build the substitution that replaces Rels by the appropriate *)
 (* inductives *)
-let ind_subst mispec =
-  let ntypes = mispec.mis_mib.mind_ntypes in
-  let make_Ik k = mkInd (mispec.mis_sp,ntypes-k-1) in 
+let ind_subst mind mib =
+  let ntypes = mib.mind_ntypes in
+  let make_Ik k = mkInd (mind,ntypes-k-1) in 
   list_tabulate make_Ik ntypes
 
-(* Instantiate both section variables and inductives *)
-let constructor_instantiate mispec c =
-  let s = ind_subst mispec in
+(* Instantiate inductives in constructor type *)
+let constructor_instantiate mind mib c =
+  let s = ind_subst mind mib in
   type_app (substl s) c
 
 (* Instantiate the parameters of the inductive type *)
@@ -102,13 +78,13 @@ let instantiate_params t args sign =
   if rem_args <> [] then fail();
   type_app (substl subs) ty
 
-let full_inductive_instantiate (mispec,params) t =
-  instantiate_params t params mispec.mis_mip.mind_params_ctxt
+let full_inductive_instantiate mip params t =
+  instantiate_params t params mip.mind_params_ctxt
 
-let full_constructor_instantiate (mispec,params) =
-  let inst_ind = constructor_instantiate mispec in
+let full_constructor_instantiate (((mind,_),mib,mip),params) =
+  let inst_ind = constructor_instantiate mind mib in
   (fun t ->
-    instantiate_params (inst_ind t) params mispec.mis_mip.mind_params_ctxt)
+    instantiate_params (inst_ind t) params mip.mind_params_ctxt)
 
 (***********************************************************************)
 (***********************************************************************)
@@ -118,107 +94,30 @@ let full_constructor_instantiate (mispec,params) =
 (* Type of an inductive type *)
 
 let type_of_inductive env i =
-  let mis = lookup_mind_instance i env in
-  let hyps = mis.mis_mib.mind_hyps in
-  mis.mis_mip.mind_user_arity
-
-(* The same, with parameters instantiated *)
-let get_arity (mispec,params as indf) =
-  let arity = mispec.mis_mip.mind_nf_arity in
-  destArity (full_inductive_instantiate indf arity)
+  let (_,mip) = lookup_mind_specif env i in
+  mip.mind_user_arity
 
 (***********************************************************************)
 (* Type of a constructor *)
 
 let type_of_constructor env cstr =
   let ind = inductive_of_constructor cstr in
-  let mispec = lookup_mind_instance ind env in
-  let specif = mispec.mis_mip.mind_user_lc in
+  let (mib,mip) = lookup_mind_specif env ind in
+  let specif = mip.mind_user_lc in
   let i = index_of_constructor cstr in
-  let nconstr = mis_nconstr mispec in
+  let nconstr = Array.length mip.mind_consnames in
   if i > nconstr then error "Not enough constructors in the type";
-  constructor_instantiate mispec specif.(i-1)
+  constructor_instantiate (fst ind) mib specif.(i-1)
 
 let arities_of_constructors env ind = 
-  let mispec = lookup_mind_instance ind env in
-  let specif = mispec.mis_mip.mind_user_lc in
-  Array.map (constructor_instantiate mispec) specif
+  let (mib,mip) = lookup_mind_specif env ind in
+  let specif = mip.mind_nf_lc in
+  Array.map (constructor_instantiate (fst ind) mib) specif
 
 
 
-(* gives the vector of constructors and of 
-   types of constructors of an inductive definition 
-   correctly instanciated *)
-
-let mis_nf_constructor_type i mispec =
-  let nconstr = mis_nconstr mispec in
-  if i > nconstr then error "Not enough constructors in the type";
-  constructor_instantiate mispec mispec.mis_mip.mind_nf_lc.(i-1)
-   
-(*s Useful functions *)
-
-type constructor_summary = {
-  cs_cstr : constructor;
-  cs_params : constr list;
-  cs_nargs : int;
-  cs_args : rel_context;
-  cs_concl_realargs : constr array
-}
-
-let process_constructor ((mispec,params) as indf) j typi =
-  let typi = full_constructor_instantiate indf typi in
-  let (args,ccl) = decompose_prod_assum typi in
-  let (_,allargs) = decompose_app ccl in
-  let (_,vargs) = list_chop mispec.mis_mip.mind_nparams allargs in
-  { cs_cstr = ith_constructor_of_inductive (mis_inductive mispec) (j+1);
-    cs_params = params;
-    cs_nargs = rel_context_length args;
-    cs_args = args;
-    cs_concl_realargs = Array.of_list vargs }
-
-let get_constructors ((mispec,params) as indf) =
-  let constr_tys = mispec.mis_mip.mind_nf_lc in
-  Array.mapi (process_constructor indf) constr_tys
-
 (***********************************************************************)
 
-(* Type of case branches *)
-
-let local_rels ctxt =
-  let (rels,_) =
-    Sign.fold_rel_context_reverse
-      (fun (rels,n) (_,copt,_) ->
-        match copt with
-            None   -> (mkRel n :: rels, n+1)
-          | Some _ -> (rels, n+1))
-      ~init:([],1)
-      ctxt 
-  in
-  rels
-
-let build_dependent_constructor cs =
-  applist
-    (mkConstruct cs.cs_cstr,
-     (List.map (lift cs.cs_nargs) cs.cs_params)@(local_rels cs.cs_args))
-
-let build_dependent_inductive ((mis, params) as indf) =
-  let arsign,_ = get_arity indf in
-  let nrealargs = mis.mis_mip.mind_nrealargs in
-  applist 
-    (mkInd (mis_inductive mis),
-     (List.map (lift nrealargs) params)@(local_rels arsign))
-
-(* [p] is the predicate and [cs] a constructor summary *)
-let build_branch_type dep p cs =
-  let args =
-    if dep then
-      Array.append cs.cs_concl_realargs [|build_dependent_constructor cs|]
-    else
-      cs.cs_concl_realargs in
-  let base = beta_appvect (lift cs.cs_nargs p) args in
-  it_mkProd_or_LetIn base cs.cs_args
-
-
 let is_info_arity env c =
   match dest_arity env c with
     | (_,Prop Null) -> false 
@@ -233,45 +132,74 @@ let error_elim_expln env kp ki =
       | Sort (Type _), Sort (Prop _) -> StrongEliminationOnNonSmallType
       | _ -> WrongArity
 
-exception Arity of (constr * constr * arity_error) option
+(* Type of case predicates *)
+
+let local_rels ctxt =
+  let (rels,_) =
+    Sign.fold_rel_context_reverse
+      (fun (rels,n) (_,copt,_) ->
+        match copt with
+            None   -> (mkRel n :: rels, n+1)
+          | Some _ -> (rels, n+1))
+      ~init:([],1)
+      ctxt 
+  in
+  rels
+
+(* Get type of inductive, with parameters instantiated *)
+let get_arity mip params =
+  let arity = mip.mind_nf_arity in
+  destArity (full_inductive_instantiate mip params arity)
+
+let build_dependent_inductive ind mip params =
+  let arsign,_ = get_arity mip params in
+  let nrealargs = mip.mind_nrealargs in
+  applist 
+    (mkInd ind, (List.map (lift nrealargs) params)@(local_rels arsign))
 
 
-let is_correct_arity env kelim (c,pj) indf t = 
-  let rec srec (pt,t) u =
+(* This exception is local *)
+exception LocalArity of (constr * constr * arity_error) option
+
+let is_correct_arity env c pj ind mip params = 
+  let kelim = mip.mind_kelim in
+  let arsign,s = get_arity mip params in
+  let nodep_ar = it_mkProd_or_LetIn (mkSort s) arsign in
+  let rec srec pt t u =
     let pt' = whd_betadeltaiota env pt in
     let t' = whd_betadeltaiota env t in
     match kind_of_term pt', kind_of_term t' with
       | Prod (_,a1,a2), Prod (_,a1',a2') ->
           let univ =
             try conv env a1 a1'
-            with NotConvertible -> raise (Arity None) in
-          srec (a2,a2') (Constraint.union u univ)
+            with NotConvertible -> raise (LocalArity None) in
+          srec a2 a2' (Constraint.union u univ)
       | Prod (_,a1,a2), _ -> 
           let k = whd_betadeltaiota env a2 in 
           let ksort = match kind_of_term k with
             | Sort s -> family_of_sort s 
-	    | _ -> raise (Arity None) in
-	  let ind = build_dependent_inductive indf in
+	    | _ -> raise (LocalArity None) in
+	  let dep_ind = build_dependent_inductive ind mip params in
           let univ =
-            try conv env a1 ind
-            with NotConvertible -> raise (Arity None) in
+            try conv env a1 dep_ind
+            with NotConvertible -> raise (LocalArity None) in
           if List.exists ((=) ksort) kelim then
-	    ((true,k), Constraint.union u univ)
+	    (true, Constraint.union u univ)
           else 
-	    raise (Arity (Some(k,t',error_elim_expln env k t')))
+	    raise (LocalArity (Some(k,t',error_elim_expln env k t')))
       | k, Prod (_,_,_) ->
-	  raise (Arity None)
+	  raise (LocalArity None)
       | k, ki -> 
 	  let ksort = match k with
             | Sort s -> family_of_sort s 
-            | _ ->  raise (Arity None) in
+            | _ ->  raise (LocalArity None) in
           if List.exists ((=) ksort) kelim then 
-	    (false, pt'), u
+	    (false, u)
           else 
-	    raise (Arity (Some(pt',t',error_elim_expln env pt' t')))
+	    raise (LocalArity (Some(pt',t',error_elim_expln env pt' t')))
   in 
-  try srec (pj.uj_type,t) Constraint.empty
-  with Arity kinds ->
+  try srec pj.uj_type nodep_ar Constraint.empty
+  with LocalArity kinds ->
     let create_sort = function 
       | InProp -> mkProp
       | InSet -> mkSet
@@ -281,32 +209,51 @@ let is_correct_arity env kelim (c,pj) indf t =
       (List.map (fun s -> make_arity env true indf (create_sort s)) kelim)
       @(List.map (fun s -> make_arity env false indf (create_sort s)) kelim)*)
     in 
-    let ind = mis_inductive (fst indf) in
     error_elim_arity env ind listarity c pj kinds
 
 
-let find_case_dep_nparams env (c,pj) (ind,params) =
-  let indf = lookup_mind_instance ind env in
-  let kelim = indf.mis_mip.mind_kelim in
-  let arsign,s = get_arity (indf,params) in
-  let glob_t = it_mkProd_or_LetIn (mkSort s) arsign in
-  let ((dep,_),univ) =
-    is_correct_arity env kelim (c,pj) (indf,params) glob_t in
-  (dep,univ)
+(***********************************************************************)
+(* Type of case branches *)
 
+(* [p] is the predicate, [i] is the constructor number (starting from 0),
+   and [cty] is the type of the constructor (params not instantiated) *)
+let build_branches_type ind mib mip params dep p =
+  let build_one_branch i cty =
+    let typi = full_constructor_instantiate ((ind,mib,mip),params) cty in
+    let (args,ccl) = decompose_prod_assum typi in
+    let nargs = rel_context_length args in
+    let (_,allargs) = decompose_app ccl in
+    let (lparams,vargs) = list_chop mip.mind_nparams allargs in
+    let cargs =
+      if dep then
+        let cstr = ith_constructor_of_inductive ind (i+1) in
+        let dep_cstr = applist (mkConstruct cstr,lparams@(local_rels args)) in
+        vargs @ [dep_cstr]
+      else
+        vargs in
+    let base = beta_appvect (lift nargs p) (Array.of_list cargs) in
+    it_mkProd_or_LetIn base args in
+  Array.mapi build_one_branch mip.mind_nf_lc
 
-let type_case_branches env (mind,largs) pj c =
-  let mispec = lookup_mind_instance mind env in 
-  let nparams = mispec.mis_mip.mind_nparams in
-  let (params,realargs) = list_chop nparams largs in
-  let indf = (mispec,params) in
-  let p = pj.uj_val in
-  let (dep,univ) = find_case_dep_nparams env (c,pj) (mind,params) in
-  let constructs = get_constructors indf in
-  let lc = Array.map (build_branch_type dep p) constructs in
+(* [p] is the predicate, [c] is the match object, [realargs] is the
+   list of real args of the inductive type *)
+let build_case_type dep p c realargs =
   let args = if dep then realargs@[c] else realargs in
-  (lc, beta_appvect p (Array.of_list args), univ)
+  beta_appvect p (Array.of_list args)
 
+let type_case_branches env (ind,largs) pj c =
+  let (mib,mip) = lookup_mind_specif env ind in 
+  let nparams = mip.mind_nparams in
+  let (params,realargs) = list_chop nparams largs in
+  let p = pj.uj_val in
+  let (dep,univ) = is_correct_arity env c pj ind mip params in
+  let lc = build_branches_type ind mib mip params dep p in
+  let ty = build_case_type dep p c realargs in
+  (lc, ty, univ)
+
+
+(***********************************************************************)
+(* Checking the case annotation is relevent *)
 
 let check_case_info env indsp ci =
   let (mib,mip) = lookup_mind_specif env indsp in
@@ -351,6 +298,33 @@ exception FixGuardError of guard_error
    Below are functions to handle such environment.
  *)
 type size = Large | Strict
+
+let size_glb s1 s2 =
+  match s1,s2 with
+      Strict, Strict -> Strict
+    | _ -> Large
+
+type subterm_spec =
+    Subterm of (size * wf_paths)
+  | Dead_code
+  | Not_subterm
+
+let spec_of_tree t =
+  if t=mk_norec then Not_subterm else Subterm(Strict,t)
+
+let subterm_spec_glb =
+  let glb2 s1 s2 =
+    match s1,s2 with
+        _, Dead_code -> s1
+      | Dead_code, _ -> s2
+      | Not_subterm, _ -> Not_subterm
+      | _, Not_subterm -> Not_subterm
+      | Subterm (a1,t1), Subterm (a2,t2) ->
+          if t1=t2 then Subterm (size_glb a1 a2, t1)
+          (* branches do not return objects with same spec *)
+          else Not_subterm in
+  Array.fold_left glb2 Dead_code
+          
 type guard_env =
   { env     : env;
     (* dB of last fixpoint *)
@@ -358,74 +332,55 @@ type guard_env =
     (* inductive of recarg of each fixpoint *)
     inds    : inductive array;
     (* the recarg information of inductive family *)
-    recvec  : recarg list array array;
+    recvec  : wf_paths array;
     (* dB of variables denoting subterms *)
-    lst     : (int * (size * recarg list array)) list;
+    genv    : subterm_spec list;
   }
 
-let make_renv env minds recarg (_,tyi as ind) =
-  let mind_recvec = lookup_recargs env ind in 
-  let lcx = mind_recvec.(tyi)  in
+let make_renv env minds recarg (sp,tyi) =
+  let mib = Environ.lookup_mind sp env in
+  let mind_recvec =
+    Array.map (fun mip -> mip.mind_recargs) mib.mind_packets in
   { env = env;
     rel_min = recarg+2;
     inds = minds;
     recvec = mind_recvec;
-    lst = [(1,(Large,mind_recvec.(tyi)))] }
+    genv = [Subterm(Large,mind_recvec.(tyi))] }
 
-let map_lift_fst_n m = List.map (function (n,t)->(n+m,t))
-
-let push_var_renv renv (x,ty) =
+let push_var renv (x,ty,spec) =
   { renv with 
     env = push_rel (x,None,ty) renv.env;
     rel_min = renv.rel_min+1;
-    lst = map_lift_fst_n 1 renv.lst }
+    genv = spec:: renv.genv }
+
+let assign_var_spec renv (i,spec) =
+  { renv with genv = list_assign renv.genv (i-1) spec }
+
+let push_var_renv renv (x,ty) =
+  push_var renv (x,ty,Not_subterm)
+
+(* Fetch recursive information about a variable p *)
+let subterm_var p renv = 
+  try List.nth renv.genv (p-1)
+  with Failure _ | Invalid_argument _ -> Not_subterm
+
+(* Add a variable and mark it as strictly smaller with information [spec]. *)
+let add_subterm renv (x,a,spec) =
+  push_var renv (x,a,spec_of_tree spec)
 
 let push_ctxt_renv renv ctxt =
   let n = rel_context_length ctxt in
   { renv with 
     env = push_rel_context ctxt renv.env;
     rel_min = renv.rel_min+n;
-    lst = map_lift_fst_n n renv.lst }
+    genv = iterate (fun ge -> Not_subterm::ge) n renv.genv }
 
 let push_fix_renv renv (_,v,_ as recdef) =
   let n = Array.length v in
   { renv with
     env = push_rec_types recdef renv.env;
     rel_min = renv.rel_min+n;
-    lst = map_lift_fst_n n renv.lst }
-
-(* Add a variable and mark it as strictly smaller with information [spec]. *)
-let add_recarg renv (x,a,spec) =
-  let renv' = push_var_renv renv (x,a) in
-  { renv' with lst = (1,(Strict,spec))::renv'.lst }
-
-
-let rec findrec p = function 
-  | (a,ta)::l -> 
-      if a < p then findrec p l else if a = p then ta else raise Not_found
-  | _ -> raise Not_found
-
-(* tells if term [c] is the variable corresponding to the recursive
-   argument of the fixpoint. *)
-(* c is supposed to be in beta-delta-iota head normal form *)
-let subterm_var_large renv c = 
-  match kind_of_term (fst (decompose_app c)) with 
-    | Rel n ->
-        (try
-          match findrec n renv.lst with
-            (Large,s) -> Some s
-          | _ -> None
-        with Not_found -> None)
-    | _ -> None
-
-(* fetch the information associated to a variable *)
-let subterm_var_spec p renv = 
-  try 
-    match findrec p renv.lst with
-      (Strict,s) -> Some s
-    | _ -> None
-  with Not_found -> None
-
+    genv = iterate (fun ge -> Not_subterm::ge) n renv.genv }
 
 
 (******************************)
@@ -437,42 +392,15 @@ let subterm_var_spec p renv =
    lrec.  mind_recvec is the recursive spec of the inductive
    definition of the decreasing argument n.
 
-   case_branches_specif renv mind_recvec lrec lc will pass the lambdas
+   case_branches_specif renv lrec lc will pass the lambdas
    of c corresponding to pattern variables and collect possibly new
    subterms variables and returns the bodies of the branches with the
    correct envs and decreasing args.
 *)
 
-
-let imbr_recarg_expand env (sp,i as ind_sp) lrc =
-  let recargs = lookup_subterms env ind_sp in
-  let rec imbr_recarg ra = 
-    match ra with 
-      | Mrec(j)        -> Imbr((sp,j),lrc)
-      | Imbr(ind_sp,l) -> Imbr(ind_sp, List.map imbr_recarg l)
-      | Norec          -> Norec
-      | Param(k)       -> List.nth lrc k in
-  Array.map (List.map imbr_recarg) recargs
-
-
-let case_branches_specif renv = 
-  let rec crec renv lrec c = 
-    let c' = strip_outer_cast c in 
-    match lrec, kind_of_term c' with 
-      |	(ra::lr,Lambda (x,a,b)) ->
-          let renv' =
-            match ra with 
-	        Mrec(i) -> add_recarg renv (x,a,renv.recvec.(i))
-              | Imbr(ind_sp,lrc) ->
-                  let lc = imbr_recarg_expand renv.env ind_sp lrc in
-                  add_recarg renv (x,a,lc)
-	      | _ -> push_var_renv renv (x,a) in
-          crec renv' lr b
-      | (_,LetIn (x,c,a,b)) -> crec renv lrec (subst1 c b)
-      (* Rq: if branch is not eta-long, then the recursive information
-         is not propagated: *)
-      | (_,_) -> (renv,c')
-  in array_map2 (crec renv) 
+let lookup_subterms env ind =
+  let (_,mip) = lookup_mind_specif env ind in
+  mip.mind_recargs
 
 (*********************************)
 
@@ -499,93 +427,87 @@ let inductive_of_fix env recarg body =
    - [Some lc] if [c] is a strict subterm of the rec. arg. (or a Meta) 
    - [None] otherwise
 *)
-let subterm_specif renv c ind = 
-  let rec crec renv c ind = 
-    let f,l = decompose_app (whd_betadeltaiota renv.env c) in
-    match kind_of_term f with 
-      | Rel k -> subterm_var_spec k renv
 
-      | Case (ci,_,c,lbr) ->
-          if Array.length lbr = 0
-            (* New: if it is built by contadiction, it is OK *)
-          then Some [||]
-          else
-            let def = Array.create (Array.length lbr) [] in
-            let lcv =
-              match subterm_var_large renv c with
-                Some s -> s
-              | _ ->
-                  (match crec renv c ci.ci_ind with
-                    Some lr -> lr
-                  | None -> def) in 
-            let lbr' = case_branches_specif renv lcv lbr in
-            let stl  =
-              Array.map (fun (renv',br') -> crec renv' br' ind) lbr' in
-            let stl0 = stl.(0) in
-	    if array_for_all (fun st -> st=stl0) stl then stl0 
-            (* branches do not return objects with same spec *)
-            else None
+let rec subterm_specif renv t ind = 
+  let f,l = decompose_app (whd_betadeltaiota renv.env t) in
+  match kind_of_term f with 
+    | Rel k -> subterm_var k renv
+
+    | Case (ci,_,c,lbr) ->
+        if Array.length lbr = 0 then Dead_code
+        else
+          let lbr_spec = case_branches_specif renv c ci.ci_ind lbr in
+          let stl  =
+            Array.map (fun (renv',br') -> subterm_specif renv' br' ind)
+              lbr_spec in
+          subterm_spec_glb stl
 	       
-      | Fix ((recindxs,i),(_,typarray,bodies as recdef)) ->
+    | Fix ((recindxs,i),(_,typarray,bodies as recdef)) ->
 (* when proving that the fixpoint f(x)=e is less than n, it is enough
    to prove that e is less than n assuming f is less than n
    furthermore when f is applied to a term which is strictly less than
    n, one may assume that x itself is strictly less than n
 *)
-          let nbfix = Array.length typarray in
-          let recargs = lookup_subterms renv.env ind in
-          (* pushing the fixpoints *)
-          let renv' = push_fix_renv renv recdef in
-          let renv' =
-            { renv' with lst=(nbfix-i,(Strict,recargs))::renv'.lst } in
-          let decrArg = recindxs.(i) in 
-          let theBody = bodies.(i)   in
-          let nbOfAbst = decrArg+1 in
-          let sign,strippedBody = decompose_lam_n_assum nbOfAbst theBody in
-          (* pushing the fix parameters *)
-          let renv'' = push_ctxt_renv renv' sign in
-          let renv'' =
-            { renv'' with
-              lst =
-              if List.length l < nbOfAbst then renv''.lst
-              else
-                let decrarg_ind =
-                  inductive_of_fix renv''.env decrArg theBody in
-                let theDecrArg  = List.nth l decrArg in
-	        try 
-	          match crec renv theDecrArg decrarg_ind with 
-		    (Some recArgsDecrArg) ->
-                      (1,(Strict,recArgsDecrArg)) :: renv''.lst 
-                  | None -> renv''.lst
-	        with Not_found -> renv''.lst } in
-	  crec renv'' strippedBody ind
+        let nbfix = Array.length typarray in
+        let recargs = lookup_subterms renv.env ind in
+        (* pushing the fixpoints *)
+        let renv' = push_fix_renv renv recdef in
+        let renv' =
+          assign_var_spec renv' (nbfix-i, Subterm(Strict,recargs)) in
+        let decrArg = recindxs.(i) in 
+        let theBody = bodies.(i)   in
+        let nbOfAbst = decrArg+1 in
+        let sign,strippedBody = decompose_lam_n_assum nbOfAbst theBody in
+        (* pushing the fix parameters *)
+        let renv'' = push_ctxt_renv renv' sign in
+        let renv'' =
+          if List.length l < nbOfAbst then renv''
+          else
+            let decrarg_ind = inductive_of_fix renv''.env decrArg theBody in
+            let theDecrArg  = List.nth l decrArg in
+	    let arg_spec    = subterm_specif renv theDecrArg decrarg_ind in
+            assign_var_spec renv'' (1, arg_spec) in
+	subterm_specif renv'' strippedBody ind
 	    
-      | Lambda (x,a,b) -> 
-          assert (l=[]);
-          crec (push_var_renv renv (x,a)) b ind
+    | Lambda (x,a,b) -> 
+        assert (l=[]);
+        subterm_specif (push_var_renv renv (x,a)) b ind
 
-      (* A term with metas is considered OK *)
-      | Meta _ -> Some (lookup_subterms renv.env ind)
-      (* Other terms are not subterms *)
-      | _ -> None
-  in 
-  crec renv c ind
+    (* A term with metas is considered OK *)
+    | Meta _ -> Dead_code
+    (* Other terms are not subterms *)
+    | _ -> Not_subterm
 
 (* Propagation of size information through Cases: if the matched
    object is a recursive subterm then compute the information
-   associated to its own subterms. *)
-let spec_subterm_large renv c ind = 
-  match subterm_var_large renv c with
-    Some s -> s
-  | _ ->
-      (let nb = Array.length (lookup_subterms renv.env ind) in
-      match subterm_specif renv c ind 
-      with Some lr -> lr | None -> Array.create nb [])
+   associated to its own subterms.
+   Rq: if branch is not eta-long, then the recursive information
+   is not propagated *)
+and case_branches_specif renv c ind lbr = 
+  let c_spec = subterm_specif renv c ind in
+  let rec push_branch_args renv lrec c = 
+    let c' = strip_outer_cast (whd_betadeltaiota renv.env c) in 
+    match lrec, kind_of_term c' with 
+      |	(ra::lr,Lambda (x,a,b)) ->
+          let renv' = push_var renv (x,a,ra) in
+          push_branch_args renv' lr b
+      | (_,_) -> (renv,c') in
+  match c_spec with
+      Subterm (_,t) ->
+        let sub_spec = Array.map (List.map spec_of_tree) (dest_subterms t) in
+        assert (Array.length sub_spec = Array.length lbr);
+        array_map2 (push_branch_args renv) sub_spec lbr
+    | Dead_code -> 
+        let t = dest_subterms (lookup_subterms renv.env ind) in
+        let sub_spec = Array.map (List.map (fun _ -> Dead_code)) t in
+        assert (Array.length sub_spec = Array.length lbr);
+        array_map2 (push_branch_args renv) sub_spec lbr
+    | Not_subterm -> Array.map (fun c -> (renv,c)) lbr
 
 (* Check term c can be applied to one of the mutual fixpoints. *)
 let check_is_subterm renv c ind = 
   match subterm_specif renv c ind with
-    Some _ -> ()
+    Subterm (Strict,_) | Dead_code -> ()
   |  _ -> raise (FixGuardError RecursionOnIllegalTerm)
 
 (***********************************************************************)
@@ -620,12 +542,11 @@ let check_one_fix renv recpos def =
 	  else List.for_all (check_rec_call renv) l        
 
       | Case (ci,p,c_0,lrest) ->
-            (* compute the recarg information for the arguments of
-               each branch *)
-            let lc  = spec_subterm_large renv c_0 ci.ci_ind in
-            let lbr = case_branches_specif renv lc lrest in
-            array_for_all (fun (renv',br') -> check_rec_call renv' br') lbr
-	    && List.for_all (check_rec_call renv) (c_0::p::l)
+          List.for_all (check_rec_call renv) (c_0::p::l) &&
+          (* compute the recarg information for the arguments of
+             each branch *)
+          let lbr = case_branches_specif renv c_0 ci.ci_ind lrest in
+          array_for_all (fun (renv',br') -> check_rec_call renv' br') lbr
 
   (* Enables to traverse Fixpoint definitions in a more intelligent
      way, ie, the rule :
@@ -647,26 +568,30 @@ let check_one_fix renv recpos def =
           array_for_all (check_rec_call renv) typarray && 
 	  let nbfix = Array.length typarray in
           let decrArg = recindxs.(i) in
-	  let theBody = bodies.(i)   in
           let renv' = push_fix_renv renv recdef in 
           if (List.length l < (decrArg+1)) then
 	    array_for_all (check_rec_call renv') bodies
           else 
-            let decrarg_ind = inductive_of_fix renv'.env decrArg theBody in
-            let theDecrArg  = List.nth l decrArg in
-	    (try 
-	      match subterm_specif renv theDecrArg decrarg_ind with
-                  Some recArgsDecrArg -> 
-		    check_nested_fix_body renv'
-                      (decrArg+1) recArgsDecrArg theBody
-		| None -> array_for_all (check_rec_call renv') bodies
-	     with Not_found -> array_for_all (check_rec_call renv') bodies)
+            let ok_vect =
+              Array.mapi
+                (fun j body ->
+                  if i=j then
+                    let decrarg_ind =
+                      inductive_of_fix renv'.env decrArg body in
+                    let theDecrArg  = List.nth l decrArg in
+	            let arg_spec =
+                      subterm_specif renv theDecrArg decrarg_ind in
+	            check_nested_fix_body renv' (decrArg+1) arg_spec body
+                  else check_rec_call renv' body)
+                bodies in
+            array_for_all (fun b -> b) ok_vect
 
       | Const sp as c -> 
           (try List.for_all (check_rec_call renv) l
            with (FixGuardError _ ) as e ->
-             if evaluable_constant renv.env sp then 
-	       check_rec_call renv (whd_betadeltaiota renv.env t)
+             if evaluable_constant sp renv.env then 
+	       check_rec_call renv
+                 (applist(constant_value renv.env sp, l))
 	     else raise e)
 
       (* The cases below simply check recursively the condition on the
@@ -702,14 +627,13 @@ let check_one_fix renv recpos def =
 
   and check_nested_fix_body renv decr recArgsDecrArg body =
     if decr = 0 then
-      check_rec_call
-        { renv with lst=(1,(Strict,recArgsDecrArg))::renv.lst } body
+      check_rec_call (assign_var_spec renv (1,recArgsDecrArg)) body
     else
       match kind_of_term body with
 	| Lambda (x,a,b) ->
+            let renv' = push_var_renv renv (x,a) in
 	    check_rec_call renv a &&
-	    check_nested_fix_body (push_var_renv renv (x,a))
-              (decr-1) recArgsDecrArg b
+	    check_nested_fix_body renv' (decr-1) recArgsDecrArg b
 	| _ -> anomaly "Not enough abstractions in fix body"
     
   in
@@ -742,7 +666,7 @@ let inductive_of_mutfix env ((nvect,bodynum),(names,types,bodies as recdef)) =
                 (* get the inductive type of the fixpoint *)
                 let (mind, _) = 
                   try find_inductive env a 
-                  with Induc -> raise_err i RecursionNotOnInductiveType in
+                  with Not_found -> raise_err i RecursionNotOnInductiveType in
                 (mind, (env', b))
 	      else check_occur env' (n+1) b
             else anomaly "check_one_fix: Bad occurrence of recursive call"
@@ -778,23 +702,17 @@ exception CoFixGuardError of guard_error
 let anomaly_ill_typed () =
   anomaly "check_one_cofix: too many arguments applied to constructor"
 
+let rec codomain_is_coind env c =
+  let b = whd_betadeltaiota env c in
+  match kind_of_term b with
+    | Prod (x,a,b) ->
+	codomain_is_coind (push_rel (x, None, a) env) b 
+    | _ -> 
+	(try find_coinductive env b
+        with Not_found ->
+	  raise (CoFixGuardError (CodomainNotInductiveType b)))
 
 let check_one_cofix env nbfix def deftype = 
-  let rec codomain_is_coind env c =
-    let b = whd_betadeltaiota env c in
-    match kind_of_term b with
-      | Prod (x,a,b) ->
-	  codomain_is_coind (push_rel (x, None, a) env) b 
-      | _ -> 
-	  try
-	    find_coinductive env b
-          with Induc ->
-	    raise (CoFixGuardError (CodomainNotInductiveType b))
-  in
-  let (mind, _) = codomain_is_coind env deftype in
-  let (sp,tyi) = mind in
-  let lvlra = lookup_recargs env (sp,tyi) in
-  let vlra = lvlra.(tyi) in  
   let rec check_rec_call env alreadygrd n vlra  t =
     if noccur_with_meta n nbfix t then
       true 
@@ -821,29 +739,20 @@ let check_one_cofix env nbfix def deftype =
             let mI = inductive_of_constructor cstr_sp in
 	    let (mib,mip) = lookup_mind_specif env mI in
             let _,realargs = list_chop mip.mind_nparams args in
-            let rec process_args_of_constr l lra =
-              match l with
-                | [] -> true 
-                | t::lr ->
-		    (match lra  with 
-                       | [] -> anomaly_ill_typed () 
-                       |  (Mrec i)::lrar -> 
-                            let newvlra = lvlra.(i) in
-                            (check_rec_call env true n newvlra t) &&
-                            (process_args_of_constr lr lrar)
-			    
-                       |  (Imbr((sp,i) as ind_sp,lrc)::lrar)  ->
-                            let lc = imbr_recarg_expand env ind_sp lrc in
-                            check_rec_call env true n lc t &
-                            process_args_of_constr lr lrar
-				 
-                       |  _::lrar  -> 
-                            if (noccur_with_meta n nbfix t) 
-                            then (process_args_of_constr lr lrar)
-                            else raise (CoFixGuardError
-					  (RecCallInNonRecArgOfConstructor t)))
-            in (process_args_of_constr realargs lra)
-		 
+            let rec process_args_of_constr = function
+              | (t::lr), (rar::lrar) -> 
+                  if rar = mk_norec then
+                    if noccur_with_meta n nbfix t
+                    then process_args_of_constr (lr, lrar)
+                    else raise (CoFixGuardError
+		                 (RecCallInNonRecArgOfConstructor t))
+                  else
+                    let spec = dest_subterms rar in
+                    check_rec_call env true n spec t &&
+                    process_args_of_constr (lr, lrar)
+              | [],_ -> true 
+              | _ -> anomaly_ill_typed () 
+            in process_args_of_constr (realargs, lra)
 		 
 	| Lambda (x,a,b) ->
 	     assert (args = []);
@@ -880,10 +789,10 @@ let check_one_cofix env nbfix def deftype =
             else 
 	      raise (CoFixGuardError (RecCallInCasePred c))
               
-	| _    -> raise (CoFixGuardError NotGuardedForm)
-	      
-  in 
-  check_rec_call env false 1 vlra def
+	| _    -> raise (CoFixGuardError NotGuardedForm) in 
+  let (mind, _) = codomain_is_coind env deftype in
+  let vlra = lookup_subterms env mind in
+  check_rec_call env false 1 (dest_subterms vlra) def
 
 (* The  function which checks that the whole block of definitions 
    satisfies the guarded condition *)
