@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: safe_typing.mli,v 1.27 2001/11/20 15:35:21 herbelin Exp $ i*)
+(*i $Id: safe_typing.mli,v 1.29 2002/04/04 15:52:21 barras Exp $ i*)
 
 (*i*)
 open Names
@@ -32,7 +32,6 @@ val push_named_assum :
 val push_named_def :
   identifier * constr * types option -> safe_environment ->
     Univ.constraints * safe_environment
-val pop_named_decls : identifier list -> safe_environment -> safe_environment
 
 (* Adding global axioms or definitions *)
 type constant_entry = {
@@ -46,11 +45,11 @@ type global_declaration =
   | GlobalRecipe of Cooking.recipe
 
 val add_constant : 
-  section_path -> global_declaration -> safe_environment -> safe_environment
+  constant -> global_declaration -> safe_environment -> safe_environment
 
 (* Adding an inductive type *)
 val add_mind : 
-  section_path -> Indtypes.mutual_inductive_entry -> safe_environment ->
+  mutual_inductive -> Indtypes.mutual_inductive_entry -> safe_environment ->
     safe_environment
 
 (* Adding universe constraints *)
