@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: sign.ml,v 1.31 2001/12/13 09:03:03 filliatr Exp $ *)
+(* $Id: sign.ml,v 1.35 2002/04/04 15:52:22 barras Exp $ *)
 
 open Names
 open Util
@@ -15,7 +15,6 @@ open Term
 (*s Signatures of named hypotheses. Used for section variables and
     goal assumptions. *)
 
-type named_declaration = identifier * constr option * types
 type named_context = named_declaration list
 
 let empty_named_context = []
@@ -29,10 +28,6 @@ let rec lookup_named id = function
 
 let named_context_length = List.length
 
-let pop_named_decl id = function
-  | (id',_,_) :: sign -> assert (id = id'); sign
-  | [] -> assert false
-
 let instance_from_named_context sign =
   let rec inst_rec = function
     | (id,None,_) :: sign -> mkVar id :: inst_rec sign
@@ -66,6 +61,13 @@ let lookup_rel n sign =
 
 let rel_context_length = List.length
 
+let rel_context_nhyps hyps =
+  let rec nhyps acc = function
+    | [] -> acc
+    | (_,None,_)::hyps -> nhyps (1+acc) hyps
+    | (_,Some _,_)::hyps -> nhyps acc hyps in
+  nhyps 0 hyps
+
 let fold_rel_context f l ~init:x = List.fold_right f l x
 let fold_rel_context_reverse f ~init:x l = List.fold_left f x l
 
@@ -91,10 +93,6 @@ let push_named_to_rel_context hyps ctxt =
 (*       Term constructors       *)
 (*********************************)
 
-let it_mkNamedProd_or_LetIn   =
-  List.fold_left (fun c d -> mkNamedProd_or_LetIn d c)
-let it_mkNamedLambda_or_LetIn =
-  List.fold_left (fun c d -> mkNamedLambda_or_LetIn d c)
 let it_mkProd_or_LetIn   = List.fold_left (fun c d -> mkProd_or_LetIn d c)
 let it_mkLambda_or_LetIn = List.fold_left (fun c d -> mkLambda_or_LetIn d c)
 
