@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: sign.mli,v 1.35 2001/11/20 23:04:25 herbelin Exp $ i*)
+(*i $Id: sign.mli,v 1.38 2002/04/04 15:52:22 barras Exp $ i*)
 
 (*i*)
 open Names
@@ -20,7 +20,6 @@ type section_context = named_context
 
 val empty_named_context : named_context
 val add_named_decl : named_declaration -> named_context -> named_context
-val pop_named_decl : identifier -> named_context -> named_context
 
 val lookup_named : identifier -> named_context -> named_declaration
 val named_context_length : named_context -> int
@@ -46,6 +45,7 @@ val add_rel_decl : rel_declaration -> rel_context -> rel_context
 
 val lookup_rel : int -> rel_context -> rel_declaration
 val rel_context_length : rel_context -> int
+val rel_context_nhyps : rel_context -> int
 
 val push_named_to_rel_context : named_context -> rel_context -> rel_context
 
@@ -58,9 +58,6 @@ val fold_rel_context_reverse :
 
 (*s Term constructors *)
 
-val it_mkNamedLambda_or_LetIn : constr -> named_context -> constr
-val it_mkNamedProd_or_LetIn : types -> named_context -> types
-
 val it_mkLambda_or_LetIn : constr -> rel_context -> constr
 val it_mkProd_or_LetIn : types -> rel_context -> types
 
