@@ -6,52 +6,38 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: univ.mli,v 1.15 2001/03/28 15:11:13 barras Exp $ i*)
-
-(*i*)
-open Names
-(*i*)
+(*i $Id: univ.mli,v 1.21 2001/11/12 12:37:59 barras Exp $ i*)
 
 (* Universes. *)
 
 type universe
 
-val dummy_univ : universe
-val implicit_univ : universe
-
 val prop_univ : universe
-val prop_univ_univ : universe
+val make_univ : Names.dir_path * int -> universe
 
-val set_module : dir_path -> unit
-
-val new_univ : unit -> universe
+(* The type of a universe *)
+val super : universe -> universe
+(* The max of 2 universes *)
+val sup   : universe -> universe -> universe
 
 (*s Graphs of universes. *)
 
 type universes
 
+(* The empty graph of universes *)
 val initial_universes : universes
 
 (*s Constraints. *)
 
-type constraint_type = Gt | Geq | Eq
-
-type univ_constraint = universe * constraint_type * universe
-
-module Constraint : Set.S with type elt = univ_constraint
+module Constraint : Set.S
 
 type constraints = Constraint.t
 
 type constraint_function = universe -> universe -> constraints -> constraints
 
-val enforce_gt : constraint_function
 val enforce_geq : constraint_function
 val enforce_eq : constraint_function
 
-val super : universe -> universe * constraints
-
-val sup : universe -> universe -> universes -> universe * constraints
-
 (*s Merge of constraints in a universes graph. 
   The function [merge_constraints] merges a set of constraints in a given
   universes graph. It raises the exception [UniverseInconsistency] if the
@@ -65,3 +51,11 @@ val merge_constraints : constraints -> universes -> universes
 
 val pr_uni : universe -> Pp.std_ppcmds
 val pr_universes : universes -> Pp.std_ppcmds
+
+val string_of_univ : universe -> string
+
+(*s Dumping to a file *)
+
+val dump_universes : out_channel -> universes -> unit
+
+val hcons1_univ : universe -> universe
