@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: options.ml,v 1.12 2001/10/11 17:27:19 herbelin Exp $ *)
+(* $Id: options.ml,v 1.14 2002/02/14 14:38:57 filliatr Exp $ *)
 
 open Util
 
@@ -49,18 +49,29 @@ let set_print_hyps_limit n = print_hyps_limit := Some n
 let unset_print_hyps_limit () = print_hyps_limit := None
 let print_hyps_limit () = !print_hyps_limit
 
-let mes_ambig = ref true
-let make_mes_ambig flag = mes_ambig:=flag
-let is_mes_ambig() = !mes_ambig
-
-let without_mes_ambig f x =
-  let old = is_mes_ambig() in
-  try make_mes_ambig false;
-      let rslt = f x in (make_mes_ambig old; rslt)
-  with e -> (make_mes_ambig old; raise e)
-
 (* A list of the areas of the system where "unsafe" operation
  * has been requested *)
 let unsafe_set = ref Stringset.empty
 let add_unsafe s = unsafe_set := Stringset.add s !unsafe_set
 let is_unsafe s = Stringset.mem s !unsafe_set
+
+
+(* Dump of globalization (to be used by coqdoc) *)
+
+let dump = ref false
+let dump_file = ref ""
+let dump_into_file f = dump := true; dump_file := f
+
+let dump_buffer = Buffer.create 8192
+
+let dump_string = Buffer.add_string dump_buffer
+
+let dump_it () = 
+  if !dump then begin
+    let mode = [Open_wronly; Open_append; Open_creat] in
+    let c = open_out_gen mode 0o666 !dump_file in
+    output_string c (Buffer.contents dump_buffer);
+    close_out c
+  end
+
+let _ = at_exit dump_it
