@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: options.mli,v 1.13 2001/10/11 17:27:20 herbelin Exp $ i*)
+(*i $Id: options.mli,v 1.15 2002/02/14 14:38:57 filliatr Exp $ i*)
 
 (* Global options of the system. *)
 
@@ -32,9 +32,12 @@ val set_print_hyps_limit : int -> unit
 val unset_print_hyps_limit : unit -> unit
 val print_hyps_limit : unit -> int option
 
-val make_mes_ambig : bool -> unit
-val is_mes_ambig : unit -> bool
-val without_mes_ambig : ('a -> 'b) -> 'a -> 'b
-
 val add_unsafe : string -> unit
 val is_unsafe : string -> bool
+
+(* Dump of globalization (to be used by coqdoc) *)
+
+val dump : bool ref
+val dump_into_file : string -> unit
+val dump_string : string -> unit
+
