@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: util.ml,v 1.58 2001/12/13 09:03:04 filliatr Exp $ *)
+(* $Id: util.ml,v 1.62 2002/04/10 16:07:52 herbelin Exp $ *)
 
 open Pp
 
@@ -121,6 +121,13 @@ let list_assign l n e =
   in 
   assrec [] (l,n)
 
+let list_map_left f = (* ensures the order in case of side-effects *)
+  let rec map_rec = function
+    | [] -> [] 
+    | x::l -> let v = f x in v :: map_rec l
+  in 
+  map_rec
+
 let list_map_i f = 
   let rec map_i_rec i = function
     | [] -> [] 
@@ -247,6 +254,11 @@ let list_splitby p =
   in 
   splitby_loop []
 
+let rec list_split3 = function
+  | [] -> ([], [], [])
+  | (x,y,z)::l ->
+      let (rx, ry, rz) = list_split3 l in (x::rx, y::ry, z::rz)
+
 let list_firstn n l =
   let rec aux acc = function
     | (0, l) -> List.rev acc
@@ -489,6 +501,11 @@ let option_app f = function
   | None -> None
   | Some x -> Some (f x)
 
+let option_compare f a b = match (a,b) with
+  | None, None -> true
+  | Some a', Some b' -> f a' b'
+  | _ -> failwith "option_compare"
+
 let map_succeed f = 
   let rec map_f = function 
     | [] -> []
@@ -503,6 +520,12 @@ let pr_fnl = fnl
 let pr_int = int
 let pr_str = str
 let pr_coma () = str "," ++ spc ()
+let pr_semicolon () = str ";" ++ spc ()
+let pr_bar () = str "|" ++ spc ()
+
+let pr_ord n =
+  let suff = match n mod 10 with 1 -> "st" | 2 -> "nd" | _ -> "th" in
+  int n ++ str suff
 
 let rec prlist elem l = match l with 
   | []   -> mt ()
