@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: declare.ml,v 1.105 2001/12/13 09:03:04 filliatr Exp $ *)
+(* $Id: declare.ml,v 1.111 2002/04/16 16:12:08 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -62,6 +62,16 @@ let make_strength_2 () =
   if depth > 2 then DischargeAt (extract_dirpath_prefix 2 cwd, depth-2)
   else NeverDischarge
 
+let is_less_persistent_strength = function
+  | (NeverDischarge,_) -> false
+  | (NotDeclare,_) -> false
+  | (_,NeverDischarge) -> true
+  | (_,NotDeclare) -> true
+  | (DischargeAt (sp1,_),DischargeAt (sp2,_)) ->
+      is_dirpath_prefix_of sp1 sp2
+
+let strength_min (stre1,stre2) =
+  if is_less_persistent_strength (stre1,stre2) then stre1 else stre2
 
 (* Section variables. *)
 
@@ -123,6 +133,9 @@ let _ = Summary.declare_summary "CONSTANT"
 	    Summary.survive_section = false }
 
 let cache_constant (sp,(cdt,stre)) =
+  (if Idmap.mem (basename sp) !vartab then
+    errorlabstrm "cache_constant" 
+      (pr_id (basename sp) ++ str " already exists"));
   (if Nametab.exists_cci sp then
     let (_,id) = repr_path sp in
     errorlabstrm "cache_constant" (pr_id id ++ str " already exists"));
@@ -204,6 +217,9 @@ let inductive_names sp mie =
   in names
 
 let check_exists_inductive (sp,_) =
+  (if Idmap.mem (basename sp) !vartab then
+    errorlabstrm "cache_inductive" 
+      (pr_id (basename sp) ++ str " already exists"));
   if Nametab.exists_cci sp then
     let (_,id) = repr_path sp in
     errorlabstrm "cache_inductive" (pr_id id ++ str " already exists")
@@ -226,7 +242,6 @@ let open_inductive (sp,mie) =
   List.iter (fun (sp, ref) -> Nametab.push 0 (restrict_path 0 sp) ref) names
 
 let dummy_one_inductive_entry mie = {
-  mind_entry_nparams = 0;
   mind_entry_params = [];
   mind_entry_typename = mie.mind_entry_typename;
   mind_entry_arity = mkProp;
@@ -385,3 +400,18 @@ let strength_of_global = function
   | ConstRef sp -> constant_strength sp
   | VarRef id -> variable_strength id
   | IndRef _ | ConstructRef _ -> NeverDischarge 
+
+let library_part ref =
+  let sp = Nametab.sp_of_global (Global.env ()) ref in
+  let dir,_ = repr_path sp in
+  match strength_of_global ref with
+  | DischargeAt (dp,n) -> 
+      extract_dirpath_prefix n dp
+  | NeverDischarge ->
+      if is_dirpath_prefix_of dir (Lib.cwd ()) then
+	(* Theorem/Lemma not yet (fully) discharged *)
+	extract_dirpath_prefix (Lib.sections_depth ()) (Lib.cwd ())
+      else
+	(* Theorem/Lemma outside its outer section of definition *)
+	dir
+  | NotDeclare -> assert false
