@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: declare.mli,v 1.61 2001/11/20 15:35:21 herbelin Exp $ i*)
+(*i $Id: declare.mli,v 1.64 2002/04/16 16:12:08 herbelin Exp $ i*)
 
 (*i*)
 open Names
@@ -31,6 +31,9 @@ type strength =
   | DischargeAt of dir_path * int
   | NeverDischarge
 
+val is_less_persistent_strength : strength * strength -> bool
+val strength_min : strength * strength -> strength
+
 type section_variable_entry =
   | SectionLocalDef of constr * types option
   | SectionLocalAssum of types
@@ -100,3 +103,5 @@ val construct_reference : Environ.env -> identifier -> constr
 val is_global : identifier -> bool
 
 val strength_of_global : global_reference -> strength
+
+val library_part : global_reference -> dir_path
