@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: ast.ml,v 1.19 2001/12/13 09:03:05 filliatr Exp $ *)
+(* $Id: ast.ml,v 1.19.4.1 2002/07/30 18:13:17 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -338,6 +338,8 @@ let alpha_eq_val = function
   | _ ->  false
 
 let rec occur_var_ast s = function
+  | Node(_,"QUALID",_::_::_) -> false
+  | Node(_,"QUALID",[Nvar(_,s2)]) -> s = s2
   | Node(loc,op,args) -> List.exists (occur_var_ast s) args
   | Nvar(_,s2) -> s = s2
   | Smetalam _ | Nmeta _ -> anomaly "occur_var: metas should not occur here"
