@@ -6,10 +6,11 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: astterm.ml,v 1.91 2001/12/13 09:03:05 filliatr Exp $ *)
+(* $Id: astterm.ml,v 1.97 2002/05/14 21:27:10 herbelin Exp $ *)
 
 open Pp
 open Util
+open Options
 open Names
 open Nameops
 open Sign
@@ -107,6 +108,25 @@ let check_number_of_pattern loc n l =
 (* Arguments normally implicit in the "Implicit Arguments mode" *)
 (* but explicitely given                                        *)
 
+(* Dump of globalization (to be used by coqdoc) *)
+
+let add_glob loc ref = 
+(*i
+  let sp = Nametab.sp_of_global (Global.env ()) ref in
+  let dir,_ = repr_path sp in
+  let rec find_module d = 
+    try 
+      let qid = let dir,id = split_dirpath d in make_qualid dir id in
+      let _ = Nametab.locate_loaded_library qid in d
+    with Not_found -> find_module (dirpath_prefix d) 
+  in
+  let s = string_of_dirpath (find_module dir) in
+  i*)
+  let sp = Nametab.sp_of_global (Global.env ()) ref in
+  let id = let _,id = repr_path sp in string_of_id id in
+  let dp = string_of_dirpath (Declare.library_part ref) in
+  dump_string (Printf.sprintf "R%d %s.%s\n" (fst loc) dp id)
+
 (* Translation of references *)
 
 let ast_to_sp = function
@@ -157,38 +177,67 @@ type pattern_qualid_kind =
   | ConstrPat of loc * constructor
   | VarPat of loc * identifier
 
-let maybe_constructor env = function
+let may_allow_variable loc allow_var l =
+  match maybe_variable l with
+    | Some s when allow_var -> 
+      (* Why a warning since there is no warning when writing [globname:T]... 
+	 warning ("Defined reference "^(string_of_qualid qid)
+	 ^" is here considered as a matching variable");
+      *)
+        VarPat (loc,s)
+    | _ -> 
+        user_err_loc (loc,"maybe_constructor",
+        str "This reference does not denote a constructor: " ++
+        str (string_of_qualid (interp_qualid l)))
+
+let maybe_constructor allow_var env = function
   | Node(loc,"QUALID",l) ->
       let qid = interp_qualid l in
-      (try 
-	match kind_of_term (global_qualified_reference qid) with 
-	  | Construct c -> ConstrPat (loc,c)
-	  | _ -> 
-	      (match maybe_variable l with
-		 | Some s -> 
-		     warning ("Defined reference "^(string_of_qualid qid)
-			      ^" is here considered as a matching variable");
-		     VarPat (loc,s)
-		 | _ -> error ("This reference does not denote a constructor: "
-			       ^(string_of_qualid qid)))
-      with Not_found -> 
-	match maybe_variable l with
-	  | Some s -> VarPat (loc,s)
-	  | _ -> error ("Unknown qualified constructor: "
-			^(string_of_qualid qid)))
+      (try match extended_locate qid with
+       | SyntacticDef sp ->
+	   (match Syntax_def.search_syntactic_definition sp with
+	    | RRef (_,(ConstructRef c as x)) ->
+		if !dump then add_glob loc x; 
+ 		ConstrPat (loc,c)
+	    | _ -> 
+		user_err_loc (loc,"maybe_constructor",
+           str "This syntactic definition should be aliased to a constructor"))
+       | TrueGlobal r ->
+           let rec unf = function
+             | ConstRef cst ->
+                 (try
+                   unf
+                     (reference_of_constr (constant_value (Global.env()) cst))
+                 with
+                     NotEvaluableConst _ | Not_found ->
+                       may_allow_variable loc allow_var l)
+             | ConstructRef c -> 
+	         if !dump then add_glob loc r; 
+	         ConstrPat (loc,c)
+             | _ -> may_allow_variable loc allow_var l
+           in unf r
+       with Not_found -> 
+	 match maybe_variable l with
+	   | Some s when allow_var -> VarPat (loc,s)
+	   | _ -> error ("Unknown qualified constructor: "
+			 ^(string_of_qualid qid)))
 
   (* This may happen in quotations *)
   | Node(loc,"MUTCONSTRUCT",[sp;Num(_,ti);Num(_,n)]) ->
       (* Buggy: needs to compute the context *)
-      ConstrPat (loc,((ast_to_sp sp,ti),n))
+      let c = (ast_to_sp sp,ti),n in
+      if !dump then add_glob loc (ConstructRef c); 
+      ConstrPat (loc,c)
 
   | Path(loc,sp) ->
       (match absolute_reference sp with 
-	| ConstructRef c -> ConstrPat (loc,c)
+	| ConstructRef c as r -> 
+	    if !dump then add_glob loc (ConstructRef c);
+	    ConstrPat (loc,c)
 	| _ ->
             error ("Unknown absolute constructor name: "^(string_of_path sp)))
 
-  | Node(loc,("CONST"|"EVAR"|"MUTIND"|"SYNCONST" as key), l) ->
+  | Node(loc,("CONST"|"SECVAR"|"EVAR"|"MUTIND"|"SYNCONST" as key), l) ->
       user_err_loc (loc,"ast_to_pattern",
    (str "Found a pattern involving global references which are not constructors"
 ))
@@ -201,6 +250,10 @@ let ast_to_global loc c =
 	let ref = ConstRef (ast_to_sp sp) in
 	let imps = implicits_of_global ref in
 	RRef (loc, ref), imps
+    | ("SECVAR", [Nvar (_,s)]) ->
+	let ref = VarRef s in
+	let imps = implicits_of_global ref in
+	RRef (loc, ref), imps
     | ("MUTIND", [sp;Num(_,tyi)]) -> 
 	let ref = IndRef (ast_to_sp sp, tyi) in
 	let imps = implicits_of_global ref in
@@ -263,6 +316,7 @@ let rawconstr_of_qualid env vars loc qid =
   (* Is it a global reference or a syntactic definition? *)
   try match Nametab.extended_locate qid with
   | TrueGlobal ref ->
+    if !dump then add_glob loc ref;
     let imps = implicits_of_global ref in
     RRef (loc, ref), imps
   | SyntacticDef sp ->
@@ -302,17 +356,19 @@ let rec ast_to_pattern env aliases = function
       ast_to_pattern env aliases' p
 
   | Node(_,"PATTCONSTRUCT", head::((_::_) as pl)) ->
-      (match maybe_constructor env head with
+      (match maybe_constructor false env head with
 	 | ConstrPat (loc,c) ->
 	     let (idsl,pl') =
 	       List.split (List.map (ast_to_pattern env ([],[])) pl) in
 	     (aliases::(List.flatten idsl),
 	      PatCstr (loc,c,pl',alias_of aliases))
 	 | VarPat (loc,s) ->
-	     user_err_loc (loc,"ast_to_pattern",mssg_hd_is_not_constructor s))
-
+(*
+	     user_err_loc (loc,"ast_to_pattern",mssg_hd_is_not_constructor s)
+*)
+              assert false)
   | ast ->
-      (match maybe_constructor env ast with
+      (match maybe_constructor true env ast with
 	 | ConstrPat (loc,c) -> ([aliases], PatCstr (loc,c,[],alias_of aliases))
 	 | VarPat (loc,s) ->
 	     let aliases = merge_aliases aliases (name_of_nvar s) in
@@ -353,6 +409,15 @@ let check_capture loc s ty = function
   | Slam _ when occur_var_ast s ty -> error_capture_loc loc s
   | _ -> ()
 
+let locate_if_isevar loc id = function
+  | RHole _ -> RHole (loc, AbstractionType id)
+  | x -> x
+
+let set_hole_implicit i = function
+  | RRef (loc,r) -> (loc,ImplicitArg (r,i))
+  | RVar (loc,id) -> (loc,ImplicitArg (VarRef id,i))
+  | _ -> anomaly "Only refs have implicits"
+
 let ast_to_rawconstr sigma env allow_soapp lvar =
   let rec dbrec (ids,impls as env) = function
     | Nvar(loc,s) ->
@@ -386,14 +451,14 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	let arityl = Array.of_list (List.map (dbrec env) lA) in
 	RRec (loc,RCoFix n, Array.of_list lf, arityl, defl)
 	  
-    | Node(loc,("PROD"|"LAMBDA"|"LETIN" as k), [c1;Slam(_,ona,c2)]) ->
+    | Node(loc,("PROD"|"LAMBDA"|"LETIN" as k), [c1;Slam(locna,ona,c2)]) ->
 	let na,ids' = match ona with
 	  | Some id -> Name id, Idset.add id ids
 	  | _ -> Anonymous, ids in
 	let c1' = dbrec env c1 and c2' = dbrec (ids',impls) c2 in
 	(match k with
 	   | "PROD" -> RProd (loc, na, c1', c2')
-	   | "LAMBDA" -> RLambda (loc, na, c1', c2')
+	   | "LAMBDA" -> RLambda (loc, na, locate_if_isevar locna na c1', c2')
 	   | "LETIN" -> RLetIn (loc, na, c1', c2')
 	   | _ -> assert false)
 
@@ -408,13 +473,14 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	  match f with
 	    | Node(locs,"QUALID",p) ->
 		rawconstr_of_qualid env lvar locs (interp_qualid p)
+            (* For globalized references (e.g. in Infix) *) 
 	    | Node(loc,
-		   ("CONST"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),
+		   ("CONST"|"SECVAR"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),
 		   l) ->
 		ast_to_global loc (key,l)
 	    | _ -> (dbrec env f, [])
         in
-	  RApp (loc, c, ast_to_impargs env impargs args)
+	  RApp (loc, c, ast_to_impargs c env impargs args)
 
     | Node(loc,"CASES", p:: Node(_,"TOMATCH",tms):: eqns) ->
 	let po = match p with 
@@ -434,7 +500,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	ROldCase (loc,isrec,po,dbrec env c,
 		  Array.of_list (List.map (dbrec env) cl))
 
-    | Node(loc,"ISEVAR",[]) -> RHole (Some loc)
+    | Node(loc,"ISEVAR",[]) -> RHole (loc, QuestionMark)
     | Node(loc,"META",[Num(_,n)]) ->
 	if n<0 then error_metavar_loc loc else RMeta (loc, n)
 
@@ -443,7 +509,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
     | Node(loc,"TYPE", _) -> RSort(loc,RType None)
 	  
     (* This case mainly parses things build in a quotation *)
-    | Node(loc,("CONST"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),l) ->
+    | Node(loc,("CONST"|"SECVAR"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),l) ->
 	fst (ast_to_global loc (key,l))
 
     | Node(loc,"CAST", [c1;c2]) ->	   
@@ -496,18 +562,19 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	let r = iterated_binder oper (n+1) ty (ids',impls) body in
 	(match oper with
 	   | "PRODLIST" -> RProd(loc, na, dbrec env ty, r)
-	   | "LAMBDALIST" -> RLambda(loc, na, dbrec env ty, r)
+	   | "LAMBDALIST" ->
+	       RLambda(loc, na, locate_if_isevar loc na (dbrec env ty), r)
 	   | _ -> assert false)
     | body -> dbrec env body
 
-  and ast_to_impargs env l args =
+  and ast_to_impargs c env l args =
     let rec aux n l args = match (l,args) with 
       | (i::l',Node(loc, "EXPL", [Num(_,j);a])::args') ->
 	  if i=n & j>=i then
 	    if j=i then 
 	      (dbrec env a)::(aux (n+1) l' args')
 	    else 
-	      (RHole None)::(aux (n+1) l' args)
+	      (RHole (set_hole_implicit i c))::(aux (n+1) l' args)
 	  else 
 	    if i<>n then
 	      error ("Bad explicitation number: found "^
@@ -517,7 +584,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 		   (string_of_int j)^" but was expecting "^(string_of_int i))
       | (i::l',a::args') -> 
 	  if i=n then 
-	    (RHole None)::(aux (n+1) l' args)
+	    (RHole (set_hole_implicit i c))::(aux (n+1) l' args)
 	  else 
 	    (dbrec env a)::(aux (n+1) l args')
       | ([],args) -> ast_to_args env args
@@ -568,7 +635,7 @@ let globalize_qualid qid =
     let ref = Nametab.extended_locate qid in
     ast_of_extended_ref_loc dummy_loc ref
   with Not_found ->
-    Options.if_verbose warning_globalize qid;
+    if_verbose warning_globalize qid;
     Termast.ast_of_qualid qid
 
 let adjust_qualid env loc ast qid =
@@ -583,7 +650,7 @@ let adjust_qualid env loc ast qid =
     let ref = Nametab.extended_locate qid in
     ast_of_extended_ref_loc loc ref
   with Not_found ->
-    Options.if_verbose warning_globalize qid;
+    if_verbose warning_globalize qid;
     ast
 
 let ast_adjust_consts sigma =
