@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: prettyp.ml,v 1.14 2001/12/21 16:07:27 herbelin Exp $ *)
+(* $Id: prettyp.ml,v 1.15 2002/04/16 15:50:44 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -294,7 +294,7 @@ let print_inductive sp = (print_mutual sp ++ fnl ())
 let print_syntactic_def sep sp =
   let id = basename sp in
   let c = Syntax_def.search_syntactic_definition sp in 
-  (str" Syntactif Definition " ++ pr_id id ++ str sep ++ pr_rawterm c ++ fnl ())
+  (str" Syntactic Definition " ++ pr_id id ++ str sep ++ pr_rawterm c ++ fnl ())
 
 let print_leaf_entry with_values sep (sp,lobj) =
   let tag = object_tag lobj in
