@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: cases.ml,v 1.85 2001/12/19 08:56:21 herbelin Exp $ *)
+(* $Id: cases.ml,v 1.93 2002/05/03 15:50:56 herbelin Exp $ *)
 
 open Util
 open Names
@@ -63,12 +63,12 @@ let error_needs_inversion env x t =
 (* A) Typing old cases                                               *)
 (* This was previously in Indrec but creates existential holes       *)
 
-let mkExistential isevars env = new_isevar isevars env (new_Type ())
+let mkExistential isevars env loc = new_isevar isevars env loc (new_Type ())
 
 let norec_branch_scheme env isevars cstr =
   let rec crec env = function
     | d::rea -> mkProd_or_LetIn d (crec (push_rel d env) rea)
-    | [] -> mkExistential isevars env in 
+    | [] -> mkExistential isevars env (dummy_loc, InternalHole) in 
   crec env (List.rev cstr.cs_args)
 
 let rec_branch_scheme env isevars (sp,j) recargs cstr =
@@ -76,9 +76,10 @@ let rec_branch_scheme env isevars (sp,j) recargs cstr =
     match args, recargs with 
       | (name,None,c as d)::rea,(ra::reca) -> 
 	  let d =
- 	    match ra with 
+ 	    match dest_recarg ra with 
 	      | Mrec k when k=j ->
-		  let t = mkExistential isevars env in
+		  let t = mkExistential isevars env (dummy_loc, InternalHole)
+		  in
 		  mkArrow t
 		    (crec (push_rel (Anonymous,None,t) env)
 		       (List.rev (lift_rel_context 1 (List.rev rea)),reca))
@@ -87,18 +88,19 @@ let rec_branch_scheme env isevars (sp,j) recargs cstr =
 
       | (name,Some b,c as d)::rea, reca -> 
 	  mkLetIn (name,b,body_of_type c,crec (push_rel d env) (rea,reca))
-      | [],[] -> mkExistential isevars env
+      | [],[] -> mkExistential isevars env (dummy_loc, InternalHole)
       | _ -> anomaly "rec_branch_scheme"
   in 
   crec env (List.rev cstr.cs_args,recargs) 
     
-let branch_scheme env isevars isrec ((ind,params) as indf) =
+let branch_scheme env isevars isrec indf =
+  let (ind,params) = dest_ind_family indf in
   let (mib,mip) = Inductive.lookup_mind_specif env ind in
   let cstrs = get_constructors env indf in 
   if isrec then
     array_map2
       (rec_branch_scheme env isevars ind)
-      mip.mind_listrec cstrs
+      (dest_subterms mip.mind_recargs) cstrs
   else 
     Array.map (norec_branch_scheme env isevars) cstrs
 
@@ -116,8 +118,10 @@ let concl_n env sigma =
 let count_rec_arg j = 
   let rec crec i = function 
     | [] -> i 
-    | (Mrec k::l) -> crec (if k=j then (i+1) else i) l
-    | (_::l) -> crec i l
+    | ra::l ->
+        (match dest_recarg ra with
+            Mrec k -> crec (if k=j then (i+1) else i) l
+          | _ -> crec i l)
   in 
   crec 0
 
@@ -139,9 +143,9 @@ exception NotInferable of ml_case_error
 
 let pred_case_ml env sigma isrec (IndType (indf,realargs)) (i,ft) =
   let pred =
-    let (ind,params) = indf in
+    let (ind,params) = dest_ind_family indf in
     let (mib,mip) = Inductive.lookup_mind_specif env ind in
-    let recargs = mip.mind_listrec in
+    let recargs = dest_subterms mip.mind_recargs in
     if Array.length recargs = 0 then raise (NotInferable MlCaseAbsurd);
     let recargi = recargs.(i) in
     let j = snd ind in (* index of inductive *)
@@ -424,7 +428,7 @@ let inh_coerce_to_ind isevars env ty tyi =
     List.fold_right
       (fun (na,ty) (env,evl) ->
 	 (push_rel (na,None,ty) env,
-	    (new_isevar isevars env ty)::evl))
+	    (new_isevar isevars env (dummy_loc, InternalHole) ty)::evl))
       ntys (env,[]) in
   let expected_typ = applist (mkInd tyi,evarl) in
      (* devrait tre indiffrent d'exiger leq ou pas puisque pour 
@@ -449,12 +453,12 @@ let unify_tomatch_with_patterns isevars env typ = function
 	 try
 	   IsInd (typ,find_rectype env (evars_of isevars) typ)
              (* will try to coerce later in check_and_adjust_constructor.. *)
-	 with Induc ->
+	 with Not_found ->
 	   NotInd (None,typ))
 	     (* error will be detected in check_all_variables *)
   | None -> 
       try IsInd (typ,find_rectype env (evars_of isevars) typ)
-      with Induc -> NotInd (None,typ)
+      with Not_found -> NotInd (None,typ)
 
 let coerce_row typing_fun isevars env cstropt tomatch =
   let j = typing_fun empty_tycon env tomatch in
@@ -651,14 +655,16 @@ let occur_in_rhs na rhs =
     | Anonymous -> false
     | Name id -> occur_rawconstr id rhs.it
 
-let is_dep_patt eqn pat = occur_in_rhs (alias_of_pat pat) eqn.rhs
+let is_dep_patt eqn = function
+  | PatVar (_,name) -> occur_in_rhs name eqn.rhs
+  | _ -> false
 
 let dependencies_in_rhs nargs eqns =
   if eqns = [] then list_tabulate (fun _ -> false) nargs (* Only "_" patts *)
   else
   let deps = List.map (fun (tms,eqn) -> List.map (is_dep_patt eqn) tms) eqns in
   let columns = matrix_transpose deps in
-  List.map (List.for_all ((=) true)) columns
+  List.map (List.exists ((=) true)) columns
 
 (* Introduction of an argument of the current constructor must be
    delayed (flag DepOnPrevious) if it depends in the rhs and depends
@@ -667,7 +673,7 @@ let dependencies_in_rhs nargs eqns =
 
 let rec is_dep_on_previous n t = function
   | ((_,IsInd _),_)::_ when dependent (mkRel n) t -> DepOnPrevious
-  | ((_,NotInd _),(DepOnPrevious,DepInRhs))::_ 
+  | ((_,NotInd _),_(*DepOnPrevious,DepInRhs*))::_ 
       when dependent (mkRel n) t -> DepOnPrevious
   | _::rest -> is_dep_on_previous (n+1) t rest
   | [] -> NotDepOnPrevious
@@ -699,14 +705,15 @@ let rec lift_subst_tomatch n (depth,ci as binder) = function
   (* and they contribute to the environment (hence [depth+1]) *)
   | ToPush ((na,t),info)::rest ->
       let t' = liftn_tomatch_type n (depth+1) t in
-      let t'' = subst_tomatch binder t' in
+      let t'' = substnl_tomatch [ci] (depth-1) t' in
       ToPush ((na,t''), info)::(lift_subst_tomatch n (depth+1,ci) rest)
 
   (* By definition Pushed terms do not depend on previous terms to match *)
   (* and are already pushed in the environment; *)
-  (* if all is correct, [c] has no variables < depth *)
+  (* if all is correct, [tm] has no variables < depth *)
+  (* +n-1 and not +n to simulate the substitution we don't apply *)
   | Pushed (lift,tm)::rest ->
-      Pushed (n+lift, tm)::(lift_subst_tomatch n binder rest)
+      Pushed (lift+n-1, tm)::(lift_subst_tomatch n (depth,ci) rest)
 
 let rec liftn_tomatch_stack n depth = function
   | [] -> []
@@ -788,7 +795,9 @@ let build_aliases_context env sigma names allpats pats =
   (* They all are defined in env and we turn them into a sign *)
   (* cuts in sign need to be done in allpats *)
   let rec insert env sign1 sign2 n newallpats oldallpats = function
-    | _::pats, Anonymous::names ->
+    | (deppat,_,_)::pats, Anonymous::names when not (isRel deppat) ->
+        (* Anonymous leaves must be considered named and treated in the *)
+        (* next clause because they may occur in implicit arguments *)
 	insert env sign1 sign2
 	  n newallpats (List.map List.tl oldallpats) (pats,names)
     | (deppat,nondeppat,d)::pats, na::names ->
@@ -936,7 +945,8 @@ let abstract_conclusion typ cs =
   let (sign,p) = decompose_prod_n n typ in
   lam_it p sign
 
-let infer_predicate loc env isevars typs cstrs ((mis,_) as indf) =
+let infer_predicate loc env isevars typs cstrs indf =
+  let (mis,_) = dest_ind_family indf in
   (* Il faudra substituer les isevars a un certain moment *)
   if Array.length cstrs = 0 then (* "TODO4-3" *)
     error "Inference of annotation for empty inductive types not implemented"
@@ -955,7 +965,7 @@ let infer_predicate loc env isevars typs cstrs ((mis,_) as indf) =
 	(* Heuristic to avoid comparison between non-variables algebric univs*)
 	new_Type ()
       else
-	mkExistential isevars env
+	mkExistential isevars env (loc, CasesType)
     in
     if array_for_all (fun (_,_,typ) -> the_conv_x_leq env isevars typ mtyp) eqns
     then
@@ -1068,10 +1078,12 @@ let abstract_predicate env sigma indf = function
       let sign = make_arity_signature env true indf in
       (true, it_mkLambda_or_LetIn_name env pred sign)
 
-let known_dependent = function
+let rec known_dependent = function
   | None -> false
   | Some (PrLetIn ((_,copt),_)) -> copt <> None
-  | Some (PrProd _ | PrCcl _ | PrNotInd _) ->
+  | Some (PrNotInd (_,p)) -> known_dependent (Some p)
+  | Some (PrCcl _) -> false
+  | Some (PrProd _) ->
       anomaly "known_dependent: can only be used when patterns remain"
 
 (*****************************************************************************)
@@ -1167,7 +1179,21 @@ let solve_constraints constr_info indt =
   (* TODO *)
   Constraints []
 
-let group_equations mind current cstrs mat =
+let rec irrefutable env = function
+  | PatVar (_,name) -> true
+  | PatCstr (_,cstr,args,_) ->
+      let ind = inductive_of_constructor cstr in
+      let (_,mip) = Inductive.lookup_mind_specif env ind in
+      let one_constr = Array.length mip.mind_user_lc = 1 in
+      one_constr & List.for_all (irrefutable env) args
+
+let first_clause_irrefutable env = function
+  | eqn::mat -> List.for_all (irrefutable env) eqn.patterns
+  | _ -> false
+
+let group_equations pb mind current cstrs mat =
+  let mat =
+    if first_clause_irrefutable pb.env mat then [List.hd mat] else mat in
   let brs = Array.create (Array.length cstrs) [] in
   let only_default = ref true in
   let _ =
@@ -1210,7 +1236,7 @@ let shift_problem (current,t) pb =
      mat = List.map remove_current_pattern pb.mat }
 
 (* Building the sub-pattern-matching problem for a given branch *)
-let build_branch current typ pb eqns const_info =
+let build_branch current pb eqns const_info =
   (* We remember that we descend through a constructor *)
   let alias_type =
     if Array.length const_info.cs_concl_realargs = 0
@@ -1245,7 +1271,8 @@ let build_branch current typ pb eqns const_info =
   let tomatchs =
     List.fold_left2
       (fun l (d,t) dep_in_rhs -> (d,find_dependencies t l dep_in_rhs)::l)
-      [] typs' (dependencies_in_rhs const_info.cs_nargs eqns) in
+      [] (List.rev typs') (dependencies_in_rhs const_info.cs_nargs eqns) in
+  let tomatchs = List.rev tomatchs in
   let topushs = List.map (fun x -> ToPush x) tomatchs in
   (* The dependent term to subst in the types of the remaining UnPushed 
      terms is relative to the current context enriched by topushs *)
@@ -1256,8 +1283,13 @@ let build_branch current typ pb eqns const_info =
        @(extended_rel_list 0 const_info.cs_args)) in
 
   (* We replace [(mkRel 1)] by its expansion [ci] *)
+  (* and context "Gamma; current" by "Gamma; current; tomatchs" *)
+  (* This is done in two steps : first from "Gamma; current |- oldtm : T(1)" *)
+  (* into  "Gamma; current; tomatchs; current |- ... : liftn 1 (n+1) T(1)" *)
+  (* then into "Gamma; current; tomatchs |- updated_old_tomatch : U" where *)
+  (* U = subst ci (lift (n+1) T(1)) *)
   let updated_old_tomatch =
-    lift_subst_tomatch (const_info.cs_nargs) (1,ci) pb.tomatch in
+    lift_subst_tomatch (const_info.cs_nargs + 1) (1,ci) pb.tomatch in
 
   { pb with
       tomatch = topushs@updated_old_tomatch;
@@ -1296,12 +1328,12 @@ and match_current pb (n,tm) =
     | IsInd (_,(IndType(indf,realargs) as indt)) ->
 	let mind,_ = dest_ind_family indf in
 	let cstrs = get_constructors pb.env indf in
-	let eqns,onlydflt = group_equations mind current cstrs pb.mat in
+	let eqns,onlydflt = group_equations pb mind current cstrs pb.mat in
 	if (cstrs <> [||] or not (initial_history pb.history)) & onlydflt then
 	  compile_aliases (shift_problem ct pb)
 	else
           let constraints = Array.map (solve_constraints indt) cstrs in
-	  let pbs = array_map2 (build_branch current typ pb) eqns cstrs in
+	  let pbs = array_map2 (build_branch current pb) eqns cstrs in
 	  let brs = Array.map compile_aliases pbs in
 	  let brvals = Array.map (fun (_,j) -> j.uj_val) brs in
 	  let brtyps = Array.map (fun (_,j) -> body_of_type j.uj_type) brs in
@@ -1484,7 +1516,7 @@ let extract_predicate_conclusion nargs pred =
       | _ -> (* eta-expansion *) applist (lift 1 p, [mkRel 1]) in
   iterate decomp_lam_force nargs pred
 
-let prepare_predicate_from_tycon dep env isevars tomatchs c =
+let prepare_predicate_from_tycon loc dep env isevars tomatchs c =
   let cook (n, l, env) = function
     | c,IsInd (_,IndType(indf,realargs)) ->
 	let indf' = lift_inductive_family n indf in
@@ -1504,7 +1536,7 @@ let prepare_predicate_from_tycon dep env isevars tomatchs c =
  (* let c = whd_betadeltaiota env (evars_of isevars) c in *)
     (* We turn all subterms possibly dependent into an evar with maximum ctxt*)
     if isEvar c or List.exists (eq_constr c) allargs then
-      mkExistential isevars env
+      mkExistential isevars env (loc, CasesType)
     else
       map_constr_with_full_binders push_rel build_skeleton env c in
   build_skeleton env (lift n c)
@@ -1544,12 +1576,12 @@ let build_initial_predicate env sigma isdep pred tomatchl =
  *        else error! (can not treat mixed dependent and non dependent case
  *)
 
-let prepare_predicate typing_fun isevars env tomatchs tycon = function
+let prepare_predicate loc typing_fun isevars env tomatchs tycon = function
   | None ->
       (match tycon with
        | None -> None
        | Some t ->
-	 let pred = prepare_predicate_from_tycon false env isevars tomatchs t in
+	 let pred = prepare_predicate_from_tycon loc false env isevars tomatchs t in
 	 Some
 	  (build_initial_predicate env (evars_of isevars) false pred tomatchs))
   | Some pred ->
@@ -1598,7 +1630,7 @@ let compile_cases loc (typing_fun,isevars) tycon env (predopt, tomatchl, eqns)=
 
   (* We build the elimination predicate if any and check its consistency *)
   (* with the type of arguments to match *)
-  let pred = prepare_predicate typing_fun isevars env tomatchs tycon predopt in
+  let pred = prepare_predicate loc typing_fun isevars env tomatchs tycon predopt in
 
   (* We deal with initial aliases *)
   let matx = prepare_initial_aliases (known_dependent pred) tomatchs matx in 
