@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: cases.mli,v 1.19 2001/12/18 09:16:00 herbelin Exp $ i*)
+(*i $Id: cases.mli,v 1.20 2002/02/07 16:07:28 barras Exp $ i*)
 
 (*i*)
 open Names
@@ -33,7 +33,7 @@ exception PatternMatchingError of env * pattern_matching_error
 (*s Used for old cases in pretyping *)
 
 val branch_scheme : 
-  env -> evar_defs -> bool -> inductive * constr list -> constr array
+  env -> evar_defs -> bool -> inductive_family -> constr array
 
 type ml_case_error =
   | MlCaseAbsurd
