@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: cbv.ml,v 1.9 2001/11/29 09:21:16 barras Exp $ *)
+(* $Id: cbv.ml,v 1.10 2002/03/01 16:51:20 herbelin Exp $ *)
 
 open Util
 open Pp
@@ -207,7 +207,11 @@ let rec norm_head info env t stack =
       (* allow substitution but leave let's in place *)
       let zeta = red_set (info_flags info) fZETA in
       let env' =
-	if zeta or red_set (info_flags info) fDELTA then 
+	if zeta 
+          (* New rule: for Cbv, Delta does not apply to locally bound variables
+          or red_set (info_flags info) fDELTA
+          *)
+        then 
 	  subs_cons (cbv_stack_term info TOP env b,env)
 	else
 	  subs_lift env in
