@@ -6,23 +6,23 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: evarconv.mli,v 1.9 2001/03/15 13:38:36 filliatr Exp $ i*)
+(*i $Id: evarconv.mli,v 1.11 2001/11/06 13:05:29 clrenard Exp $ i*)
 
 (*i*)
 open Term
 open Sign
 open Environ
-open Reduction
+open Reductionops
 open Evarutil
 (*i*)
 
-val the_conv_x : env -> 'a evar_defs -> constr -> constr -> bool
+val the_conv_x : env -> evar_defs -> constr -> constr -> bool
 
-val the_conv_x_leq : env -> 'a evar_defs -> constr -> constr -> bool
+val the_conv_x_leq : env -> evar_defs -> constr -> constr -> bool
 
 (*i For debugging *)
-val evar_conv_x : env -> 'a evar_defs -> conv_pb -> constr -> constr -> bool
+val evar_conv_x : env -> evar_defs -> conv_pb -> constr -> constr -> bool
 val evar_eqappr_x : 
-  env -> 'a evar_defs ->
+  env -> evar_defs ->
     conv_pb -> constr * constr list -> constr * constr list -> bool
 (*i*)
