@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: indrec.ml,v 1.2 2001/12/13 09:03:06 filliatr Exp $ *)
+(* $Id: indrec.ml,v 1.8 2002/02/14 15:54:00 barras Exp $ *)
 
 open Pp
 open Util
@@ -42,7 +42,6 @@ let mkLambda_string s t c = mkLambda (Name (id_of_string s), t, c)
 
 let mis_make_case_com depopt env sigma (ind,mib,mip) kind =
   let lnamespar = mip.mind_params_ctxt in
-  let nparams = mip.mind_nparams in
   let dep = match depopt with 
     | None -> mip.mind_sort <> (Prop Null)
     | Some d -> d
@@ -59,13 +58,13 @@ let mis_make_case_com depopt env sigma (ind,mib,mip) kind =
   (* mais pas trs joli ... (mais manque get_sort_of  ce niveau) *)
   let env' = push_rel_context lnamespar env in
 
-  let indf = (ind, extended_rel_list 0 lnamespar) in
+  let indf = make_ind_family(ind, extended_rel_list 0 lnamespar) in
   let constrs = get_constructors env indf in
 
   let rec add_branch env k = 
     if k = Array.length mip.mind_consnames then
       let nbprod = k+1 in
-      let indf = (ind,extended_rel_list nbprod lnamespar) in
+      let indf = make_ind_family(ind,extended_rel_list nbprod lnamespar) in
       let lnamesar,_ = get_arity env indf in
       let ci = make_default_case_info env ind in
       it_mkLambda_or_LetIn_name env'
@@ -103,7 +102,7 @@ let mis_make_case_com depopt env sigma (ind,mib,mip) kind =
  * on it with which predicate and which recursive function. 
  *)
 
-let type_rec_branch dep env sigma (vargs,depPvect,decP) tyi cs recargs = 
+let type_rec_branch is_rec dep env sigma (vargs,depPvect,decP) tyi cs recargs = 
   let make_prod = make_prod_dep dep in
   let nparams = List.length vargs in
   let process_pos env depK pk =
@@ -120,7 +119,8 @@ let type_rec_branch dep env sigma (vargs,depPvect,decP) tyi cs recargs =
 	    let (_,realargs) = list_chop nparams largs in
 	    let base = applist (lift i pk,realargs) in
             if depK then 
-	      mkApp (base, [|applist (mkRel (i+1),extended_rel_list 0 sign)|])
+	      Reduction.beta_appvect
+                base [|applist (mkRel (i+1),extended_rel_list 0 sign)|]
             else 
 	      base
       	| _ -> assert false 
@@ -132,12 +132,14 @@ let type_rec_branch dep env sigma (vargs,depPvect,decP) tyi cs recargs =
       | Prod (n,t,c_0) ->
           let (optionpos,rest) = 
 	    match recargs with 
-	      | [] -> None,[] 
-	      | Param _ :: rest -> (None,rest)
-	      | Norec :: rest   -> (None,rest)
-	      | Imbr _ :: rest  -> 
-		  warning "Ignoring recursive call"; (None,rest) 
-	      | Mrec j :: rest -> (depPvect.(j),rest)
+	      | [] -> None,[]
+              | ra::rest ->
+                  (match dest_recarg ra with 
+	            | Mrec j when is_rec -> (depPvect.(j),rest)
+	            | Imbr _  -> 
+		        Options.if_verbose warning "Ignoring recursive call"; 
+		        (None,rest) 
+                    | _ -> (None, rest))
 	  in 
           (match optionpos with 
 	     | None -> 
@@ -166,53 +168,8 @@ let type_rec_branch dep env sigma (vargs,depPvect,decP) tyi cs recargs =
 	let realargs = List.map (fun k -> mkRel (i-k)) (List.rev li) in
         let params = List.map (lift i) vargs in
         let co = applist (mkConstruct cs.cs_cstr,params@realargs) in
-	mkApp (c, [|co|])
+	Reduction.beta_appvect c [|co|]
       else c
-(*
-    let c', largs = whd_stack c in
-    match kind_of_term c' with 
-      | Prod (n,t,c_0) ->
-          let (optionpos,rest) = 
-	    match recargs with 
-	      | [] -> None,[] 
-	      | Param _ :: rest -> (None,rest)
-	      | Norec :: rest   -> (None,rest)
-	      | Imbr _ :: rest  -> 
-		  warning "Ignoring recursive call"; (None,rest) 
-	      | Mrec j :: rest -> (depPvect.(j),rest)
-	  in 
-          (match optionpos with 
-	     | None -> 
-		 make_prod env
-		   (n,t,
-		    process_constr (push_rel (n,None,t) env) (i+1) c_0 rest 
-		      (mkApp (lift 1 co, [|mkRel 1|])))
-             | Some(dep',p) -> 
-		 let nP = lift (i+1+decP) p in
-		 let t_0 = process_pos env dep' nP (lift 1 t) in 
-		 make_prod_dep (dep or dep') env
-                   (n,t,
-		    mkArrow t_0
-		      (process_constr
-			 (push_rel (n,None,t)
-			    (push_rel (Anonymous,None,t_0) env))
-			 (i+2) (lift 1 c_0) rest
-			 (mkApp (lift 2 co, [|mkRel 2|])))))
-      | LetIn (n,b,t,c_0) ->
-	  mkLetIn (n,b,t,
-		   process_constr
-		     (push_rel (n,Some b,t) env)
-		     (i+1) c_0 recargs (lift 1 co))
-
-      | Ind ((_,tyi),_) -> 
-      	  let nP = match depPvect.(tyi) with 
-	    | Some(_,p) -> lift (i+decP) p
-	    | _ -> assert false in
-      	  let (_,realargs) = list_chop nparams largs in
-      	  let base = applist (nP,realargs) in
-          if dep then mkApp (base, [|co|]) else base
-      | _ -> assert false
-*)
   in
   let nhyps = List.length cs.cs_args in
   let nP = match depPvect.(tyi) with 
@@ -245,8 +202,7 @@ let make_rec_branch_arg env sigma (nparams,fvect,decF) f cstr recargs =
   let rec process_constr env i f = function
     | (n,None,t as d)::cprest, recarg::rest -> 
         let optionpos = 
-	  match recarg with 
-            | Param i -> None
+	  match dest_recarg recarg with 
             | Norec   -> None
             | Imbr _  -> None
             | Mrec i  -> fvect.(i)
@@ -290,10 +246,9 @@ let mis_make_indrec env sigma listdepkind (ind,mib,mip) =
             (Array.set depPvec (snd indi) (Some(dep,mkRel k));
              assign (k-1) rest)
     in 
-    assign nrec listdepkind  
-  in 
+    assign nrec listdepkind in 
   let recargsvec =
-    Array.map (fun mip -> mip.mind_listrec) mib.mind_packets in
+    Array.map (fun mip -> mip.mind_recargs) mib.mind_packets in
   let make_one_rec p =
     let makefix nbconstruct =
       let rec mrec i ln ltyp ldef = function
@@ -302,9 +257,10 @@ let mis_make_indrec env sigma listdepkind (ind,mib,mip) =
 	    let nctyi =
               Array.length mipi.mind_consnames in (* nb constructeurs du type *) 
 
-            (* arity in the context P1..P_nrec f1..f_nbconstruct *)
+            (* arity in the context of the fixpoint, i.e.
+                P1..P_nrec f1..f_nbconstruct *)
 	    let args = extended_rel_list (nrec+nbconstruct) lnamespar in
-	    let indf = (indi,args) in
+	    let indf = make_ind_family(indi,args) in
 	    let lnames,_ = get_arity env indf in
 
 	    let nar = mipi.mind_nrealargs in
@@ -312,17 +268,19 @@ let mis_make_indrec env sigma listdepkind (ind,mib,mip) =
 	    let dect = nar+nrec+nbconstruct in
 	    let vecfi = rel_vect (dect+1-i-nctyi) nctyi in
 
-	    let args = extended_rel_list (decf+1) lnamespar in
-	    let constrs = get_constructors env (indi,args) in
+            (* constructors in context of the Cases expr, i.e.
+                P1..P_nrec f1..f_nbconstruct F_1..F_nrec a_1..a_nar x:I *)
+	    let args' = extended_rel_list (decf+1) lnamespar in
+            let indf' = make_ind_family(indi,args') in
+	    let constrs = get_constructors env indf' in
 	    let branches = 
 	      array_map3
 		(make_rec_branch_arg env sigma (nparams,depPvec,nar+1))
-                vecfi constrs recargsvec.(tyi) in
+                vecfi constrs
+                (dest_subterms recargsvec.(tyi)) in
 	    let j = (match depPvec.(tyi) with 
 		       | Some (_,c) when isRel c -> destRel c 
 		       | _ -> assert false) in
-	    let args = extended_rel_list (nrec+nbconstruct) lnamespar in
-	    let indf = (indi,args) in
 	    let deftyi = 
 	      it_mkLambda_or_LetIn_name env
 		(lambda_create env
@@ -364,12 +322,13 @@ let mis_make_indrec env sigma listdepkind (ind,mib,mip) =
 	    if j = nconstr then 
 	      make_branch env (i+j) rest 
 	    else 
-	      let recarg = recargsvec.(tyi).(j) in
+	      let recarg = (dest_subterms recargsvec.(tyi)).(j) in
 	      let vargs = extended_rel_list (nrec+i+j) lnamespar in
 	      let indf = (indi, vargs) in
 	      let cs = get_constructor (indi,mibi,mipi,vargs) (j+1) in
 	      let p_0 =
-		type_rec_branch dep env sigma (vargs,depPvec,i+j) tyi cs recarg
+		type_rec_branch
+                  true dep env sigma (vargs,depPvec,i+j) tyi cs recarg
 	      in 
 	      mkLambda_string "f" p_0
 		(onerec (push_rel (Anonymous,None,p_0) env) (j+1))
@@ -389,7 +348,8 @@ let mis_make_indrec env sigma listdepkind (ind,mib,mip) =
     let (indi,mibi,mipi,dep,kind) = List.nth listdepkind p in
     let env' = push_rel_context lnamespar env in
     if mis_is_recursive_subset
-      (List.map (fun (indi,_,_,_,_) -> snd indi) listdepkind) mipi
+      (List.map (fun (indi,_,_,_,_) -> snd indi) listdepkind)
+      mipi.mind_recargs
     then 
       it_mkLambda_or_LetIn_name env (put_arity env' 0 listdepkind) lnamespar
     else 
@@ -436,6 +396,24 @@ let instanciate_indrec_scheme sort =
   in
   drec
 
+(* Change the sort in the type of an inductive definition, builds the corresponding eta-expanded term *)
+let instanciate_type_indrec_scheme sort npars term =
+  let rec drec np elim =
+    match kind_of_term elim with
+      | Prod (n,t,c) -> 
+	  if np = 0 then 
+            let t' = change_sort_arity sort t in
+            mkProd (n, t', c),
+            mkLambda (n, t', mkApp(term,Termops.rel_vect 0 (npars+1)))
+	  else 
+            let c',term' = drec (np-1) c in
+	    mkProd (n, t, c'), mkLambda (n, t, term')
+      | LetIn (n,b,t,c) -> let c',term' = drec np c in
+           mkLetIn (n,b,t,c'), mkLetIn (n,b,t,term') 
+      | _ -> anomaly "instanciate_type_indrec_scheme: wrong elimination type"
+  in
+  drec npars
+
 (**********************************************************************)
 (* Interface to build complex Scheme *)
 
@@ -478,43 +456,38 @@ let build_indrec env sigma ind =
 (* To handle old Case/Match syntax in Pretyping                       *)
 
 (***********************************)
-(* To interpret the Match operator *)
+(* To interpret Case and Match operators *)
 
-(* TODO: check that we can drop universe constraints ? *)
-let type_mutind_rec env sigma (IndType (indf,realargs) as indt) pj c = 
+let type_rec_branches recursive env sigma indt pj c = 
+  let IndType (indf,realargs) = indt in
   let p = pj.uj_val in
   let (ind,params) = dest_ind_family indf in
-  let tyi = snd ind in
   let (mib,mip) = lookup_mind_specif env ind in
-  if mis_is_recursive_subset [tyi] mip then
-    let (dep,_) = find_case_dep_nparams env (c,pj) indf in 
-    let init_depPvec i = if i = tyi then Some(dep,p) else None in
-    let depPvec = Array.init mib.mind_ntypes init_depPvec in
-    let vargs = Array.of_list params in
-    let constructors = get_constructors env indf in
-    let recargs = mip.mind_listrec in
-    let lft = array_map2 (type_rec_branch dep env sigma (params,depPvec,0) tyi)
-                constructors recargs in
-    (lft,
-     if dep then applist(p,realargs@[c]) 
-     else applist(p,realargs) )
-  else 
+  let recargs = mip.mind_recargs in
+  let tyi = snd ind in
+  let is_rec = recursive && mis_is_recursive_subset [tyi] recargs in
+  let dep = is_dependent_elimination env pj.uj_type indf in 
+  let init_depPvec i = if i = tyi then Some(dep,p) else None in
+  let depPvec = Array.init mib.mind_ntypes init_depPvec in
+  let vargs = Array.of_list params in
+  let constructors = get_constructors env indf in
+  let lft =
+    array_map2
+      (type_rec_branch recursive dep env sigma (params,depPvec,0) tyi)
+      constructors (dest_subterms recargs) in
+  (lft,
+   if dep then Reduction.beta_appvect p (Array.of_list (realargs@[c])) 
+   else Reduction.beta_appvect p (Array.of_list realargs))
+(* Non recursive case. Pb: does not deal with unification
     let (p,ra,_) = type_case_branches env (ind,params@realargs) pj c in
     (p,ra)
-
-let type_rec_branches recursive env sigma indt pj c =
-  if recursive then 
-    type_mutind_rec env sigma indt pj c
-  else 
-    let IndType((ind,params),rargs) = indt in
-    let (p,ra,_) = type_case_branches env (ind,params@rargs) pj c in
-    (p,ra)
-
+*)
 
 (*s Eliminations. *)
 
-let eliminations =
-  [ (InProp,"_ind") ; (InSet,"_rec") ; (InType,"_rect") ]
+let type_suff = "_rect"
+
+let non_type_eliminations = [ (InProp,"_ind") ; (InSet,"_rec")]
 
 let elimination_suffix = function
   | InProp -> "_ind"
@@ -526,14 +499,14 @@ let make_elimination_ident id s = add_suffix id (elimination_suffix s)
 let declare_one_elimination ind =
   let (mib,mip) = Global.lookup_inductive ind in 
   let mindstr = string_of_id mip.mind_typename in
-  let declare na c =
-    let _ = Declare.declare_constant (id_of_string na)
+  let declare na c t =
+    let sp = Declare.declare_constant (id_of_string na)
       (ConstantEntry
         { const_entry_body = c;
-          const_entry_type = None;
+          const_entry_type = t;
           const_entry_opaque = false }, 
        NeverDischarge) in
-    Options.if_verbose ppnl (str na ++ str " is defined")
+    Options.if_verbose ppnl (str na ++ str " is defined"); sp
   in
   let env = Global.env () in
   let sigma = Evd.empty in
@@ -541,11 +514,24 @@ let declare_one_elimination ind =
   let npars = mip.mind_nparams in
   let make_elim s = instanciate_indrec_scheme s npars elim_scheme in
   let kelim = mip.mind_kelim in
-  List.iter
+(* in case the inductive has a type elimination, generates only one induction scheme, 
+   the other ones share the same code with the apropriate type *)
+   if List.mem InType kelim then
+    let cte =
+      declare (mindstr^type_suff) (make_elim (new_sort_in_family InType)) None
+    in let c = mkConst cte and t = constant_type (Global.env ()) cte
+    in List.iter  
+	 (fun (sort,suff) -> 
+            let (t',c') = instanciate_type_indrec_scheme (new_sort_in_family sort) npars c t
+            in let _ = declare (mindstr^suff) c' (Some t')
+            in ())
+	 non_type_eliminations
+   else (* Impredicative or logical inductive definition *)
+     List.iter
     (fun (sort,suff) -> 
        if List.mem sort kelim then
-	 declare (mindstr^suff) (make_elim (new_sort_in_family sort)))
-    eliminations
+	 let _ = declare (mindstr^suff) (make_elim (new_sort_in_family sort)) None in ())
+     non_type_eliminations
 
 let declare_eliminations sp =
   let mib = Global.lookup_mind sp in
@@ -575,9 +561,9 @@ let lookup_eliminator ind_sp s =
     try construct_reference env id
     with Not_found ->
       errorlabstrm "default_elim"
-	(str "Cannot find the elimination combinator :" ++
+	(str "Cannot find the elimination combinator " ++
            pr_id id ++ spc () ++
-	   str "The elimination of the inductive definition :" ++
+	   str "The elimination of the inductive definition " ++
            pr_id base ++ spc () ++ str "on sort " ++ 
-           spc () ++ print_sort (new_sort_in_family s) ++
+           spc () ++ print_sort_family s ++
 	   str " is probably not allowed")
