@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: pretyping.ml,v 1.89 2001/12/18 09:15:35 herbelin Exp $ *)
+(* $Id: pretyping.ml,v 1.96 2002/04/11 14:21:09 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -43,20 +43,20 @@ let lift_context n l =
 
 let transform_rec loc env sigma (pj,c,lf) indt = 
   let p = pj.uj_val in
-  let ((ind,params) as indf,realargs) = dest_ind_type indt in
+  let (indf,realargs) = dest_ind_type indt in
+  let (ind,params) = dest_ind_family indf in
   let (mib,mip) = lookup_mind_specif env ind in
+  let recargs = mip.mind_recargs in
   let mI = mkInd ind in
-  let recargs = mip.mind_listrec in
-  let tyi = snd ind in
   let ci = make_default_case_info env ind in
   let nconstr = Array.length mip.mind_consnames in
   if Array.length lf <> nconstr then 
     (let cj = {uj_val=c; uj_type=mkAppliedInd indt} in
      error_number_branches_loc loc env sigma cj nconstr);
-  if mis_is_recursive_subset [tyi] mip then
-    let (dep,_) =
-      find_case_dep_nparams env
-        (nf_evar sigma c, j_nf_evar sigma pj) indf in 
+  let tyi = snd ind in
+  if mis_is_recursive_subset [tyi] recargs then
+    let dep =
+      is_dependent_elimination env (nf_evar sigma pj.uj_type) indf in 
     let init_depFvec i = if i = tyi then Some(dep,mkRel 1) else None in
     let depFvec = Array.init mib.mind_ntypes init_depFvec in
     (* build now the fixpoint *)
@@ -71,7 +71,7 @@ let transform_rec loc env sigma (pj,c,lf) indt =
              (Indrec.make_rec_branch_arg env sigma
                 (nparams,depFvec,nar+1)
                 f t reca))
-        (Array.map (lift (nar+2)) lf) constrs recargs 
+        (Array.map (lift (nar+2)) lf) constrs (dest_subterms recargs) 
     in 
     let deffix = 
       it_mkLambda_or_LetIn_name env
@@ -132,6 +132,23 @@ let evar_type_fixpoint loc env isevars lna lar vdefj =
             i lna vdefj lar
       done
 
+let error_unsolvable_implicit (loc,kind) =
+  let message = match kind with
+    | QuestionMark -> str "Cannot infer a term for this placeholder"
+    | CasesType ->
+	str "Cannot infer the type of this pattern-matching problem"
+    | AbstractionType (Name id) ->
+	str "Cannot infer a type for " ++ Nameops.pr_id id
+    | AbstractionType Anonymous ->
+	str "Cannot infer a type of this anonymous abstraction"
+    | ImplicitArg (c,n) ->
+	str "Cannot infer the " ++ pr_ord n ++
+	str " implicit argument of " ++ Nametab.pr_global_env (Global.env()) c
+    | InternalHole ->
+        str "Cannot infer a term for an internal placeholder"
+  in
+    user_err_loc (loc,"pretype",message)
+
 let check_branches_message loc env isevars c (explft,lft) = 
   for i = 0 to Array.length explft - 1 do
     if not (the_conv_x_leq env isevars lft.(i) explft.(i)) then 
@@ -213,14 +230,9 @@ let rec pretype tycon env isevars lvar lmeta = function
   | RHole loc ->
       if !compter then nbimpl:=!nbimpl+1;
       (match tycon with
-	 | Some ty -> { uj_val = new_isevar isevars env ty; uj_type = ty }
-	 | None ->
-	     (match loc with
-		  None -> error "There is an implicit argument I cannot solve"
-		| Some loc -> 
-		    user_err_loc
-		      (loc,"pretype",
-		       (str "Cannot infer a term for this placeholder"))))
+	 | Some ty ->
+	     { uj_val = new_isevar isevars env loc ty; uj_type = ty }
+	 | None -> error_unsolvable_implicit loc)
 
   | RRec (loc,fixkind,names,lar,vdef) ->
       let larj =
@@ -321,11 +333,18 @@ let rec pretype tycon env isevars lvar lmeta = function
   | ROldCase (loc,isrec,po,c,lf) ->
       let cj = pretype empty_tycon env isevars lvar lmeta c in
       let (IndType (indf,realargs) as indt) = 
-	try find_rectype env Evd.empty (nf_evar (evars_of isevars) cj.uj_type)
-	with Induc ->
-          error_case_not_inductive_loc loc env (evars_of isevars) cj in
-      let pj = match po with
-	| Some p -> pretype empty_tycon env isevars lvar lmeta p
+	try find_rectype env (evars_of isevars) cj.uj_type
+	with Not_found ->
+	  let cloc = loc_of_rawconstr c in
+	  error_case_not_inductive_loc cloc env (evars_of isevars) cj in
+      let (dep,pj) = match po with
+	| Some p ->
+            let pj = pretype empty_tycon env isevars lvar lmeta p in
+            let dep = is_dependent_elimination env pj.uj_type indf in
+            let ar =
+              arity_of_case_predicate env indf dep (Type (new_univ())) in
+            let _ = the_conv_x_leq env isevars pj.uj_type ar in
+            (dep, pj)
 	| None -> 
             (* get type information from type of branches *)
 	    let expbr = Cases.branch_scheme env isevars isrec indf in
@@ -337,7 +356,8 @@ let rec pretype tycon env isevars lvar lmeta = function
                 (* may be Type while Prop or Set would be expected *)
 	        match tycon with
 		  | Some pred ->
-                      Retyping.get_judgment_of env (evars_of isevars) pred
+                      (false,
+                       Retyping.get_judgment_of env (evars_of isevars) pred)
 	          | None ->
                       let sigma = evars_of isevars in
                       error_cant_find_case_type_loc loc env sigma cj.uj_val
@@ -355,13 +375,11 @@ let rec pretype tycon env isevars lvar lmeta = function
                       Retyping.get_type_of env (evars_of isevars) pred in
 		    let pj = { uj_val = pred; uj_type = pty } in
                     let _ = option_app (the_conv_x_leq env isevars pred) tycon
-                    in pj
+                    in (false,pj)
 		with Cases.NotInferable _ -> findtype (i+1) in
 	    findtype 0 in
       let pj = j_nf_evar (evars_of isevars) pj in
 
-      let (dep,_) = find_case_dep_nparams env (cj.uj_val,pj) indf in
-
       let pj =
 	if dep then pj else
 	  let n = List.length realargs in
@@ -436,7 +454,8 @@ and pretype_type valcon env isevars lvar lmeta = function
 	       utj_type = Retyping.get_sort_of env (evars_of isevars) v }
 	 | None ->
 	     let s = new_Type_sort () in
-	     { utj_val = new_isevar isevars env (mkSort s); utj_type = s})
+	     { utj_val = new_isevar isevars env loc (mkSort s);
+	       utj_type = s})
   | c ->
       let j = pretype empty_tycon env isevars lvar lmeta c in
       let tj = inh_coerce_to_sort env isevars j in
@@ -466,22 +485,18 @@ let unsafe_infer_type valcon isevars env lvar lmeta constr =
  *)
 (* assumes the defined existentials have been replaced in c (should be
    done in unsafe_infer and unsafe_infer_type) *)
-let check_evars fail_evar initial_sigma sigma c =
-  let metamap = ref [] in
+let check_evars fail_evar initial_sigma isevars c =
+  let sigma = evars_of isevars in
   let rec proc_rec c =
     match kind_of_term c with
       | Evar (ev,args as k) ->
           assert (Evd.in_dom sigma ev);
 	  if not (Evd.in_dom initial_sigma ev) then
 	    (if fail_evar then
-	      errorlabstrm "whd_ise"
-		(str"There is an unknown subterm I cannot solve")
-	    else (* try to avoid duplication *)
-              (if not (List.exists (fun (k',_) -> k=k') !metamap) then
-	        metamap := (k, existential_type sigma k) :: !metamap))
+	       error_unsolvable_implicit (evar_source ev isevars))
       | _ -> iter_constr proc_rec c      
   in
-  (proc_rec c; !metamap)
+  proc_rec c
 
 (* TODO: comment faire remonter l'information si le typage a resolu des
        variables du sigma original. il faudrait que la fonction de typage
@@ -489,15 +504,13 @@ let check_evars fail_evar initial_sigma sigma c =
 *)
 
 (* constr with holes *)
-type open_constr = (existential * types) list * constr
+type open_constr = evar_map * constr
 
 let ise_resolve_casted_gen fail_evar sigma env lvar lmeta typ c =
   let isevars = create_evar_defs sigma in
   let j = unsafe_infer (mk_tycon typ) isevars env lvar lmeta c in
-  let metamap =
-    check_evars fail_evar sigma (evars_of isevars)
-      (mkCast(j.uj_val,j.uj_type)) in
-  (metamap, j)
+  check_evars fail_evar sigma isevars (mkCast(j.uj_val,j.uj_type));
+  (evars_of isevars, j)
 
 let ise_resolve_casted sigma env typ c =
   ise_resolve_casted_gen true sigma env [] [] typ c
@@ -509,17 +522,14 @@ let ise_infer_gen fail_evar sigma env lvar lmeta exptyp c =
   let tycon = match exptyp with None -> empty_tycon | Some t -> mk_tycon t in
   let isevars = create_evar_defs sigma in
   let j = unsafe_infer tycon isevars env lvar lmeta c in
-  let metamap =
-    check_evars fail_evar sigma (evars_of isevars)
-      (mkCast(j.uj_val,j.uj_type)) in
-  (metamap, j)
+  check_evars fail_evar sigma isevars (mkCast(j.uj_val,j.uj_type));
+  (evars_of isevars, j)
 
 let ise_infer_type_gen fail_evar sigma env lvar lmeta c =
   let isevars = create_evar_defs sigma in
   let tj = unsafe_infer_type empty_valcon isevars env lvar lmeta c in
-  let metamap =
-    check_evars fail_evar sigma (evars_of isevars) tj.utj_val in
-  (metamap, tj)
+  check_evars fail_evar sigma isevars tj.utj_val;
+  (evars_of isevars, tj)
 
 type meta_map = (int * unsafe_judgment) list
 type var_map = (identifier * unsafe_judgment) list
