@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: rawterm.ml,v 1.21 2001/11/05 16:48:03 barras Exp $ *)
+(* $Id: rawterm.ml,v 1.23 2002/04/11 14:21:10 herbelin Exp $ *)
 
 (*i*)
 open Util
@@ -32,6 +32,13 @@ type fix_kind = RFix of (int array * int) | RCoFix of int
 
 type binder_kind = BProd | BLambda | BLetIn
 
+type hole_kind =
+  | ImplicitArg of global_reference * int
+  | AbstractionType of name
+  | QuestionMark
+  | CasesType
+  | InternalHole
+
 type 'ctxt reference =
   | RConst of constant * 'ctxt
   | RInd of inductive * 'ctxt
@@ -56,7 +63,7 @@ type rawconstr =
   | RRec of loc * fix_kind * identifier array * 
       rawconstr array * rawconstr array
   | RSort of loc * rawsort
-  | RHole of loc option
+  | RHole of (loc * hole_kind)
   | RCast of loc * rawconstr * rawconstr
   | RDynamic of loc * Dyn.t
 
@@ -69,7 +76,6 @@ type rawconstr =
    dans le contexte servant  typer les body ???]
 
    - boolean in POldCase means it is recursive
-   - option in PHole tell if the "?" was apparent or has been implicitely added
 i*)
 
 let dummy_loc = (0,0)
@@ -87,8 +93,7 @@ let loc_of_rawconstr = function
   | ROldCase (loc,_,_,_,_) -> loc
   | RRec (loc,_,_,_,_) -> loc
   | RSort (loc,_) -> loc
-  | RHole (Some loc) -> loc
-  | RHole (None) -> dummy_loc
+  | RHole (loc,_) -> loc
   | RCast (loc,_,_) -> loc
   | RDynamic (loc,_) -> loc
 
@@ -105,7 +110,7 @@ let set_loc_of_rawconstr loc = function
   | ROldCase (_,a,b,c,d) -> ROldCase (loc,a,b,c,d) 
   | RRec (_,a,b,c,d) -> RRec (loc,a,b,c,d) 
   | RSort (_,a)      -> RSort (loc,a) 
-  | RHole _          -> RHole (Some loc)
+  | RHole (_,a)      -> RHole (loc,a)
   | RCast (_,a,b)    -> RCast (loc,a,b) 
   | RDynamic (_,d)   -> RDynamic (loc,d)
 
