@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: rawterm.mli,v 1.26 2001/11/05 16:48:03 barras Exp $ i*)
+(*i $Id: rawterm.mli,v 1.28 2002/04/11 14:21:10 herbelin Exp $ i*)
 
 (*i*)
 open Names
@@ -31,6 +31,13 @@ type fix_kind = RFix of (int array * int) | RCoFix of int
 
 type binder_kind = BProd | BLambda | BLetIn
 
+type hole_kind =
+  | ImplicitArg of global_reference * int
+  | AbstractionType of name
+  | QuestionMark
+  | CasesType
+  | InternalHole
+
 type 'ctxt reference =
   | RConst of constant * 'ctxt
   | RInd of inductive * 'ctxt
@@ -54,7 +61,7 @@ type rawconstr =
   | RRec of loc * fix_kind * identifier array * 
       rawconstr array * rawconstr array
   | RSort of loc * rawsort
-  | RHole of loc option
+  | RHole of (loc * hole_kind)
   | RCast of loc * rawconstr * rawconstr
   | RDynamic of loc * Dyn.t
 
