@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: typing.ml,v 1.28 2001/11/12 12:38:04 barras Exp $ *)
+(* $Id: typing.ml,v 1.30 2002/03/28 10:17:11 barras Exp $ *)
 
 open Util
 open Names
@@ -106,17 +106,17 @@ let rec execute mf env sigma cstr =
         let varj = type_judgment env sigma j in
 	let env1 = push_rel (name,None,varj.utj_val) env in
         let j' = execute mf env1 sigma c2 in
-        let varj' = type_judgment env sigma j' in
-	judge_of_product env1 name varj varj'
+        let varj' = type_judgment env1 sigma j' in
+	judge_of_product env name varj varj'
 
      | LetIn (name,c1,c2,c3) ->
         let j1 = execute mf env sigma c1 in
         let j2 = execute mf env sigma c2 in
         let j2 = type_judgment env sigma j2 in
+        let _ = conv_leq env sigma j1.uj_type j2.utj_val in
         let env1 = push_rel (name,Some j1.uj_val,j2.utj_val) env in
         let j3 = execute mf env1 sigma c3 in
-        let (j,_) = judge_of_letin env name j1 j2 j3 in
-        j
+        judge_of_letin env name j1 j2 j3
   
     | Cast (c,t) ->
         let cj = execute mf env sigma c in
