@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: clenv.ml,v 1.61 2001/12/17 19:35:34 herbelin Exp $ *)
+(* $Id: clenv.ml,v 1.80 2002/05/14 15:17:06 barras Exp $ *)
 
 open Pp
 open Util
@@ -25,26 +25,47 @@ open Reductionops
 open Tacmach
 open Evar_refiner
 
+(* if lname_typ is [xn,An;..;x1,A1] and l is a list of terms,
+   gives [x1:A1]..[xn:An]c' such that c converts to ([x1:A1]..[xn:An]c' l) *)
+
+let abstract_scheme env c l lname_typ =
+  List.fold_left2 
+    (fun t (locc,a) (na,_,ta) ->
+       if occur_meta ta then error "cannot find a type for the generalisation"
+       else if occur_meta a then lambda_name env (na,ta,t)
+       else lambda_name env (na,ta,subst_term_occ env locc a t))
+    c
+    (List.rev l)
+    lname_typ
+
+let abstract_list_all env sigma typ c l =
+  let ctxt,_ = decomp_n_prod env sigma (List.length l) typ in
+  let p = abstract_scheme env c (List.map (function a -> [],a) l) ctxt in 
+  try 
+    if is_conv_leq env sigma (Typing.type_of env sigma p) typ then p
+    else error "abstract_list_all"
+  with UserError _ ->
+    raise (RefinerError (CannotGeneralize typ))
 
 (* Generator of metavariables *)
 let meta_ctr=ref 0;;
 
 let new_meta ()=incr meta_ctr;!meta_ctr;;
 
-(* replaces a mapping of existentials into a mapping of metas. *)
-let exist_to_meta (emap, c) =
-  let subst = ref [] in
-  let mmap = ref [] in
-  let add_binding (e,ty) =
+(* replaces a mapping of existentials into a mapping of metas.
+   Problem if an evar appears in the type of another one (pops anomaly) *)
+let exist_to_meta sigma (emap, c) =
+  let metamap = ref [] in
+  let change_exist evar =
+    let ty = nf_betaiota (nf_evar emap (existential_type emap evar)) in
     let n = new_meta() in
-    subst := (e, mkMeta n) :: !subst;
-    mmap := (n, ty) :: !mmap in
-  List.iter add_binding emap;
+    metamap := (n, ty) :: !metamap;
+    mkMeta n in
   let rec replace c =
     match kind_of_term c with
-        Evar k -> List.assoc k !subst
+        Evar (k,_ as ev) when not (Evd.in_dom sigma k) -> change_exist ev
       | _ -> map_constr replace c in
-  (!mmap, replace c)
+  (!metamap, replace c)
 
 type 'a freelisted = {
   rebus : 'a;
@@ -61,8 +82,6 @@ type 'a clausenv = {
   env : clbinding Intmap.t;
   hook : 'a }
 
-type wc = named_context sigma
-
 let applyHead n c wc = 
   let rec apprec n c cty wc =
     if n = 0 then 
@@ -103,92 +122,69 @@ let mimick_evar hdc nargs sp wc =
    Attention : pas d'unification entre les diffrences instances d'une
    mme meta ou evar, il peut rester des doublons *)
 
-let unify_0 cv_pb mc wc m n =
+(* Unification order: *)
+(* Left to right: unifies first argument and then the other arguments *)
+let unify_l2r x = List.rev x
+(* Right to left: unifies last argument and then the other arguments *)
+let unify_r2l x = x
+
+let sort_eqns = unify_r2l
+
+
+let unify_0 cv_pb wc m n =
   let env = w_env wc
   and sigma = w_Underlying wc in
+  let trivial_unify pb substn m n =
+    if (not(occur_meta m)) & is_fconv pb env sigma m n then substn
+    else error_cannot_unify (m,n) in
   let rec unirec_rec pb ((metasubst,evarsubst) as substn) m n =
     let cM = Evarutil.whd_castappevar sigma m
     and cN = Evarutil.whd_castappevar sigma n in 
-    try 
-      match (kind_of_term cM,kind_of_term cN) with
-	| Cast (c,_), _ -> unirec_rec pb substn c cN
-	| _, Cast (c,_) -> unirec_rec pb substn cM c
-	| Meta k1, Meta k2 ->
-	    if k1 < k2 then (k1,cN)::metasubst,evarsubst
-	    else if k1 = k2 then substn
-	    else (k2,cM)::metasubst,evarsubst
-	| Meta k, _    -> (k,cN)::metasubst,evarsubst
-	| _, Meta k    -> (k,cM)::metasubst,evarsubst
-	| Lambda (_,t1,c1), Lambda (_,t2,c2) ->
-	    unirec_rec CONV (unirec_rec CONV substn t1 t2) c1 c2
-	| Prod (_,t1,c1), Prod (_,t2,c2) ->
-	    unirec_rec pb (unirec_rec CONV substn t1 t2) c1 c2
+    match (kind_of_term cM,kind_of_term cN) with
+      | Meta k1, Meta k2 ->
+	  if k1 < k2 then (k1,cN)::metasubst,evarsubst
+	  else if k1 = k2 then substn
+	  else (k2,cM)::metasubst,evarsubst
+      | Meta k, _    -> (k,cN)::metasubst,evarsubst
+      | _, Meta k    -> (k,cM)::metasubst,evarsubst
+      | Evar _, _ -> metasubst,((cM,cN)::evarsubst)
+      | _, Evar _ -> metasubst,((cN,cM)::evarsubst)
 
-	| App (f1,l1), App (f2,l2) ->
-	    let len1 = Array.length l1
-	    and len2 = Array.length l2 in
-	    if len1 = len2 then
-              array_fold_left2 (unirec_rec CONV)
-		(unirec_rec CONV substn f1 f2) l1 l2
+      | Lambda (_,t1,c1), Lambda (_,t2,c2) ->
+	  unirec_rec CONV (unirec_rec CONV substn t1 t2) c1 c2
+      | Prod (_,t1,c1), Prod (_,t2,c2) ->
+	  unirec_rec pb (unirec_rec CONV substn t1 t2) c1 c2
+      | LetIn (_,b,_,c), _ -> unirec_rec pb substn (subst1 b c) cN
+      | _, LetIn (_,b,_,c) -> unirec_rec pb substn cM (subst1 b c)
+
+      | App (f1,l1), App (f2,l2) ->
+	  let len1 = Array.length l1
+	  and len2 = Array.length l2 in
+          let (f1,l1,f2,l2) =
+	    if len1 = len2 then (f1,l1,f2,l2)
 	    else if len1 < len2 then
               let extras,restl2 = array_chop (len2-len1) l2 in 
-	      array_fold_left2 (unirec_rec CONV)
-		(unirec_rec CONV substn f1 (appvect (f2,extras)))
-                l1 restl2
+              (f1, l1, appvect (f2,extras), restl2)
 	    else 
 	      let extras,restl1 = array_chop (len1-len2) l1 in 
-	      array_fold_left2 (unirec_rec CONV)
-		(unirec_rec CONV substn (appvect (f1,extras)) f2)
-                restl1 l2
-		
-	| Case (_,p1,c1,cl1), Case (_,p2,c2,cl2) ->
+              (appvect (f1,extras), restl1, f2, l2) in
+          (try
             array_fold_left2 (unirec_rec CONV)
-	      (unirec_rec CONV (unirec_rec CONV substn p1 p2) c1 c2) cl1 cl2
-		
-	| Construct _, Construct _ ->
-	    if is_conv env sigma cM cN then
-	      substn
-	    else
-	      error_cannot_unify (m,n)
-	      
-	| Ind _, Ind _ ->
-	    if is_conv env sigma  cM cN then
-	      substn
-	    else
-	      error_cannot_unify (m,n)
-	      
-	| Evar _, _ ->
-	    metasubst,((cM,cN)::evarsubst)
-	| _, Evar _ ->
-	    metasubst,((cN,cM)::evarsubst)
+	      (unirec_rec CONV substn f1 f2) l1 l2
+	  with ex when catchable_exception ex ->
+            trivial_unify pb substn cM cN)
+      | Case (_,p1,c1,cl1), Case (_,p2,c2,cl2) ->
+          array_fold_left2 (unirec_rec CONV)
+	    (unirec_rec CONV (unirec_rec CONV substn p1 p2) c1 c2) cl1 cl2
 
-	| (Const _ | Var _ | Rel _), _ ->
-	    if is_conv env sigma cM cN then
-	      substn
-	    else 
-	      error_cannot_unify (m,n)
-		
-	| _, (Const _ | Var _| Rel _) ->
-	    if (not (occur_meta cM)) & is_conv env sigma cM cN then 
-	      substn
-	    else 
-	      error_cannot_unify (m,n)
+      | _ -> trivial_unify pb substn cM cN
 
-	| LetIn (_,b,_,c), _ -> unirec_rec pb substn (subst1 b c) cN
-		
-	| _ -> error_cannot_unify (m,n)
-	      
-    with ex when catchable_exception ex ->
-      if (not(occur_meta cM)) & is_fconv pb env sigma cM cN then 
-	substn
-      else 
-	raise ex
-
-    in
-    if (not(occur_meta m)) & is_fconv cv_pb env sigma m n then 
-      (mc,[])
-    else 
-      unirec_rec cv_pb (mc,[]) m n
+  in
+  if (not(occur_meta m)) & is_fconv cv_pb env sigma m n then 
+    ([],[])
+  else 
+    let (mc,ec) = unirec_rec cv_pb ([],[]) m n in
+    (sort_eqns mc, sort_eqns ec)
 
 
 (* Unification
@@ -240,8 +236,8 @@ let unify_0 cv_pb mc wc m n =
  * close it off.  But this might not always work,
  * since other metavars might also need to be resolved. *)
 
-let rec w_Unify cv_pb m n mc wc =
-  let (mc',ec') = unify_0 cv_pb mc wc m n in 
+let rec w_Unify cv_pb m n wc =
+  let (mc',ec') = unify_0 cv_pb wc m n in 
   w_resrec mc' ec' wc
 
 and w_resrec metas evars wc =
@@ -258,8 +254,8 @@ and w_resrec metas evars wc =
 
 	| Evar (evn,_) ->
 	    if w_defined_evar wc evn then
-	      let (wc',metas') = w_Unify CONV rhs lhs metas wc in 
-	      w_resrec metas' t wc'
+	      let (wc',metas') = w_Unify CONV rhs lhs wc in 
+	      w_resrec (metas@metas') t wc'
 	    else
 	      (try 
 		 w_resrec metas t (w_Define evn rhs wc)
@@ -280,7 +276,7 @@ and w_resrec metas evars wc =
 (* let unifyTerms m n = walking (fun wc -> fst (w_Unify CONV m n [] wc)) *)
 let unifyTerms m n gls = 
   tclIDTAC {it = gls.it; 
-	    sigma = (get_gc (fst (w_Unify CONV m n [] (Refiner.project_with_focus gls))))}
+	    sigma = (get_gc (fst (w_Unify CONV m n (Refiner.project_with_focus gls))))}
 
 let unify m gls =
   let n = pf_concl gls in unifyTerms m n gls
@@ -340,7 +336,7 @@ let mk_clenv wc cty =
 let clenv_environments bound c =
   let rec clrec (ne,e,metas) n c =
     match n, kind_of_term c with
-      | (0, _) -> (ne, e, List.rev metas, c)
+      | (Some 0, _) -> (ne, e, List.rev metas, c)
       | (n, Cast (c,_)) -> clrec (ne,e,metas) n c
       | (n, Prod (na,c1,c2)) ->
 	  let mv = new_meta () in
@@ -360,9 +356,10 @@ let clenv_environments bound c =
 	      ne 
 	  in
 	  let e' = Intmap.add mv (Cltyp (mk_freelisted c1)) e in 
-	  clrec (ne',e', (mkMeta mv)::metas) (n-1)
+	  clrec (ne',e', (mkMeta mv)::metas) (option_app ((+) (-1)) n)
 	    (if dep then (subst1 (mkMeta mv) c2) else c2)
-      | (n, LetIn (na,b,_,c)) -> clrec (ne,e,metas) (n-1) (subst1 b c)
+      | (n, LetIn (na,b,_,c)) ->
+	  clrec (ne,e,metas) (option_app ((+) (-1)) n) (subst1 b c)
       | (n, _) -> (ne, e, List.rev metas, c)
   in 
   clrec (Intmap.empty,Intmap.empty,[]) bound c
@@ -375,13 +372,7 @@ let mk_clenv_from_n wc n (c,cty) =
     env = env;
     hook = wc }
 
-let mk_clenv_from wc (c,cty) =
-  let (namenv,env,args,concl) = clenv_environments (-1) cty in
-  { templval = mk_freelisted (match args with [] -> c | _ -> applist (c,args));
-    templtyp = mk_freelisted concl;
-    namenv = namenv;
-    env = env;
-    hook = wc }
+let mk_clenv_from wc = mk_clenv_from_n wc None
 
 let connect_clenv wc clenv =
   { templval = clenv.templval;
@@ -416,8 +407,6 @@ let mk_clenv_rename_hnf_constr_type_of wc t =
 
 let mk_clenv_type_of wc t = mk_clenv_from wc (t,w_type_of wc t)
 			      
-let mk_clenv_printable_type_of = mk_clenv_type_of
-				   
 let clenv_assign mv rhs clenv =
   let rhs_fls = mk_freelisted rhs in 
   if intset_exists (mentions clenv mv) rhs_fls.freemetas then
@@ -571,7 +560,9 @@ let clenv_instance_type_of ce c =
    or in evars, possibly generating new unification problems; if [b]
    is true, unification of types of metas is required *)
 
-let clenv_merge with_types =
+let clenv_merge with_types metas evars clenv =
+  let ty_metas = ref [] in
+  let ty_evars = ref [] in
   let rec clenv_resrec metas evars clenv =
     match (evars,metas) with
       | ([], []) -> clenv
@@ -586,7 +577,7 @@ let clenv_merge with_types =
 
 	  | Evar (evn,_) ->
     	      if w_defined_evar clenv.hook evn then
-		let (metas',evars') = unify_0 CONV [] clenv.hook rhs lhs in
+		let (metas',evars') = unify_0 CONV clenv.hook rhs lhs in
 		clenv_resrec (metas'@metas) (evars'@t) clenv
     	      else begin
 		let rhs' = 
@@ -611,21 +602,28 @@ let clenv_merge with_types =
       | ([], (mv,n)::t) ->
     	  if clenv_defined clenv mv then
             let (metas',evars') =
-              unify_0 CONV [] clenv.hook (clenv_value clenv mv).rebus n in
+              unify_0 CONV clenv.hook (clenv_value clenv mv).rebus n in
             clenv_resrec (metas'@t) evars' clenv
     	  else
-	    let mc,ec =
-	      let mvty = clenv_instance_type clenv mv in
+            begin
 	      if with_types (* or occur_meta mvty *) then
-		(try 
-		let nty = clenv_type_of clenv 
-			    (clenv_instance clenv (mk_freelisted n)) in
-		unify_0 CUMUL [] clenv.hook nty mvty
-		with e when Logic.catchable_exception e -> ([],[]))
-	      else ([],[]) in
-	    clenv_resrec (mc@t) ec (clenv_assign mv n clenv)
-
-  in clenv_resrec
+	        (let mvty = clenv_instance_type clenv mv in
+		try 
+		  let nty = clenv_type_of clenv 
+			      (clenv_instance clenv (mk_freelisted n)) in
+		  let (mc,ec) = unify_0 CUMUL clenv.hook nty mvty in
+                  ty_metas := mc @ !ty_metas;
+                  ty_evars := ec @ !ty_evars
+		with e when Logic.catchable_exception e -> ());
+	      clenv_resrec t [] (clenv_assign mv n clenv)
+            end in
+  (* merge constraints *)
+  let clenv' = clenv_resrec metas evars clenv in
+  if with_types then
+    (* merge constraints about types: if they fail, don't worry *)
+    try clenv_resrec !ty_metas !ty_evars clenv'
+    with e when Logic.catchable_exception e -> clenv'
+  else clenv'
 
 (* [clenv_unify M N clenv]
    performs a unification of M and N, generating a bunch of
@@ -637,89 +635,13 @@ let clenv_merge with_types =
    [clenv_typed_unify M N clenv] expects in addition that expected
    types of metavars are unifiable with the types of their instances    *)
 
-let clenv_unify_core with_types cv_pb m n clenv =
-  let (mc,ec) = unify_0 cv_pb [] clenv.hook m n in 
+let clenv_unify_core_0 with_types cv_pb m n clenv =
+  let (mc,ec) = unify_0 cv_pb clenv.hook m n in 
   clenv_merge with_types mc ec clenv
-    
-(* let clenv_unify = clenv_unify_core false *)
-let clenv_unify = clenv_unify_core false
-let clenv_typed_unify = clenv_unify_core true
 
-(* [clenv_bchain mv clenv' clenv]
- *
- * Resolves the value of "mv" (which must be undefined) in clenv to be
- * the template of clenv' be the value "c", applied to "n" fresh
- * metavars, whose types are chosen by destructing "clf", which should
- * be a clausale forme generated from the type of "c".  The process of
- * resolution can cause unification of already-existing metavars, and
- * of the fresh ones which get created.  This operation is a composite
- * of operations which pose new metavars, perform unification on
- * terms, and make bindings.  *)
+let clenv_unify_0 = clenv_unify_core_0 false
+let clenv_typed_unify = clenv_unify_core_0 true
 
-let clenv_bchain mv subclenv clenv =
-  (* Add the metavars of [subclenv] to [clenv], with their name-environment *)
-  let clenv' =
-    { templval = clenv.templval;
-      templtyp = clenv.templtyp;
-      namenv =
-	List.fold_left (fun ne (mv,id) ->
-			  if clenv_defined subclenv mv then 
-			    ne
-			  else if intmap_in_dom mv ne then begin
-			    warning ("Cannot put metavar "^(string_of_int mv)^
-				     " in name-environment twice");
-			    ne
-			  end else 
-			    Intmap.add mv id ne)
-          clenv.namenv (intmap_to_list subclenv.namenv);
-      env = List.fold_left (fun m (n,v) -> Intmap.add n v m) 
-	       clenv.env (intmap_to_list subclenv.env);
-      hook = clenv.hook } 
-  in
-  (* unify the type of the template of [subclenv] with the type of [mv] *)
-  let clenv'' =
-    clenv_unify CUMUL (clenv_instance clenv' (clenv_template_type subclenv))
-      (clenv_instance_type clenv' mv)
-      clenv' 
-  in
-  (* assign the metavar *)
-  let clenv''' =
-    clenv_assign mv (clenv_instance clenv' (clenv_template subclenv)) clenv''
-  in 
-  clenv'''
-
-
-(* swaps the "hooks" in [clenv1] and [clenv2], so we can then use
-   backchain to hook them together *)
-
-let clenv_swap clenv1 clenv2 =
-  let clenv1' = { templval = clenv1.templval;
-		  templtyp = clenv1.templtyp;
-		  namenv = clenv1.namenv;
-		  env = clenv1.env;
-		  hook = clenv2.hook} 
-  and clenv2' = { templval = clenv2.templval;
-		  templtyp = clenv2.templtyp;
-		  namenv = clenv2.namenv;
-		  env = clenv2.env;
-		  hook = clenv1.hook}
-  in 
-  (clenv1',clenv2')
-
-let clenv_fchain mv nextclenv clenv =
-  let (clenv',nextclenv') = clenv_swap clenv nextclenv in 
-  clenv_bchain mv clenv' nextclenv'
-
-let clenv_refine kONT clenv gls =
-  tclTHEN
-    (kONT clenv.hook) 
-    (refine (clenv_instance_template clenv)) gls
-
-let clenv_refine_cast kONT clenv gls =
-  tclTHEN
-    (kONT clenv.hook) 
-    (refine (clenv_cast_meta clenv (clenv_instance_template clenv)))
-    gls
 
 (* takes a substitution s, an open term op and a closed term cl
    try to find a subterm of cl which matches op, if op is just a Meta
@@ -734,12 +656,15 @@ let iter_fail f a =
       with ex when catchable_exception ex -> ffail (i+1)
   in ffail 0
 
-let constrain_clenv_to_subterm clause (op,cl) =
+(* Tries to find an instance of term [cl] in term [op].
+   Unifies [cl] to every subterm of [op] until it finds a match.
+   Fails if no match is found *)
+let unify_to_subterm clause (op,cl) =
   let rec matchrec cl =
     let cl = strip_outer_cast cl in
     (try 
        if closed0 cl 
-       then clenv_unify CONV op cl clause,cl
+       then clenv_unify_0 CONV op cl clause,cl
        else error "Bound 1"
      with ex when catchable_exception ex ->
        (match kind_of_term cl with 
@@ -792,13 +717,13 @@ let constrain_clenv_to_subterm clause (op,cl) =
 
 (* Possibly gives K-terms in case the operator does not contain 
    a meta : BUG ?? *)
-let constrain_clenv_using_subterm_list allow_K clause oplist t = 
+let unify_to_subterm_list allow_K clause oplist t = 
   List.fold_right 
     (fun op (clause,l) -> 
        if occur_meta op then 
          let (clause',cl) =
            (try 
-	      constrain_clenv_to_subterm clause (strip_outer_cast op,t)
+	      unify_to_subterm clause (strip_outer_cast op,t)
             with e when catchable_exception e ->
               if allow_K then (clause,op) else raise e)
          in 
@@ -808,6 +733,159 @@ let constrain_clenv_using_subterm_list allow_K clause oplist t =
     oplist 
     (clause,[])
 
+let secondOrderAbstraction allow_K typ (p, oplist) clause =
+  let env = w_env clause.hook in
+  let sigma = w_Underlying clause.hook in
+  let (clause',cllist) = unify_to_subterm_list allow_K clause oplist typ in
+  let typp = clenv_instance_type clause' p in 
+  let pred = abstract_list_all env sigma typp typ cllist in
+  clenv_unify_0 CONV (mkMeta p) pred clause'
+
+let clenv_unify2 allow_K cv_pb ty1 ty2 clause =
+  let c1, oplist1 = whd_stack ty1 in
+  let c2, oplist2 = whd_stack ty2 in
+  match kind_of_term c1, kind_of_term c2 with
+    | Meta p1, _ ->
+        (* Find the predicate *)
+	let clause' =
+          secondOrderAbstraction allow_K ty2 (p1,oplist1) clause in 
+        (* Resume first order unification *)
+	clenv_unify_0 cv_pb (clenv_instance_term clause' ty1) ty2 clause'
+    | _, Meta p2 ->
+        (* Find the predicate *)
+	let clause' =
+          secondOrderAbstraction allow_K ty1 (p2, oplist2) clause in 
+        (* Resume first order unification *)
+	clenv_unify_0 cv_pb ty1 (clenv_instance_term clause' ty2) clause'
+    | _ -> error "clenv_unify2"
+
+
+(* The unique unification algorithm works like this: If the pattern is
+   flexible, and the goal has a lambda-abstraction at the head, then
+   we do a first-order unification.
+
+   If the pattern is not flexible, then we do a first-order
+   unification, too.
+
+   If the pattern is flexible, and the goal doesn't have a
+   lambda-abstraction head, then we second-order unification. *)
+
+(* We decide here if first-order or second-order unif is used for Apply *)
+(* We apply a term of type (ai:Ai)C and try to solve a goal C'          *)
+(* The type C is in clenv.templtyp.rebus with a lot of Meta to solve    *)
+
+(* 3-4-99 [HH] New fo/so choice heuristic :
+   In case we have to unify (Meta(1) args) with ([x:A]t args')
+   we first try second-order unification and if it fails first-order.
+   Before, second-order was used if the type of Meta(1) and [x:A]t was
+   convertible and first-order otherwise. But if failed if e.g. the type of
+   Meta(1) had meta-variables in it. *)
+let clenv_unify allow_K cv_pb ty1 ty2 clenv =
+  let hd1,l1 = whd_stack ty1 in
+  let hd2,l2 = whd_stack ty2 in
+  match kind_of_term hd1, l1<>[], kind_of_term hd2, l2<>[] with
+    (* Pattern case *)
+    | (Meta _, true, Lambda _, _ | Lambda _, _, Meta _, true)
+      when List.length l1 = List.length l2 ->
+	(try 
+	   clenv_typed_unify cv_pb ty1 ty2 clenv
+	 with ex when catchable_exception ex -> 
+	   try 
+	     clenv_unify2 allow_K cv_pb ty1 ty2 clenv
+	   with ex when catchable_exception ex -> 
+	     error "Cannot solve a second-order unification problem")
+
+     (* Second order case *)
+     | (Meta _, true, _, _ | _, _, Meta _, true) -> 
+	(try 
+	   clenv_unify2 allow_K cv_pb ty1 ty2 clenv
+         with ex when catchable_exception ex -> 
+           try 
+	     clenv_typed_unify cv_pb ty1 ty2 clenv
+           with ex when catchable_exception ex -> 
+             error "Cannot solve a second-order unification problem")
+
+     (* General case: try first order *)
+     | _ -> clenv_unify_0 cv_pb ty1 ty2 clenv
+
+
+(* [clenv_bchain mv clenv' clenv]
+ *
+ * Resolves the value of "mv" (which must be undefined) in clenv to be
+ * the template of clenv' be the value "c", applied to "n" fresh
+ * metavars, whose types are chosen by destructing "clf", which should
+ * be a clausale forme generated from the type of "c".  The process of
+ * resolution can cause unification of already-existing metavars, and
+ * of the fresh ones which get created.  This operation is a composite
+ * of operations which pose new metavars, perform unification on
+ * terms, and make bindings.  *)
+
+let clenv_bchain mv subclenv clenv =
+  (* Add the metavars of [subclenv] to [clenv], with their name-environment *)
+  let clenv' =
+    { templval = clenv.templval;
+      templtyp = clenv.templtyp;
+      namenv =
+	List.fold_left (fun ne (mv,id) ->
+			  if clenv_defined subclenv mv then 
+			    ne
+			  else if intmap_in_dom mv ne then begin
+			    warning ("Cannot put metavar "^(string_of_int mv)^
+				     " in name-environment twice");
+			    ne
+			  end else 
+			    Intmap.add mv id ne)
+          clenv.namenv (intmap_to_list subclenv.namenv);
+      env = List.fold_left (fun m (n,v) -> Intmap.add n v m) 
+	       clenv.env (intmap_to_list subclenv.env);
+      hook = clenv.hook } 
+  in
+  (* unify the type of the template of [subclenv] with the type of [mv] *)
+  let clenv'' =
+    clenv_unify true CUMUL
+      (clenv_instance clenv' (clenv_template_type subclenv))
+      (clenv_instance_type clenv' mv)
+      clenv' 
+  in
+  (* assign the metavar *)
+  let clenv''' =
+    clenv_assign mv (clenv_instance clenv' (clenv_template subclenv)) clenv''
+  in 
+  clenv'''
+
+
+(* swaps the "hooks" in [clenv1] and [clenv2], so we can then use
+   backchain to hook them together *)
+
+let clenv_swap clenv1 clenv2 =
+  let clenv1' = { templval = clenv1.templval;
+		  templtyp = clenv1.templtyp;
+		  namenv = clenv1.namenv;
+		  env = clenv1.env;
+		  hook = clenv2.hook} 
+  and clenv2' = { templval = clenv2.templval;
+		  templtyp = clenv2.templtyp;
+		  namenv = clenv2.namenv;
+		  env = clenv2.env;
+		  hook = clenv1.hook}
+  in 
+  (clenv1',clenv2')
+
+let clenv_fchain mv nextclenv clenv =
+  let (clenv',nextclenv') = clenv_swap clenv nextclenv in 
+  clenv_bchain mv clenv' nextclenv'
+
+let clenv_refine kONT clenv gls =
+  tclTHEN
+    (kONT clenv.hook) 
+    (refine (clenv_instance_template clenv)) gls
+
+let clenv_refine_cast kONT clenv gls =
+  tclTHEN
+    (kONT clenv.hook) 
+    (refine (clenv_cast_meta clenv (clenv_instance_template clenv)))
+    gls
+
 (* [clenv_metavars clenv mv]
  * returns a list of the metavars which appear in the type of
  * the metavar mv.  The list is unordered. *)
@@ -819,12 +897,13 @@ let clenv_metavars clenv mv =
 
 let clenv_template_metavars clenv = clenv.templval.freemetas
 
-(* [clenv_dependent clenv cval ctyp]
- * returns a list of the metavariables which appear in the term cval,
+(* [clenv_dependent hyps_only clenv]
+ * returns a list of the metavars which appear in the template of clenv,
  * and which are dependent, This is computed by taking the metavars in cval,
  * in right-to-left order, and collecting the metavars which appear
- * in their types, and adding in all the metavars appearing in ctyp, the
- * type of cval. *)
+ * in their types, and adding in all the metavars appearing in the
+ * type of clenv.
+ * If [hyps_only] then metavariables occurring in the type are _excluded_ *)
 
 let dependent_metas clenv mvs conclmetas =
   List.fold_right
@@ -832,76 +911,42 @@ let dependent_metas clenv mvs conclmetas =
        Intset.union deps (clenv_metavars clenv mv))
     mvs conclmetas
 
-let clenv_dependent clenv (cval,ctyp) =
-  let mvs = collect_metas (clenv_instance clenv cval) in
-  let ctyp_mvs = metavars_of (clenv_instance clenv ctyp) in
+let clenv_dependent hyps_only clenv =
+  let mvs = collect_metas (clenv_instance_template clenv) in
+  let ctyp_mvs = metavars_of (clenv_instance_template_type clenv) in
   let deps = dependent_metas clenv mvs ctyp_mvs in
-  List.filter (fun mv -> Intset.mem mv deps) mvs
+  List.filter
+    (fun mv -> Intset.mem mv deps && not (hyps_only && Intset.mem mv ctyp_mvs))
+    mvs
 
+let clenv_missing c = clenv_dependent true c
 
-(* [clenv_independent clenv (cval,ctyp)]
+(* [clenv_independent clenv]
  * returns a list of metavariables which appear in the term cval,
  * and which are not dependent.  That is, they do not appear in
  * the types of other metavars which are in cval, nor in the type
  * of cval, ctyp. *)
 
-let clenv_independent clenv (cval,ctyp) =
-  let mvs = collect_metas (clenv_instance clenv cval) in
-  let ctyp_mvs = metavars_of (clenv_instance clenv ctyp) in
+let clenv_independent clenv =
+  let mvs = collect_metas (clenv_instance_template clenv) in
+  let ctyp_mvs = metavars_of (clenv_instance_template_type clenv) in
   let deps = dependent_metas clenv mvs ctyp_mvs in
   List.filter (fun mv -> not (Intset.mem mv deps)) mvs
 
-
-(* [clenv_missing clenv (cval,ctyp)]
- * returns a list of the metavariables which appear in the term cval,
- * and which are dependent, and do NOT appear in ctyp. *)
-
-let clenv_missing clenv (cval,ctyp) =
-  let mvs = collect_metas (clenv_instance clenv cval) in
-  let ctyp_mvs = metavars_of (clenv_instance clenv ctyp) in
-  let deps = dependent_metas clenv mvs ctyp_mvs in
-  List.filter 
-    (fun n -> Intset.mem n deps && not (Intset.mem n ctyp_mvs))
-    mvs
+let clenv_constrain_dep_args hyps_only clause = function
+  | [] -> clause 
+  | mlist ->
+      let occlist = clenv_dependent hyps_only clause in
+      if List.length occlist = List.length mlist then
+	List.fold_left2
+          (fun clenv k c -> clenv_unify true CONV (mkMeta k) c clenv)
+          clause occlist mlist
+      else 
+	error ("Not the right number of missing arguments (expected "
+	       ^(string_of_int (List.length occlist))^")")
 
 let clenv_constrain_missing_args mlist clause =
-  if mlist = [] then 
-    clause 
-  else
-    let occlist = clenv_missing clause (clenv_template clause,
-                                   	(clenv_template_type clause)) in 
-    if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
-        clause occlist mlist
-    else 
-      error ("Not the right number of missing arguments (expected "
-	     ^(string_of_int (List.length occlist))^")")
-
-let clenv_constrain_dep_args mlist clause =
-  if mlist = [] then 
-    clause 
-  else
-    let occlist = clenv_dependent clause (clenv_template clause,
-					  (clenv_template_type clause)) in 
-    if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
-        clause occlist mlist
-    else 
-      error ("Not the right number of missing arguments (expected "
-	     ^(string_of_int (List.length occlist))^")")
-
-let clenv_constrain_dep_args_of mv mlist clause =
-  if mlist = [] then 
-    clause 
-  else
-    let occlist = clenv_dependent clause (clenv_value clause mv,
-                                          clenv_type clause mv) in 
-    if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
-        clause occlist mlist
-    else 
-      error ("clenv_constrain_dep_args_of: Not the right number " ^ 
-	     "of dependent arguments")
+  clenv_constrain_dep_args true clause mlist
 
 let clenv_lookup_name clenv id =
   match intmap_inv clenv.namenv id with
@@ -914,10 +959,7 @@ let clenv_lookup_name clenv id =
 	 anomaly "clenv_lookup_name: a name occurs more than once in clause"
 
 let clenv_match_args s clause =
-  let mvs = clenv_independent clause
-	      (clenv_template clause,
-               clenv_template_type clause)
-  in 
+  let mvs = clenv_independent clause in 
   let rec matchrec clause = function
     | [] -> clause
     | (b,c)::t ->
@@ -936,13 +978,14 @@ let clenv_match_args s clause =
 		      str " occurs more than once in binding");
 		 (try 
 		    List.nth mvs (n-1)
-		  with Failure "nth" -> errorlabstrm "clenv_match_args"
-                      (str "No such binder"))
+		  with (Failure _|Invalid_argument _) ->
+                    errorlabstrm "clenv_match_args" (str "No such binder"))
 	     | Com -> anomaly "No free term in clenv_match_args")
 	in
 	let k_typ = w_hnf_constr clause.hook (clenv_instance_type clause k)
 	and c_typ = w_hnf_constr clause.hook (w_type_of clause.hook c) in
-	matchrec (clenv_assign k c (clenv_unify CUMUL c_typ k_typ clause)) t
+        matchrec 
+       	    (clenv_assign k c (clenv_unify true CUMUL c_typ k_typ clause)) t
   in 
   matchrec clause s
 
@@ -954,8 +997,7 @@ let clenv_match_args s clause =
  * metas. *)
 
 let clenv_pose_dependent_evars clenv =
-  let dep_mvs = clenv_dependent clenv (clenv_template clenv,
-                                       clenv_template_type clenv) in 
+  let dep_mvs = clenv_dependent false clenv in
   List.fold_left
     (fun clenv mv ->
        let evar = Evarutil.new_evar_in_sign (w_env clenv.hook) in
@@ -973,95 +1015,11 @@ let clenv_add_sign (id,sign) clenv =
     env = clenv.env;
     hook = w_add_sign (id,sign) clenv.hook}
 
-(* The unique unification algorithm works like this: If the pattern is
-   flexible, and the goal has a lambda-abstraction at the head, then
-   we do a first-order unification.
-
-   If the pattern is not flexible, then we do a first-order
-   unification, too.
-
-   If the pattern is flexible, and the goal doesn't have a
-   lambda-abstraction head, then we second-order unification. *)
-
-let clenv_fo_resolver clenv gls =
-  clenv_unify CUMUL (clenv_instance_template_type clenv) (pf_concl gls) clenv
-
-let clenv_typed_fo_resolver clenv gls =
-  clenv_typed_unify CUMUL (clenv_instance_template_type clenv) (pf_concl gls) clenv
-
-(* if lname_typ is [xn,An;..;x1,A1] and l is a list of terms,
-   gives [x1:A1]..[xn:An]c' such that c converts to ([x1:A1]..[xn:An]c' l) *)
-
-let abstract_scheme env c l lname_typ =
-  List.fold_left2 
-    (fun t (locc,a) (na,ta) ->
-       if occur_meta ta then error "cannot find a type for the generalisation"
-       else if occur_meta a then lambda_name env (na,ta,t)
-       else lambda_name env (na,ta,subst_term_occ locc a t))
-    c
-    (List.rev l)
-    lname_typ
-
-let abstract_list_all env sigma typ c l =
-  let lname_typ,_ = splay_prod env sigma typ in
-  let p = abstract_scheme env c (List.map (function a -> [],a) l) lname_typ in 
-  try 
-    if is_conv_leq env sigma (Typing.type_of env sigma p) typ then p
-    else error "abstract_list_all"
-  with UserError _ ->
-    raise (RefinerError (CannotGeneralize typ))
-
-let secondOrderAbstraction allow_K gl p oplist clause =
-  let (clause',cllist) = 
-    constrain_clenv_using_subterm_list allow_K clause oplist (pf_concl gl) in
-  let typp = clenv_instance_type clause' p in 
-  clenv_unify CONV (mkMeta p)
-    (abstract_list_all (pf_env gl) (project gl) typp (pf_concl gl) cllist) 
-    clause'
-
-let clenv_so_resolver allow_K clause gl =
-  let c, oplist = whd_stack (clenv_instance_template_type clause) in
-  match kind_of_term c with
-    | Meta p ->
-	let clause' = secondOrderAbstraction allow_K gl p oplist clause in 
-	clenv_fo_resolver clause' gl
-    | _ -> error "clenv_so_resolver"
-
-(* We decide here if first-order or second-order unif is used for Apply *)
-(* We apply a term of type (ai:Ai)C and try to solve a goal C'          *)
-(* The type C is in clenv.templtyp.rebus with a lot of Meta to solve    *)
-
-(* 3-4-99 [HH] New fo/so choice heuristic :
-   In case we have to unify (Meta(1) args) with ([x:A]t args')
-   we first try second-order unification and if it fails first-order.
-   Before, second-order was used if the type of Meta(1) and [x:A]t was
-   convertible and first-order otherwise. But if failed if e.g. the type of
-   Meta(1) had meta-variables in it. *)
-
-let clenv_unique_resolver allow_K clenv gls =
-  let pathd,_ = whd_stack (clenv_instance_template_type clenv) in
-  let glhd,_ = whd_stack (pf_concl gls) in
-  match kind_of_term pathd, kind_of_term glhd with
-    | Meta _, Lambda _ ->
-	(try 
-	   clenv_typed_fo_resolver clenv gls
-	 with ex when catchable_exception ex -> 
-	   try 
-	     clenv_so_resolver allow_K clenv gls
-	   with ex when catchable_exception ex -> 
-	     error "Cannot solve a second-order unification problem")
-
-    | Meta _, _ -> 
-	(try 
-	   clenv_so_resolver allow_K clenv gls
-         with ex when catchable_exception ex -> 
-           try 
-	     clenv_typed_fo_resolver clenv gls
-           with ex when catchable_exception ex -> 
-             error "Cannot solve a second-order unification problem")
-
-    | _ -> clenv_fo_resolver clenv gls
+(***************************)
 
+let clenv_unique_resolver allow_K clause gl =
+  clenv_unify allow_K CUMUL
+    (clenv_instance_template_type clause) (pf_concl gl) clause 
 
 let res_pf kONT clenv gls =
   clenv_refine kONT (clenv_unique_resolver false clenv gls) gls
@@ -1081,12 +1039,12 @@ let e_res_pf kONT clenv gls =
 let collect_com lbind = 
   map_succeed (function (Com,c)->c | _ -> failwith "Com") lbind
 
-let make_clenv_binding_apply wc n (c,t) lbind = 
+let make_clenv_binding_gen n wc (c,t) lbind = 
   let largs = collect_com lbind in
   let lcomargs = List.length largs in
   if lcomargs = List.length lbind then 
     let clause = mk_clenv_from_n wc n (c,t) in
-    clenv_constrain_missing_args largs clause
+    clenv_constrain_dep_args (n <> None) clause largs
   else if lcomargs = 0 then 
     let clause = mk_clenv_rename_from_n wc n (c,t) in
     clenv_match_args lbind clause
@@ -1094,18 +1052,8 @@ let make_clenv_binding_apply wc n (c,t) lbind =
     errorlabstrm "make_clenv_bindings"
       (str "Cannot mix bindings and free associations")
 
-let make_clenv_binding wc (c,t) lbind = 
-  let largs    = collect_com lbind in
-  let lcomargs = List.length largs in 
-  if lcomargs = List.length lbind then 
-    let clause = mk_clenv_from wc (c,t) in  
-    clenv_constrain_dep_args largs clause
-  else if lcomargs = 0 then 
-    let clause = mk_clenv_rename_from wc (c,t) in  
-    clenv_match_args lbind clause
-  else 
-    errorlabstrm "make_clenv_bindings"
-      (str "Cannot mix bindings and free associations")
+let make_clenv_binding_apply wc n = make_clenv_binding_gen (Some n) wc
+let make_clenv_binding = make_clenv_binding_gen None
 
 open Printer
 
