@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: evar_refiner.ml,v 1.27 2001/11/19 15:44:28 clrenard Exp $ *)
+(* $Id: evar_refiner.ml,v 1.29 2002/04/10 15:49:50 barras Exp $ *)
 
 open Pp
 open Util
@@ -24,6 +24,8 @@ open Proof_type
 open Logic
 open Refiner
 
+type wc = named_context sigma (* for a better reading of the following *)
+
 let rc_of_pfsigma sigma = rc_of_gc sigma.sigma sigma.it.goal
 let rc_of_glsigma sigma = rc_of_gc sigma.sigma sigma.it
 
@@ -87,7 +89,6 @@ let w_type_of wc c  =
   type_of (Global.env_of_context wc.it) wc.sigma c
 let w_env     wc    = get_env wc
 let w_hyps    wc    = named_context (get_env wc)
-let w_defined_const wc sp = defined_constant (w_env wc) sp
 let w_defined_evar wc k      = Evd.is_defined (w_Underlying wc) k
 let w_const_value wc         = constant_value (w_env wc)
 let w_conv_x wc m n          = is_conv (w_env wc) (w_Underlying wc) m n
