@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: proof_trees.ml,v 1.44 2001/12/13 09:03:09 filliatr Exp $ *)
+(* $Id: proof_trees.ml,v 1.45 2002/02/07 16:07:31 barras Exp $ *)
 
 open Closure
 open Util
@@ -85,11 +85,11 @@ let get_hyps  evc = evc.it
 let get_env   evc = Global.env_of_context evc.it
 let get_gc    evc = evc.sigma
 
-let pf_lookup_name_as_renamed hyps ccl s =
-  Detyping.lookup_name_as_renamed hyps ccl s
+let pf_lookup_name_as_renamed env ccl s =
+  Detyping.lookup_name_as_renamed env ccl s
 
-let pf_lookup_index_as_renamed ccl n =
-  Detyping.lookup_index_as_renamed ccl n
+let pf_lookup_index_as_renamed env ccl n =
+  Detyping.lookup_index_as_renamed env ccl n
 
 (*********************************************************************)
 (*                  Pretty printing functions                        *)
