@@ -31,28 +31,20 @@ type hyp_location = (* To distinguish body and type of local defs *)
   | InHyp of identifier
   | InHypType of identifier
 
-type prim_rule_name = 
-  | Intro
-  | Intro_after
-  | Intro_replacing
-  | Cut of bool
-  | FixRule
-  | Cofix
-  | Refine 
-  | Convert_concl
-  | Convert_hyp
-  | Convert_defbody
-  | Convert_deftype
-  | Thin
-  | ThinBody
-  | Move of bool
+type prim_rule =
+  | Intro of identifier
+  | Intro_replacing of identifier
+  | Cut of bool * identifier * types
+  | FixRule of identifier * int * (identifier * int * constr) list
+  | Cofix of identifier * (identifier * constr) list
+  | Refine of constr
+  | Convert_concl of types
+  | Convert_hyp of named_declaration
+  | Thin of identifier list
+  | ThinBody of identifier list
+  | Move of bool * identifier * identifier
+  | Rename of identifier * identifier
 
-type prim_rule = {
-  name : prim_rule_name;
-  hypspecs : identifier list;
-  newids : identifier list;
-  params : Coqast.t list;
-  terms : constr list }
 
 (* Signature useful to define the tactic type *)
 type 'a sigma = { 
