@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: tacinterp.ml,v 1.69 2001/12/20 22:32:42 herbelin Exp $ *)
+(* $Id: tacinterp.ml,v 1.77.2.1 2002/09/16 11:03:01 herbelin Exp $ *)
 
 open Astterm
 open Closure
@@ -41,10 +41,11 @@ let err_msg_tactic_not_found macro_loc macro =
 
 (* Values for interpretation *)
 type value =
-  | VTactic of tactic
-  | VFTactic of tactic_arg list * (tactic_arg list->tactic)
+  | VTacticClos of interp_sign * Coqast.t
+  | VFTactic of tactic_arg list * string
   | VRTactic of (goal list sigma * validation)
-  | VContext of (interp_sign -> goal sigma -> value)
+  | VTactic of tactic (* For mixed ML/Ltac tactics (e.g. Tauto) *)
+  | VContext of interp_sign * Coqast.t * Coqast.t list
   | VArg of tactic_arg
   | VFun of (identifier * value) list * identifier option list * Coqast.t
   | VVoid
@@ -62,14 +63,6 @@ and interp_sign =
 (* For tactic_of_value *)
 exception NotTactic
 
-(* Gives the tactic corresponding to the tactic value *)
-let tactic_of_value vle g =
-  match vle with
-  | VTactic tac -> (tac g)
-  | VFTactic (largs,f) -> (f largs g) 
-  | VRTactic res -> res
-  | _ -> raise NotTactic
-
 (* Gives the identifier corresponding to an Identifier tactic_arg *)
 let id_of_Identifier = function
   | Identifier id -> id
@@ -105,21 +98,33 @@ let make_ids ast = function
   | _ -> anomalylabstrm "make_ids" (str "Not an identifier")
 
 (* Gives Qualid's and makes the possible injection identifier -> qualid *)
-let make_qid = function
+(*let make_qid = function
   | VArg (Qualid _) as arg -> arg
   | VArg (Identifier id) -> VArg (Qualid (make_short_qualid id))
   | VArg (Constr c) ->
     (match (kind_of_term c) with
     | Const cst -> VArg (Qualid (qualid_of_sp cst))
     | _ -> anomalylabstrm "make_qid" (str "Not a Qualid"))
-  | _ -> anomalylabstrm "make_qid" (str "Not a Qualid")
+  | _ -> anomalylabstrm "make_qid" (str "Not a Qualid")*)
+
+(* Gives qualid's and makes the possible injection identifier -> qualid *)
+let make_qid = function
+  | VArg (Qualid q) -> q
+  | VArg (Identifier id) -> make_short_qualid id
+  | VArg (Constr c) ->
+    (match (kind_of_term c) with
+    | Const cst -> qualid_of_sp cst
+    | Var id -> make_short_qualid id
+    | _ -> anomalylabstrm "make_qid" (str "Not a qualid"))
+  | _ -> anomalylabstrm "make_qid" (str "Not a qualid")
 
 (* Transforms a named_context into a (string * constr) list *)
 let make_hyps = List.map (fun (id,_,typ) -> (string_of_id id,body_of_type typ))
 
 (* Transforms an id into a constr if possible *)
-let constr_of_id id = function
-  | None -> raise Not_found
+let constr_of_id ist id =
+  match ist.goalopt with
+  | None -> construct_reference ist.env id
   | Some goal ->
     let hyps = pf_hyps goal in
     if mem_named_context id hyps then
@@ -131,21 +136,21 @@ let constr_of_id id = function
       | _ -> csr)
 
 (* Extracted the constr list from lfun *)
-let rec constr_list goalopt = function
-  | (id,VArg(Constr c))::tl -> (id,c)::(constr_list goalopt tl)
+let rec constr_list_aux ist = function
+  | (id,VArg(Constr c))::tl -> (id,c)::(constr_list_aux ist tl)
   | (id0,VArg(Identifier id))::tl ->
-    (try (id0,(constr_of_id id goalopt))::(constr_list goalopt tl)
-     with | Not_found -> (constr_list goalopt tl))
-  | _::tl -> constr_list goalopt tl
+    (try (id0,(constr_of_id ist id))::(constr_list_aux ist tl)
+     with | Not_found -> (constr_list_aux ist tl))
+  | _::tl -> constr_list_aux ist tl
   | [] -> []
 
+let constr_list ist = constr_list_aux ist ist.lfun
+
 let interp_constr ist c ocl =
-  interp_constr_gen ist.evc ist.env
-    (constr_list ist.goalopt ist.lfun) ist.lmatch c ocl
+  interp_constr_gen ist.evc ist.env (constr_list ist) ist.lmatch c ocl
 
 let interp_openconstr ist c ocl =
-  interp_openconstr_gen ist.evc ist.env
-    (constr_list ist.goalopt ist.lfun) ist.lmatch c ocl
+  interp_openconstr_gen ist.evc ist.env (constr_list ist) ist.lmatch c ocl
 
 (* To embed several objects in Coqast.t *)
 let ((tactic_in : (interp_sign -> Coqast.t) -> Dyn.t),
@@ -382,18 +387,9 @@ let rec read_match_rule evc env lfun = function
 exception No_match
 exception Not_coherent_metas
 
-(* Evaluation with FailError catching *)
-let eval_with_fail interp ast goal =
-  try 
-    (match interp ast with
-    | VTactic tac -> VRTactic (tac goal)
-    | VFTactic (largs,f) -> VRTactic (f largs goal)
-    | a -> a)
-  with | FailError lvl ->
-    if lvl = 0 then
-      raise No_match
-    else
-      raise (FailError (lvl - 1))
+let is_match_catchable = function
+  | No_match | FailError _ -> true
+  | e -> Logic.catchable_exception e
 
 (* Verifies if the matched list is coherent with respect to lcm *)
 let rec verify_metas_coherence ist lcm = function
@@ -478,6 +474,7 @@ let rec val_interp ist ast =
 
   let value_interp ist =
   match ast with
+    (* Immediate evaluation *)
     | Node(_,"APP",l) ->
         let fv = val_interp ist (List.hd l)
         and largs = List.map (val_interp ist) (List.tl l) in
@@ -487,42 +484,24 @@ let rec val_interp ist ast =
     | Node(_,"LET",[Node(_,"LETDECL",l);u]) ->
         let addlfun=letin_interp ist ast l in
         val_interp { ist with lfun=addlfun@ist.lfun } u
-    | Node(_,"LETCUT",[Node(_,"LETDECL",l)]) ->
-        VTactic (letcut_interp ist ast l)
-    | Node(_,"MATCHCONTEXT",lmr) -> match_context_interp ist ast lmr
+    | Node(_,"MATCHCONTEXT",lmr) ->
+       (match ist.goalopt with
+          | None -> VContext (ist,ast,lmr)
+          | Some g -> match_context_interp ist ast lmr g)
     | Node(_,"MATCH",lmr) -> match_interp ist ast lmr
-    | Node(_,"IDTAC",[]) -> VTactic tclIDTAC
-    | Node(_,"FAIL",[]) -> VTactic (tclFAIL 0)
-    | Node(_,"FAIL",[n]) -> VTactic (tclFAIL (num_of_ast n))
-    | Node(_,"PROGRESS",[tac]) ->
-        VTactic (tclPROGRESS (tactic_interp ist tac))
-    | Node(_,"TACTICLIST",l) ->
-        VTactic (interp_semi_list tclIDTAC ist.lfun ist.lmatch ist.debug l)
-    | Node(_,"DO",[n;tac]) ->
-        VTactic (tclDO (num_of_ast n) (tactic_interp ist tac))
-    | Node(_,"TRY",[tac]) -> VTactic (tclTRY (tactic_interp ist tac))
-    | Node(_,"INFO",[tac]) -> VTactic (tclINFO (tactic_interp ist tac))
-    | Node(_,"REPEAT",[tac]) -> VTactic (tclREPEAT (tactic_interp ist tac))
-    | Node(_,"ORELSE",[tac1;tac2]) ->
-        VTactic (tclORELSE (tactic_interp ist tac1) (tactic_interp ist tac2))
-    | Node(_,"FIRST",l) -> VTactic (tclFIRST (List.map (tactic_interp ist) l))
-    | Node(_,"TCLSOLVE",l) ->
-        VTactic (tclSOLVE (List.map (tactic_interp ist) l))
-(*    | Node(loc0,"APPTACTIC",[Node(loc1,s,l)]) ->
-        val_interp ist
-          (Node(loc0,"APP",[Node(loc1,"PRIM-TACTIC",[Node(loc1,s,[])])]@l))*)
-    | Node(_,"PRIMTACTIC",[Node(loc,opn,[])]) ->
-        VFTactic ([],(interp_atomic opn))
+    (* Delayed evaluation *)
+    | Node(loc,("LETCUT"|"IDTAC"|"FAIL"|"PROGRESS"|"TACTICLIST"|"DO"|"TRY"|"INFO"|"REPEAT"|"ORELSE"|"FIRST"|"TCLSOLVE"),_) -> VTacticClos (ist,ast)
+    (* Arguments and primitive tactics *)
     | Node(_,"VOID",[]) -> VVoid
     | Nvar(_,s) ->
         (try (unrec (List.assoc s ist.lfun))
         with | Not_found ->
                  (try (vcontext_interp ist (lookup s))
                  with | Not_found -> VArg (Identifier s)))
-    | Node(_,"QUALIDARG",[Nvar(_,s)]) ->
+  (*  | Node(_,"QUALIDARG",[Nvar(_,s)]) ->
         (try (make_qid (unrec (List.assoc s ist.lfun)))
         with | Not_found ->
-                 VArg (Qualid (qid_interp ist ast)))
+                 VArg (Qualid (qid_interp ist ast)))*)
     | Node(_,"QUALIDARG",_) | Node(_,"QUALIDMETA",_) ->
         VArg (Qualid (qid_interp ist ast))
     | Str(_,s) -> VArg (Quoted_string s)
@@ -548,23 +527,19 @@ let rec val_interp ist ast =
         VArg ((Intropattern (cvt_intro_pattern ist ast)))
     | Node(_,"LETPATTERNS",astl) ->
         VArg (Letpatterns (List.fold_left (cvt_letpattern ist) (None,[]) astl))
+    | Node(loc,s,[]) ->
+        VFTactic ([],s)
     | Node(loc,s,l) ->
         (try
           ((look_for_interp s) ist ast)
         with
            Not_found ->
              if l = [] then
-               VFTactic ([],(interp_atomic s))
+               VFTactic ([],s)
              else
-               let fv = val_interp ist
-                          (Node(loc,"PRIMTACTIC",[Node(loc,s,[])]))
+               let fv = VFTactic ([],s)
                and largs = List.map (val_interp ist) l in
                app_interp ist fv largs ast)
-
-       (*     val_interp ist (Node(dummy_loc,"APP",[Node(loc,"PRIMTACTIC",
-                            [Node(loc,s,[])])]@l)))*)
- 
-(*             val_interp ist (Node(dummy_loc,"APPTACTIC",[ast])))*)
     | Dynamic(_,t) ->
       let tg = (tag t) in
       if tg = "tactic" then
@@ -581,11 +556,36 @@ let rec val_interp ist ast =
           (str "Unrecognizable ast: " ++ print_ast ast)) in
   if ist.debug = DebugOn then
     match debug_prompt ist.goalopt ast with
-    | Exit -> VTactic tclIDTAC
+    | Exit -> VTacticClos (ist,Node(dummy_loc,"IDTAC",[]))
     | v -> value_interp {ist with debug=v}
   else
     value_interp ist
 
+and eval_tactic ist ast =
+  match ast with
+    | Node(_,"LETCUT",[Node(_,"LETDECL",l)]) -> letcut_interp ist ast l
+    | Node(_,"IDTAC",[]) -> tclIDTAC
+    | Node(_,"FAIL",[]) -> tclFAIL 0
+    | Node(_,"FAIL",[n]) -> tclFAIL (num_of_ast n)
+    | Node(_,"PROGRESS",[tac]) -> tclPROGRESS (tactic_interp ist tac)
+    | Node(_,"TACTICLIST",l) ->
+	interp_semi_list tclIDTAC ist.lfun ist.lmatch ist.debug l
+    | Node(_,"DO",[n;tac]) ->
+        tclDO (num_of_ast n) (tactic_interp ist tac)
+    | Node(_,"TRY",[tac]) -> tclTRY (tactic_interp ist tac)
+    | Node(_,"INFO",[tac]) -> tclINFO (tactic_interp ist tac)
+    | Node(_,"REPEAT",[tac]) -> tclREPEAT (tactic_interp ist tac)
+    | Node(_,"ORELSE",[tac1;tac2]) ->
+        tclORELSE (tactic_interp ist tac1) (tactic_interp ist tac2)
+    | Node(_,"FIRST",l) -> tclFIRST (List.map (tactic_interp ist) l)
+    | Node(_,"TCLSOLVE",l) -> tclSOLVE (List.map (tactic_interp ist) l)
+(*    | Node(loc0,"APPTACTIC",[Node(loc1,s,l)]) ->
+        val_interp ist
+          (Node(loc0,"APP",[Node(loc1,"PRIM-TACTIC",[Node(loc1,s,[])])]@l))*)
+    | _ ->
+        anomaly_loc (Ast.loc ast, "Tacinterp.val_interp",
+          (str "Unrecognizable ast: " ++ print_ast ast))
+
 (* Interprets an application node *)
 and app_interp ist fv largs ast =
   match fv with
@@ -604,6 +604,29 @@ and app_interp ist fv largs ast =
       user_err_loc (Ast.loc ast, "Tacinterp.app_interp",
                     (str"Illegal tactic application: " ++ print_ast ast))
 
+(* Gives the tactic corresponding to the tactic value *)
+and tactic_of_value vle g =
+  match vle with
+  | VTacticClos (ist,tac) -> eval_tactic ist tac g
+  | VFTactic (largs,f) -> (interp_atomic f largs g) 
+  | VRTactic res -> res
+  | VTactic tac -> tac g
+  | _ -> raise NotTactic
+
+(* Evaluation with FailError catching *)
+and eval_with_fail interp ast goal =
+  try 
+    (match interp ast with
+    | VTacticClos (ist,tac) -> VRTactic (eval_tactic ist tac goal)
+    | VFTactic (largs,f) -> VRTactic (interp_atomic f largs goal)
+    | VTactic tac -> VRTactic (tac goal)
+    | a -> a)
+  with | FailError lvl ->
+    if lvl = 0 then
+      raise No_match
+    else
+      raise (FailError (lvl - 1))
+
 (* Interprets recursive expressions *)
 and rec_interp ist ast = function
   | [Node(_,"RECCLAUSE",_)] as l ->
@@ -651,7 +674,7 @@ and letin_interp ist ast = function
       (id,VArg (Constr (mkCast (csr,typ))))::(letin_interp ist ast tl)
     | VArg (Identifier id) ->
       (try
-         (id,VArg (Constr (mkCast (constr_of_id id ist.goalopt,typ))))::
+         (id,VArg (Constr (mkCast (constr_of_id ist id,typ))))::
          (letin_interp ist ast tl)
        with | Not_found ->
          errorlabstrm "Tacinterp.letin_interp"
@@ -705,7 +728,7 @@ and letcut_interp ist ast = function
       (try
          let cutt = interp_atomic "Cut" [Constr typ]
          and exat =
-           interp_atomic "Exact" [Constr (constr_of_id ir ist.goalopt)] in
+           interp_atomic "Exact" [Constr (constr_of_id ist ir)] in
          tclTHENS cutt [tclTHEN (introduction id)
          (letcut_interp ist ast tl);exat]
        with | Not_found ->
@@ -737,7 +760,10 @@ and letcut_interp ist ast = function
       "LetCut not well formed: " ++ print_ast ast))
 
 (* Interprets the Match Context expressions *)
-and match_context_interp ist ast lmr =
+and match_context_interp ist ast lmr g =
+  let lr,lmr = match lmr with 
+    | Str(_,dir)::lmr ->  (dir="LR",lmr)
+    | _ -> anomaly "Ill-formed Match Context" in
 (*  let goal =
     (match goalopt with
     | None ->
@@ -772,8 +798,9 @@ and match_context_interp ist ast lmr =
        with No_match | _ ->
          apply_match_context ist goal tl)
     | (Pat (mhyps,mgoal,mt))::tl ->
-      let hyps = make_hyps (List.rev (pf_hyps goal))
-      and concl = pf_concl goal in
+      let hyps = make_hyps (pf_hyps goal) in
+      let hyps = if lr then List.rev hyps else hyps in
+      let concl = pf_concl goal in
       (match mgoal with
       |	Term mg ->
         (try
@@ -787,38 +814,38 @@ and match_context_interp ist ast lmr =
               apply_hyps_context { ist with goalopt=Some goal} mt lgoal mhyps
                 hyps
             end)
-         with | No_match | _ -> apply_match_context ist goal tl)
+         with e when is_match_catchable e -> apply_match_context ist goal tl)
       |	Subterm (id,mg) ->
         (try
           apply_goal_sub ist goal 0 (id,mg) concl mt mhyps hyps
-         with
-        | No_match | _ -> apply_match_context ist goal tl))
+         with e when is_match_catchable e -> apply_match_context ist goal tl))
     | _ ->
       errorlabstrm "Tacinterp.apply_match_context" (str
         "No matching clauses for Match Context") in
   let app_wo_goal = 
     (fun ist goal ->
        apply_match_context ist goal
-         (read_match_context_rule ist.evc ist.env
-           (constr_list ist.goalopt ist.lfun) lmr)) in
+         (read_match_context_rule ist.evc ist.env (constr_list ist) lmr)) in
 
 (*    (fun goal ->
        let evc = project goal
        and env = pf_env goal in
        apply_match_context ist goal
-         (read_match_context_rule ist.evc ist.env
-           (constr_list ist.goalopt ist.lfun) lmr)) in*)
+         (read_match_context_rule ist.evc ist.env (constr_list ist) lmr)) in*)
 
 (*    (fun ist goal ->
      apply_match_context ist goal
-     (read_match_context_rule evc env (constr_list goalopt lfun) lmr)) in*)
+     (read_match_context_rule evc env (constr_list ist) lmr)) in*)
 
+(*
  (match ist.goalopt with
     | None -> VContext app_wo_goal
     | Some g -> app_wo_goal ist g)
+*)
+        app_wo_goal ist g
 
-(*  apply_match_context evc env lfun lmatch goal (read_match_context_rule evc env
-    (constr_list goalopt lfun) lmr)*)
+(*  apply_match_context evc env lfun lmatch goal (read_match_context_rule evc
+      env (constr_list ist) lmr)*)
 
 (* Tries to match the hypotheses in a Match Context *)
 and apply_hyps_context ist mt lgmatch mhyps hyps =
@@ -846,7 +873,7 @@ and apply_hyps_context ist mt lgmatch mhyps hyps =
             (lfun@lid@lc) (lmatch@lm) tl nextlhyps nextlhyps None
          with
         | (FailError _) as e -> raise e
-        | _ ->
+	|  e when is_match_catchable e -> 
           (match noccopt with
           | None ->
             apply_hyps_context_rec ist mt lfun
@@ -862,10 +889,12 @@ and apply_hyps_context ist mt lgmatch mhyps hyps =
 
 (* Interprets a VContext value *)
 and vcontext_interp ist = function
-  | (VContext f) as v ->
+  | (VContext (ist', ast, lmr)) as v ->
     (match ist.goalopt with
     | None -> v
-    | Some g -> f ist g)
+    | Some g as go -> 
+        let ist = { ist' with goalopt = go; env = pf_env g; evc = project g }
+        in match_context_interp ist ast lmr g)
   | v -> v
 
 (* Interprets the Match expressions *)
@@ -889,20 +918,21 @@ and match_interp ist ast lmr =
        with
        | NextOccurrence n -> raise No_match
        | (FailError _) as e -> raise e
-       | _ -> apply_sub_match ist (nocc + 1) (id,c) csr mt)
+       | e when is_match_catchable e ->
+	   apply_sub_match ist (nocc + 1) (id,c) csr mt)
   in
   let rec apply_match ist csr = function
     | (All t)::_ ->
       (match ist.goalopt with
         | None ->
           (try val_interp ist t
-           with | No_match | _ -> apply_match ist csr [])
+           with e when is_match_catchable e -> apply_match ist csr [])
         | Some g ->
           (try
             eval_with_fail (val_interp ist) t g
            with
            | (FailError _) as e -> raise e
-           | _ -> apply_match ist csr []))
+           | e when is_match_catchable e -> apply_match ist csr []))
     | (Pat ([],mp,mt))::tl ->
       (match mp with
       |	Term c ->
@@ -911,7 +941,7 @@ and match_interp ist ast lmr =
           (try
              val_interp
                { ist with lmatch=(apply_matching c csr)@ist.lmatch } mt
-           with | No_match | _ ->
+           with | e when is_match_catchable e ->
              apply_match ist csr tl)
         | Some g ->
           (try
@@ -919,7 +949,7 @@ and match_interp ist ast lmr =
               { ist with lmatch=(apply_matching c csr)@ist.lmatch }) mt g
            with
            | (FailError _) as e -> raise e
-           | _ -> apply_match ist csr tl))
+           | e when is_match_catchable e -> apply_match ist csr tl))
       |	Subterm (id,c) ->
         (try
            apply_sub_match ist 0 (id,c) csr mt
@@ -929,8 +959,7 @@ and match_interp ist ast lmr =
       errorlabstrm "Tacinterp.apply_match" (str
         "No matching clauses for Match") in
   let csr = constr_of_Constr (unvarg (val_interp ist (List.hd lmr)))
-  and ilr = read_match_rule ist.evc ist.env (constr_list ist.goalopt ist.lfun)
-    (List.tl lmr) in
+  and ilr = read_match_rule ist.evc ist.env (constr_list ist) (List.tl lmr) in
   apply_match ist csr ilr
 
 and tactic_interp ist ast g =
@@ -951,7 +980,7 @@ and tac_interp lfun lmatch debug ast g =
     "Interpretation gives a non-tactic value") *)
 
 (*    match (val_interp (evc,env,lfun,lmatch,(Some g),debug) ast) with
-      | VTactic tac -> (tac g)
+      | VTacticClos tac -> (tac g)
       | VFTactic (largs,f) -> (f largs g) 
       | VRTactic res -> res
       | _ ->
@@ -1077,9 +1106,16 @@ and cast_opencom_interp ist com =
 
 (* Interprets a qualified name. This can be a metavariable to be injected *)
 and qid_interp ist = function
-  | Node(loc,"QUALIDARG",p) -> interp_qualid p
+  | Node(_,"QUALIDARG",p) -> 
+    (match p with
+     | [Nvar(_,s)] ->
+       (try (make_qid (unrec (List.assoc s ist.lfun)))
+        with | Not_found -> interp_qualid p)
+     | _ -> interp_qualid p)
   | Node(loc,"QUALIDMETA",[Num(_,n)]) ->
-      Nametab.qualid_of_sp (destConst(List.assoc n ist.lmatch))
+    (try Nametab.qualid_of_sp (destConst (List.assoc n ist.lmatch)) with
+     | Invalid_argument "destConst" ->
+       make_qualid (make_dirpath []) (destVar (List.assoc n ist.lmatch)))
   | ast -> 
       anomaly_loc (Ast.loc ast, "Tacinterp.qid_interp",(str
         "Unrecognizable qualid ast: " ++ print_ast ast))
