@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: auto.ml,v 1.45 2001/12/13 09:03:10 filliatr Exp $ *)
+(* $Id: auto.ml,v 1.48 2002/03/04 13:29:41 barras Exp $ *)
 
 open Pp
 open Util
@@ -190,7 +190,7 @@ let (inAutoHint,outAutoHint) =
 (**************************************************************************)
 
 let rec nb_hyp c = match kind_of_term c with
-  | Prod(_,_,c2) -> if dependent (mkRel 1) c2 then nb_hyp c2 else 1+(nb_hyp c2)
+  | Prod(_,_,c2) -> if noccurn 1 c2 then 1+(nb_hyp c2) else nb_hyp c2
   | _ -> 0 
 
 (* adding and removing tactics in the search table *)
@@ -217,9 +217,7 @@ let make_apply_entry env sigma (eapply,verbose) name (c,cty) =
 	let pat = Pattern.pattern_of_constr c' in
         let hd = (try head_pattern_bound pat
                   with BoundPattern -> failwith "make_apply_entry") in
-        let nmiss = 
-	  List.length 
-            (clenv_missing ce (clenv_template ce,clenv_template_type ce)) 
+        let nmiss = List.length (clenv_missing ce) 
         in 
 	if eapply & (nmiss <> 0) then begin
           if verbose then 
@@ -752,10 +750,10 @@ let rec search_gen decomp n db_list local_db extra_sign goal =
 	(tclTRY_sign decomp_empty_term extra_sign)
 	::
 	(List.map 
-	   (fun id -> tclTHEN (decomp_unary_term (mkVar id)) 
-		(tclTHEN 
-		   (clear_one id)
-		   (search_gen decomp p db_list local_db [])))
+	   (fun id -> tclTHENSEQ
+               [decomp_unary_term (mkVar id);
+                clear [id];
+		search_gen decomp p db_list local_db []])
 	   (pf_ids_of_hyps goal)) 
   in
   let intro_tac = 
@@ -881,12 +879,9 @@ let compileAutoArg contac = function
            (List.map 
               (fun (id,_,typ) -> 
                 let cl = snd (decompose_prod (body_of_type typ)) in
-                 if (Hipattern.is_conjunction cl)
+                 if Hipattern.is_conjunction cl
 		 then 
-		   (tclTHEN
-                      (tclTHEN (simplest_elim (mkVar id)) 
-                         (clear_one id))
-                      contac) 
+		   tclTHENSEQ [simplest_elim (mkVar id); clear [id]; contac] 
                  else 
 		   tclFAIL 0) ctx) g)
   | UsingTDB ->  
