@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: elim.ml,v 1.23 2001/12/13 09:03:11 filliatr Exp $ *)
+(* $Id: elim.ml,v 1.27 2002/02/15 18:02:04 barras Exp $ *)
 
 open Pp
 open Util
@@ -71,8 +71,9 @@ and general_decompose recognizer id =
   elimHypThen
     (introElimAssumsThen
        (fun bas ->
-	  tclTHEN (clear_one id)
-	    (tclMAP (general_decompose_on_hyp recognizer) bas.assums)))
+	  tclTHEN (clear [id])
+	    (tclMAP (general_decompose_on_hyp recognizer)
+               (ids_of_named_context bas.assums))))
     id
 
 (* Faudrait ajouter un COMPLETE pour que l'hypothse cre ne reste
@@ -85,10 +86,11 @@ let up_to_delta = ref false (* true *)
 let general_decompose recognizer c gl = 
   let typc = pf_type_of gl c in  
   (tclTHENS (cut typc) 
-     [(tclTHEN (intro_using tmphyp_name)
-         (onLastHyp
-	    (ifOnHyp recognizer (general_decompose recognizer) clear_one)));
-      (exact_no_check c)]) gl
+    [tclTHEN (intro_using tmphyp_name)
+      (onLastHyp
+	(ifOnHyp recognizer (general_decompose recognizer)
+          (fun id -> clear [id])));
+     exact_no_check c]) gl
 
 let head_in gls indl t =
   try
@@ -97,7 +99,7 @@ let head_in gls indl t =
       then find_mrectype (pf_env gls) (project gls) t
       else extract_mrectype t
     in List.mem ity indl
-  with Induc -> false
+  with Not_found -> false
        
 let inductive_of_qualid gls qid =
   let c = 
@@ -163,19 +165,21 @@ let induction_trailer abs_i abs_j bargs =
        (fun id gls ->
 	  let idty = pf_type_of gls (mkVar id) in
 	  let fvty = global_vars (pf_env gls) idty in
-	  let possible_bring_ids =
-	    (List.tl (nLastHyps (abs_j - abs_i) gls))
-            @bargs.assums in
-	  let (ids,_) = List.fold_left 
-			  (fun (bring_ids,leave_ids) cid ->
-                             let cidty = pf_type_of gls (mkVar cid) in
-                             if not (List.mem cid leave_ids)
-                             then (cid::bring_ids,leave_ids)
-                             else (bring_ids,cid::leave_ids))
-			  ([],fvty) possible_bring_ids 
-	  in 
-	  (tclTHEN (tclTHEN (bring_hyps ids) (clear_clauses (List.rev ids)))
-             (simple_elimination (mkVar id))) gls))
+	  let possible_bring_hyps =
+	    (List.tl (nLastHyps (abs_j - abs_i) gls)) @ bargs.assums
+          in
+	  let (hyps,_) =
+            List.fold_left 
+	      (fun (bring_ids,leave_ids) (cid,_,cidty as d) ->
+                 if not (List.mem cid leave_ids)
+                 then (d::bring_ids,leave_ids)
+                 else (bring_ids,cid::leave_ids))
+	      ([],fvty) possible_bring_hyps
+	  in
+          let ids = List.rev (ids_of_named_context hyps) in
+	  (tclTHENSEQ
+            [bring_hyps hyps; clear ids; simple_elimination (mkVar id)])
+          gls))
 
 let double_ind abs_i abs_j gls =
   let cl = pf_concl gls in 
@@ -198,18 +202,17 @@ let _ = add_tactic "DoubleInd" dyn_double_ind
 (*****************************)
 
 let rec intro_pattern p = 
-  let clear_last = (tclLAST_HYP (fun c -> (clear_one (destVar c))))
-  and case_last  = (tclLAST_HYP h_simplest_case) in  
+  let clear_last = tclLAST_HYP (fun c -> (clear [destVar c]))
+  and case_last  = tclLAST_HYP h_simplest_case in  
   match p with
-    | WildPat    -> (tclTHEN intro clear_last)
-    | IdPat  id  -> intro_mustbe id
-    | DisjPat l  -> (tclTHEN introf
+    | WildPat    -> tclTHEN intro clear_last
+    | IdPat  id  -> intro_mustbe_force id
+    | DisjPat l  -> tclTHEN introf
                        (tclTHENS 
 			  (tclTHEN case_last clear_last)
-			  (List.map intro_pattern l)))
-    | ConjPat l  -> (tclTHEN introf
-                       (tclTHEN (tclTHEN  case_last clear_last)
-                          (intros_pattern l)))
+			  (List.map intro_pattern l))
+    | ConjPat l  ->
+        tclTHENSEQ [introf; case_last; clear_last; intros_pattern l]
     | ListPat l  ->  intros_pattern l
 
 and intros_pattern l = tclMAP intro_pattern l
