@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: elim.mli,v 1.4 2001/03/15 13:38:43 filliatr Exp $ i*)
+(*i $Id: elim.mli,v 1.6 2001/08/05 18:45:33 herbelin Exp $ i*)
 
 (*i*)
 open Names
@@ -24,12 +24,11 @@ val introElimAssumsThen :
 val introCaseAssumsThen :
   (branch_assumptions -> tactic) -> branch_args -> tactic
 
-val general_decompose_clause : (clause * constr -> bool) -> clause -> tactic
-val general_decompose        : (clause * constr -> bool) -> constr -> tactic
-val decompose_nonrec         : constr -> tactic
-val decompose_and            : constr -> tactic
-val decompose_or             : constr -> tactic
-val h_decompose              : identifier list -> constr -> tactic
+val general_decompose : (identifier * constr -> bool) -> constr -> tactic
+val decompose_nonrec  : constr -> tactic
+val decompose_and     : constr -> tactic
+val decompose_or      : constr -> tactic
+val h_decompose       : section_path list -> constr -> tactic
 
 val double_ind : int -> int -> tactic
 
