@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: equality.ml,v 1.72 2001/12/13 09:03:11 filliatr Exp $ *)
+(* $Id: equality.ml,v 1.82 2002/05/14 14:10:23 barras Exp $ *)
 
 open Pp
 open Util
@@ -73,6 +73,12 @@ let general_rewrite_bindings lft2rgt (c,l) gl =
       and did not fail for useless conditional rewritings generating an
       extra condition *)
 
+(* Conditional rewriting, the success of a rewriting is related 
+   to the resolution of the conditions by a given tactic *)
+let conditional_rewrite lft2rgt tac (c,bl) = 
+  tclTHEN_i (general_rewrite_bindings lft2rgt (c,bl))
+    (fun i -> if i=1 then tclIDTAC else tclCOMPLETE tac)
+
 let general_rewrite lft2rgt c = general_rewrite_bindings lft2rgt (c,[])
 
 let rewriteLR_bindings = general_rewrite_bindings true
@@ -95,6 +101,82 @@ let dyn_rewriteRL = function
       rewriteRL_bindings (c,binds)
   | _ -> assert false
 
+let dyn_conditional_rewrite lft2rgt = function
+  | [(Tacexp tac); (Command com);(Bindings binds)] -> 
+      tactic_com_bind_list 
+	(conditional_rewrite lft2rgt (Tacinterp.interp tac)) 
+	(com,binds)
+  | [(Tac (tac,_)); (Constr c);(Cbindings binds)] -> 
+      conditional_rewrite lft2rgt tac (c,binds)
+  | _ -> assert false
+                 
+let v_rewriteLR = hide_tactic "RewriteLR" dyn_rewriteLR
+let h_rewriteLR_bindings (c,bl) = v_rewriteLR [(Constr c);(Cbindings bl)] 
+let h_rewriteLR c = h_rewriteLR_bindings (c,[])
+
+let v_rewriteRL = hide_tactic "RewriteRL" dyn_rewriteRL
+let h_rewriteRL_bindings (c,bl) = v_rewriteRL [(Constr c);(Cbindings bl)] 
+let h_rewriteRL c = h_rewriteRL_bindings (c,[])
+	
+let v_conditional_rewriteLR = 
+  hide_tactic "CondRewriteLR" (dyn_conditional_rewrite true)
+let v_conditional_rewriteRL = 
+  hide_tactic "CondRewriteRL" (dyn_conditional_rewrite false)
+
+
+(* The Rewrite in tactic *)
+let general_rewrite_in lft2rgt id (c,l) gl =
+  let ctype = pf_type_of gl c in 
+  let env = pf_env gl in
+  let sigma = project gl in 
+  let _,t = splay_prod env sigma ctype in
+  match match_with_equation t with
+    | None -> (* Do not deal with setoids yet *) 
+        error "The term provided does not end with an equation" 
+    | Some (hdcncl,_) -> 
+        let hdcncls = string_of_inductive hdcncl in 
+	let suffix =
+          Indrec.elimination_suffix (elimination_sort_of_hyp id gl) in
+        let hdcncls = string_of_inductive hdcncl in 
+	let suffix =
+          Indrec.elimination_suffix (elimination_sort_of_hyp id gl) in
+        let rwr_thm =
+          if lft2rgt then hdcncls^suffix else hdcncls^suffix^"_r" in
+        let elim =
+	  try pf_global gl (id_of_string rwr_thm)
+          with Not_found ->
+            error ("Cannot find rewrite principle "^rwr_thm) in 
+	general_elim_in id (c,l) (elim,[]) gl
+
+let conditional_rewrite_in lft2rgt id tac (c,bl) = 
+  tclTHEN_i (general_rewrite_in lft2rgt id (c,bl))
+    (fun i -> if i=1 then tclIDTAC else tclCOMPLETE tac)
+
+
+let dyn_rewrite_in lft2rgt = function
+  | [Identifier id;(Command com);(Bindings binds)] -> 
+      tactic_com_bind_list (general_rewrite_in lft2rgt id) (com,binds)
+  | [Identifier id;(Constr c);(Cbindings binds)] -> 
+      general_rewrite_in lft2rgt id (c,binds)
+  | _ -> assert false
+    
+let dyn_conditional_rewrite_in lft2rgt = function
+  | [(Tacexp tac); Identifier id; (Command com);(Bindings binds)] -> 
+      tactic_com_bind_list 
+	(conditional_rewrite_in lft2rgt id (Tacinterp.interp tac)) 
+	(com,binds)
+  | [(Tac (tac,_)); Identifier id; (Constr c);(Cbindings binds)] -> 
+      conditional_rewrite_in lft2rgt id tac (c,binds)
+  | _ -> assert false
+
+let rewriteLR_in_tac =  hide_tactic "RewriteLRin" (dyn_rewrite_in true)
+let rewriteRL_in_tac = hide_tactic "RewriteRLin" (dyn_rewrite_in false)
+let v_conditional_rewriteLR_in = 
+  hide_tactic "CondRewriteLRin" (dyn_conditional_rewrite_in true) 
+let v_conditional_rewriteRL_in = 
+  hide_tactic "CondRewriteRLin" (dyn_conditional_rewrite_in false)
+
+
 (* Replacing tactics *)
 
 (* eq,symeq : equality on Set and its symmetry theorem
@@ -133,40 +215,10 @@ let dyn_replace args gl =
     | [(Constr c1);(Constr c2)] -> 
        	replace c1 c2 gl
     | _ -> assert false
-                 
-let v_rewriteLR = hide_tactic "RewriteLR" dyn_rewriteLR
-let h_rewriteLR_bindings (c,bl) = v_rewriteLR [(Constr c);(Cbindings bl)] 
-let h_rewriteLR c = h_rewriteLR_bindings (c,[])
-
-let v_rewriteRL = hide_tactic "RewriteRL" dyn_rewriteRL
-let h_rewriteRL_bindings (c,bl) = v_rewriteRL [(Constr c);(Cbindings bl)] 
-let h_rewriteRL c = h_rewriteRL_bindings (c,[])
 
 let v_replace = hide_tactic "Replace" dyn_replace
 let h_replace c1 c2 = v_replace [(Constr c1);(Constr c2)]
 
-(* Conditional rewriting, the success of a rewriting is related 
-   to the resolution of the conditions by a given tactic *)
-
-let conditional_rewrite lft2rgt tac (c,bl) = 
-  tclTHEN_i (general_rewrite_bindings lft2rgt (c,bl))
-    (fun i -> if i=1 then tclIDTAC else tclCOMPLETE tac)
-
-let dyn_conditional_rewrite lft2rgt = function
-  | [(Tacexp tac); (Command com);(Bindings binds)] -> 
-      tactic_com_bind_list 
-	(conditional_rewrite lft2rgt (Tacinterp.interp tac)) 
-	(com,binds)
-  | [(Tacexp tac); (Constr c);(Cbindings binds)] -> 
-      conditional_rewrite lft2rgt (Tacinterp.interp tac) (c,binds)
-  | _ -> assert false
-	
-let v_conditional_rewriteLR = 
-  hide_tactic "CondRewriteLR" (dyn_conditional_rewrite true)
-
-let v_conditional_rewriteRL = 
-  hide_tactic "CondRewriteRL" (dyn_conditional_rewrite false)
-
 (* End of Eduardo's code. The rest of this file could be improved
    using the functions match_with_equation, etc that I defined
    in Pattern.ml.
@@ -177,12 +229,6 @@ let v_conditional_rewriteRL =
    relation This code  will work with any equivalence relation which 
    is substitutive *)
 
-let find_constructor env sigma c =
-  let hd,stack = whd_betadeltaiota_stack env sigma c in
-  match kind_of_term hd with
-    | Construct _ -> (hd,stack)
-    | _ -> error "find_constructor"
-
 (* Patterns *)
 
 let build_coq_eq eq = eq.eq ()
@@ -373,12 +419,6 @@ let discriminable env sigma t1 t2 =
 
    the continuation then constructs the case-split.
  *)
-let push_rel_type sigma (na,c,t) env =
-  push_rel (na,c,t) env
-
-let push_rels vars env =
-  List.fold_right (fun nvar env -> push_rel_type Evd.empty nvar env) vars env
-
 let descend_then sigma env head dirn =
   let IndType (indf,_) as indt =
     try find_rectype env sigma (get_type_of env sigma head)
@@ -387,7 +427,7 @@ let descend_then sigma env head dirn =
   let (mib,mip) = lookup_mind_specif env ind in
   let cstr = get_constructors env indf in
   let dirn_nlams = cstr.(dirn-1).cs_nargs in
-  let dirn_env = push_rels cstr.(dirn-1).cs_args env in
+  let dirn_env = push_rel_context cstr.(dirn-1).cs_args env in
   (dirn_nlams,
    dirn_env,
    (fun dirnval (dfltval,resty) ->
@@ -420,7 +460,7 @@ let descend_then sigma env head dirn =
    giving [True], and all the rest giving False. *)
 
 let construct_discriminator sigma env dirn c sort =
-  let (IndType((ind,_) as indf,_) as indt) =
+  let (IndType(indf,_) as indt) =
     try find_rectype env sigma (type_of env sigma c)
     with Not_found ->
        (* one can find Rel(k) in case of dependent constructors 
@@ -431,6 +471,7 @@ let construct_discriminator sigma env dirn c sort =
       errorlabstrm "Equality.construct_discriminator"
 	(str "Cannot discriminate on inductive constructors with 
 		 dependent types") in
+  let (ind,_) = dest_ind_family indf in
   let (mib,mip) = lookup_mind_specif env ind in
   let arsign,arsort = get_arity env indf in
   let (true_0,false_0,sort_0) = 
@@ -453,8 +494,9 @@ let rec build_discriminator sigma env dirn c sort = function
   | [] -> construct_discriminator sigma env dirn c sort
   | ((sp,cnum),argnum)::l ->
       let cty = type_of env sigma c in
-      let IndType ((ind,_)as indf,_) =
+      let IndType (indf,_) =
 	try find_rectype env sigma cty with Not_found -> assert false in
+      let (ind,_) = dest_ind_family indf in
       let (mib,mip) = lookup_mind_specif env ind in
       let _,arsort = get_arity env indf in
       let nparams = mip.mind_nparams  in
@@ -539,7 +581,7 @@ let discr id gls =
 	 errorlabstrm "discr" (str" Not a discriminable equality")
      | Inl (cpath, (_,dirn), _) ->
 	 let e = pf_get_new_id (id_of_string "ee") gls in
-	 let e_env = push_named_decl (e,None,t) env in
+	 let e_env = push_named (e,None,t) env in
 	 let discriminator =
 	   build_discriminator sigma e_env dirn (mkVar e) sort cpath in
 	 let (indt,_) = find_mrectype env sigma t in 
@@ -788,7 +830,7 @@ let inj id gls =
 	   (str"Nothing to do, it is an equality between convertible terms")
     | Inr posns ->
 	let e = pf_get_new_id (id_of_string "e") gls in
-	let e_env = push_named_decl (e,None,t) env in
+	let e_env = push_named (e,None,t) env in
 	let injectors =
 	  map_succeed
 	    (fun (cpath,t1_0,t2_0) ->
@@ -812,7 +854,7 @@ let inj id gls =
 			       mkVar id]) 
 	     in
 	     let ty =
-	       try pf_type_of gls pf
+	       try pf_nf gls (pf_type_of gls pf)
                with
 		 | UserError("refiner__fail",_) -> 
 		     errorlabstrm "InjClause" 
@@ -842,7 +884,7 @@ let decompEqThen ntac id gls =
   (match find_positions env sigma t1 t2 with
      | Inl (cpath, (_,dirn), _) ->
 	 let e = pf_get_new_id (id_of_string "e") gls in
-	 let e_env = push_named_decl (e,None,t) env in
+	 let e_env = push_named (e,None,t) env in
 	 let discriminator =
 	   build_discriminator sigma e_env dirn (mkVar e) sort cpath in
 	 let (pf, absurd_term) =
@@ -851,12 +893,11 @@ let decompEqThen ntac id gls =
 	   ((tclTHENS (cut_intro absurd_term)
 	       ([onLastHyp gen_absurdity;
 		 refine (mkApp (pf, [| mkVar id |]))]))) gls
-     | Inr [] ->
-	 errorlabstrm "Equality.inj" 
-	   (str"Nothing to do, it is an equality between convertible terms")
+     | Inr [] -> (* Change: do not fail, simplify clear this trivial hyp *)
+         tclIDTAC gls
      | Inr posns ->
 	 (let e = pf_get_new_id (id_of_string "e") gls in
-	  let e_env = push_named_decl (e,None,t) env in
+	  let e_env = push_named (e,None,t) env in
 	  let injectors =
 	    map_succeed
 	      (fun (cpath,t1_0,t2_0) ->
@@ -871,16 +912,16 @@ let decompEqThen ntac id gls =
 	  if injectors = [] then
 	    errorlabstrm "Equality.decompEqThen" 
               (str "Discriminate failed to decompose the equality");
-	  ((tclTHEN
+	  (tclTHEN
 	      (tclMAP (fun (injfun,resty) ->
 			 let pf = applist(lbeq.congr (),
 					  [t;resty;injfun;t1;t2;
 					   mkVar id]) in
-			 let ty = pf_type_of gls pf in
+			 let ty = pf_nf gls (pf_type_of gls pf) in
 			 ((tclTHENS (cut ty) 
 			     ([tclIDTAC;refine pf]))))
 		 (List.rev injectors))
-	      (ntac (List.length injectors))))
+	      (ntac (List.length injectors)))
 	  gls))
 
 let decompEq = decompEqThen (fun x -> tclIDTAC)
@@ -1097,170 +1138,8 @@ let try_rewrite tac gls =
 	errorlabstrm "try_rewrite"
           (str "Cannot find a well type generalisation of the goal that" ++
              str " makes progress the proof.")
+	 
 
-(* list_int n 0 [] gives the list [1;2;...;n] *)
-let rec list_int n cmr l =
-  if cmr = n then
-    l @ [n]
-  else
-    list_int n (cmr+1) (l @ [cmr])
-
-(* Tells if two constrs are equal modulo unification *)
-
-exception NotEqModRel
-
-let eq_mod_rel l_meta t0 t1 =
-  let bindings = ref l_meta in
-  let rec eq_rec t0 t1 = 
-    match kind_of_term t1 with
-      | Meta n ->
-	  if not (List.mem_assoc n !bindings) then
-	    (bindings := (n,t0) :: !bindings; true)
-	  else (List.assoc n l_meta) = t0
-      | _ -> compare_constr eq_rec t0 t1 in
-  if eq_rec t0 t1 then !bindings else raise NotEqModRel
-
-(* Verifies if the constr has an head constant *)
-
-let is_hd_const c = match kind_of_term c with
-  | App (f,args) ->
-      (match kind_of_term f with
-         | Const c -> Some (c, args)
-         |_ -> None)
-  | _ -> None
-
-(* Gives the occurrences number of a t in u
-   Rem: t is assumed closed then there is no need to lift it
-*)
-
-let nb_occ_term t u =
-  let rec nbrec nocc u =
-    if eq_constr t u then
-      nocc + 1
-    else
-      fold_constr nbrec nocc u
-  in nbrec 0 u
-    
-
-(* Gives Some(first instance of ceq in cref,occurence number for this
-   instance) or None if no instance of ceq can be found in cref
-   Rem: t_eq is assumed closed then there is no need to lift it
-*)
-let sub_term_with_unif cref ceq =
-  let rec find_match hdsp t_args (l_meta,nb_occ) u =
-    match kind_of_term u with
-    | App(f,args) ->
-	(match kind_of_term f with
-          | Const sp when sp = hdsp -> begin
-	      try (array_fold_left2 eq_mod_rel l_meta args t_args, nb_occ+1)
-	      with NotEqModRel ->
-		Array.fold_left (find_match hdsp t_args) (l_meta,nb_occ) args
-	    end
-
-          | (Const _ | Var _ | Ind _ | Construct _ | Fix _ | CoFix _) ->
-              fold_constr (find_match hdsp t_args) (l_meta,nb_occ) u
-
-	   (* Pourquoi ne rcurre-t-on pas dans f ? *)
-           | _ -> (l_meta,nb_occ))
-
-    | _ ->
-        fold_constr (find_match hdsp t_args) (l_meta,nb_occ) u
-
-  in
-  match (is_hd_const ceq) with
-    | None ->
-        if (occur_meta ceq) then
-          None
-        else
-          let nb_occ = nb_occ_term ceq cref in
-          if nb_occ = 0 then
-            None
-          else
-            Some (ceq,nb_occ)
-    |Some (head,t_args) ->
-        let (l,nb) = find_match head t_args ([],0) cref in
-        if nb = 0 then
-          None
-        else
-          Some ((plain_instance l ceq),nb)
-	    
-(*The Rewrite in tactic*)
-let general_rewrite_in lft2rgt id (c,lb) gls =
-  let typ_id = pf_get_hyp_typ gls id in
-  let (wc,_) = startWalk gls
-  and (_,t) = pf_reduce_to_quantified_ind gls (pf_type_of gls c) in
-  let ctype = type_clenv_binding wc (c,t) lb in
-  match (match_with_equation ctype) with
-    | None -> error "The term provided does not end with an equation" 
-    | Some (hdcncl,l) ->
-        let mbr_eq =
-          if lft2rgt then
-            List.hd (List.tl (List.rev l))
-          else
-            List.hd (List.rev l)
-        in
-        (match sub_term_with_unif typ_id mbr_eq with
-           | None ->
-               errorlabstrm "general_rewrite_in" 
-		 (str "Nothing to rewrite in: " ++ pr_id id)
-           | Some (l2,nb_occ) ->
-               (tclTHENSI
-		  (tclTHEN
-		     (tclTHEN (generalize [(pf_global gls id)])
-			(reduce (Pattern [(list_int nb_occ 1 [],l2,
-					   pf_type_of gls l2)]) []))
-		     (general_rewrite_bindings lft2rgt (c,lb))) 
-		  [(tclTHEN (clear_one id) (introduction id))]) gls)
-
-let dyn_rewrite_in lft2rgt = function
-  | [Identifier id;(Command com);(Bindings binds)] -> 
-      tactic_com_bind_list (general_rewrite_in lft2rgt id) (com,binds)
-  | [Identifier id;(Constr c);(Cbindings binds)] -> 
-      general_rewrite_in lft2rgt id (c,binds)
-  | _ -> assert false
-
-let rewriteLR_in_tac =  hide_tactic "RewriteLRin" (dyn_rewrite_in true)
-let rewriteRL_in_tac = hide_tactic "RewriteRLin" (dyn_rewrite_in false)
-			 
-let conditional_rewrite_in lft2rgt id tac (c,bl) = 
-  tclTHEN_i (general_rewrite_in lft2rgt id (c,bl))
-    (fun i -> if i=1 then tclIDTAC else tclCOMPLETE tac)
-    
-let dyn_conditional_rewrite_in lft2rgt = function
-  | [(Tacexp tac); Identifier id; (Command com);(Bindings binds)] -> 
-      tactic_com_bind_list 
-	(conditional_rewrite_in lft2rgt id (Tacinterp.interp tac)) 
-	(com,binds)
-  | [(Tacexp tac); Identifier id; (Constr c);(Cbindings binds)] -> 
-      conditional_rewrite_in lft2rgt id (Tacinterp.interp tac) (c,binds)
-  | _ -> assert false
-
-let v_conditional_rewriteLR_in = 
-  hide_tactic "CondRewriteLRin" (dyn_conditional_rewrite_in true) 
-
-let v_conditional_rewriteRL_in = 
-  hide_tactic "CondRewriteRLin" (dyn_conditional_rewrite_in false)
-
-(* Rewrite c in id. Rewrite -> c in id. Rewrite <- c in id. 
-   Does not work when c is a conditional equation *)
-
-let rewrite_in lR com id gls =
-  (try 
-     let _ = lookup_named id (pf_env gls) in () 
-   with Not_found -> 
-     errorlabstrm "rewrite_in" (str"No such hypothesis : " ++pr_id id));
-  let c = pf_interp_constr gls com in
-  let eqn = pf_type_of gls c in
-  try
-    let _ = find_eq_data_decompose eqn in
-    (try 
-       (tclTHENS 
-          ((if lR then substInHyp else revSubstInHyp) eqn id) 
-          ([tclIDTAC ; exact_no_check c])) gls
-     with UserError("SubstInHyp",_) -> tclIDTAC gls)
-  with UserError ("find_eq_data_decompose",_)->  
-    errorlabstrm "rewrite_in" (str"No equality here") 
-      
 let subst eqn cls gls =
   match cls with
     | None ->    substInConcl eqn gls
@@ -1334,6 +1213,7 @@ let substConcl_RL_tac =
       (function 
 	 | [Command eqn] -> 
 	     (fun gls ->  substConcl_RL (pf_interp_constr gls eqn)  gls)
+         | [Constr c] -> substConcl_RL c
 	 | _ -> assert false)
   in 
   fun eqn  -> gentac [Command eqn] 
@@ -1697,89 +1577,3 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
         (fun l -> validation_gen nlvalid l)
     in
     (repackage sigr gl,validation_fun)
-
-(*Collects the arguments of AutoRewrite ast node*)
-(*let dyn_autorewrite largs=
-  let rec explicit_base largs =
-    let tacargs = List.map cvt_arg largs in 
-    List.map 
-      (function
-	 | Redexp ("LR", [Coqast.Node(_,"Command", [ast])]) -> ast, true
-	 | Redexp ("RL", [Coqast.Node(_,"Command", [ast])]) -> ast, false
-	 | _ -> anomaly "Equality.explicit_base") 
-      tacargs
-  and list_bases largs =
-    let tacargs = List.map cvt_arg largs in 
-    List.map 
-      (function 
-	 | Redexp ("ByName", [Coqast.Nvar(_,s)]) -> 
-	     By_name (id_of_string s)
-	 | Redexp ("Explicit", l) ->
-	     Explicit (explicit_base l)
-	 | _ -> anomaly "Equality.list_bases") 
-      tacargs
-  and int_arg=function
-    | [(Integer n)] -> n
-    | _ -> anomalylabstrm "dyn_autorewrite" 
-	  (str "Bad call of int_arg (not an INTEGER)")
-  and list_args_rest (lstep,evstep) (ostep,evostep) (lrest,evrest)
-    (orest,evorest) (depth,evdepth) = function
-      | [] -> (lstep,ostep,lrest,orest,depth)
-      | (Redexp (s,l))::tail ->
-	  if s="Step" & not evstep then
-            list_args_rest ((List.map Tacinterp.interp l),true) (ostep,evostep)
-              (lrest,evrest) (orest,evorest) (depth,evdepth) tail
-	  else if s="SolveStep" & not evostep then
-            list_args_rest (lstep,evstep) (Solve,true) (lrest,evrest)
-              (orest,evorest) (depth,evdepth) tail
-	  else if s="Use" & not evostep then
-            list_args_rest (lstep,evstep) (Use,true) (lrest,evrest) 
-	      (orest,evorest) (depth,evdepth) tail
-	  else if s="All" & not evostep then
-            list_args_rest (lstep,evstep) (All,true) (lrest,evrest) 
-	      (orest,evorest) (depth,evdepth) tail
-	  else if s="Rest" & not evrest then
-            list_args_rest (lstep,evstep) (ostep,evostep) 
-	      ((List.map Tacinterp.interp l),true) (orest,evorest) 
-	      (depth,evdepth) tail
-	  else if s="SolveRest" & not evorest then
-            list_args_rest (lstep,evstep) (ostep,evostep) (lrest,evrest)
-              (false,true) (depth,evdepth) tail
-	  else if s="Cond" & not evorest then
-            list_args_rest (lstep,evstep) (ostep,evostep) (lrest,evrest)
-              (true,true) (depth,evdepth) tail
-	  else if s="Depth" & not evdepth then
-            (let dth = int_arg (List.map cvt_arg l) in
-             if dth > 0 then
-               list_args_rest (lstep,evstep) (ostep,evostep) (lrest,evrest)
-		 (orest,evorest) (dth,true) tail
-             else
-               errorlabstrm "dyn_autorewrite" 
-		 (str "Depth value lower or equal to 0"))
-	  else
-            anomalylabstrm "dyn_autorewrite" 
-	      (str "Bad call of list_args_rest")
-      | _ -> 
-	  anomalylabstrm "dyn_autorewrite" 
-	    (str "Bad call of list_args_rest")
-  and list_args = function
-    | (Redexp (s,lbases))::tail ->
-	if s = "BaseList" then
-          (let (lstep,ostep,lrest,orest,depth) = 
-	     list_args_rest ([],false) (Solve,false) ([],false) (false,false) 
-	       (100,false) tail
-           in
-           autorewrite (list_bases lbases) 
-	     (if lstep = [] then None else Some lstep) 
-	     ostep (if lrest=[] then None else Some lrest) orest depth)
-	else
-          anomalylabstrm "dyn_autorewrite" 
-	    (str "Bad call of list_args (not a BaseList tagged REDEXP)")
-    | _ ->
-	anomalylabstrm "dyn_autorewrite" 
-	  (str "Bad call of list_args (not a REDEXP)")
-  in
-  list_args largs*)
-
-(*Adds and hides the AutoRewrite tactic*)
-(*let h_autorewrite = hide_tactic "AutoRewrite" dyn_autorewrite*)
