@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: inv.mli,v 1.5 2001/03/15 13:38:43 filliatr Exp $ i*)
+(*i $Id: inv.mli,v 1.6 2001/08/10 14:42:21 herbelin Exp $ i*)
 
 (*i*)
 open Names
@@ -24,4 +24,4 @@ val half_dinv_with : identifier -> constr -> tactic
 val dinv_with : identifier -> constr -> tactic
 val dinv_clear_with : identifier -> constr -> tactic
 
-val invIn_tac : string -> identifier -> identifier list -> tactic
+val invIn_tac : identifier -> identifier -> identifier list -> tactic
