@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: setoid_replace.ml,v 1.10 2001/12/13 09:03:11 filliatr Exp $ *)
+(* $Id: setoid_replace.ml,v 1.13.2.1 2002/10/01 09:16:29 clrenard Exp $ *)
 
 open Tacmach
 open Proof_type
@@ -78,7 +78,7 @@ let coq_fleche = lazy(constant ["Setoid"] "fleche")
 
 (* Coq constants *)
 
-let coqeq = lazy(global_constant ["Logic"] "eq")
+let coqeq = lazy(global_constant ["Logic_Type"] "eqT")
 
 let coqconj = lazy(global_constant ["Logic"] "conj")
 
@@ -162,6 +162,7 @@ let find_theory a =
     
 (* Add a Setoid to the database after a type verification. *)
 
+(* Proof of (a,b,c,d:A)(eq a b)->(eq c d)->(eq a c)->(eq b d) *)
 let eq_lem_proof env a eq sym trans = 
   lambda_create env
     (a, lambda_create env
@@ -178,6 +179,7 @@ let eq_lem_proof env a eq sym trans =
 					 [|(mkRel 7);(mkRel 5);(mkRel 4);(mkRel 1);(mkRel 2)|]))|]
 			      )))))))))
 
+(* Proof of (a,b,c,d:A)(eq a b)->(eq c d)->((eq a c)<->(eq b d)) *)
 let eq_lem2_proof env a eq sym trans = 
   lambda_create env
     (a, lambda_create env
@@ -301,7 +303,7 @@ let gen_lem_name m = match kind_of_term m with
   | Const sp -> add_suffix (basename sp) "_ext"
   | Ind (sp, i) -> add_suffix (basename sp) ((string_of_int i)^"_ext")
   | Construct ((sp,i),j) -> add_suffix
-      (basename sp) ((string_of_int i)^(string_of_int i)^"_ext")
+      (basename sp) ((string_of_int i)^(string_of_int j)^"_ext")
   | _ -> errorlabstrm "New Morphism" (str "The term " ++ prterm m ++ str "is not a known name")
 
 let gen_lemma_tail m lisset body n =
