@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: tacticals.ml,v 1.43 2001/11/12 15:56:09 clrenard Exp $ *)
+(* $Id: tacticals.ml,v 1.49 2002/03/04 13:29:43 barras Exp $ *)
 
 open Pp
 open Util
@@ -48,6 +48,9 @@ let tclDO            = Tacmach.tclDO
 let tclPROGRESS      = Tacmach.tclPROGRESS
 let tclWEAK_PROGRESS = Tacmach.tclWEAK_PROGRESS
 
+(* [rclTHENSEQ [t1;..;tn] is equivalent to t1;..;tn *)
+let tclTHENSEQ = List.fold_left tclTHEN tclIDTAC
+
 (* map_tactical f [x1..xn] = (f x1);(f x2);...(f xn) *)
 (* tclMAP f [x1..xn] = (f x1);(f x2);...(f xn) *)
 let tclMAP tacfun l = 
@@ -153,7 +156,7 @@ let allHyps gl = pf_ids_of_hyps gl
    after id *)
 
 let afterHyp id gl =
-  fst (list_splitby (fun hyp -> hyp = id) (pf_ids_of_hyps gl))
+  fst (list_splitby (fun (hyp,_,_) -> hyp = id) (pf_hyps gl))
     
 
 (* Create a singleton clause list with the last hypothesis from then context *)
@@ -164,7 +167,7 @@ let lastHyp gl = List.hd (pf_ids_of_hyps gl)
 (* Create a clause list with the n last hypothesis from then context *)
 
 let nLastHyps n gl =
-  try list_firstn n (pf_ids_of_hyps gl)
+  try list_firstn n (pf_hyps gl)
   with Failure "firstn" -> error "Not enough hypotheses in the goal"
 
 
@@ -249,68 +252,33 @@ type branch_args = {
 
 type branch_assumptions = {
   ba        : branch_args;     (* the branch args *)
-  assums    : identifier list; (* the list of assumptions introduced *)
+  assums    : named_context; (* the list of assumptions introduced *)
   cargs     : identifier list; (* the constructor arguments *)
   constargs : identifier list; (* the CONSTANT constructor arguments *)
   recargs   : identifier list; (* the RECURSIVE constructor arguments *)
   indargs   : identifier list} (* the inductive arguments *)
 
-(* Hum ... the following function looks quite similar to the one 
- * (previously) defined with the same name in Tactics.ml. 
- *  --Eduardo (11/8/97) *)
-
-(* This function reduces less than Tacred.reduce_to_quantified_ind (no
-   beta/iota reduction if not induced by a constant), but since it is called
-   with types coming from pf_type_of which betaiota reduces, it should
-   behaves the same.  Moreover, the env in the call to pf_one_step_reduce
-   is not synchronous with the local environment (HH 9/9/01)
-
-   Let's try to replace it by pf_reduce_to_quantified_ind
-
-let pf_one_step_reduce = pf_reduce one_step_reduce
-
-let reduce_to_ind_goal gl t = 
-  let rec elimrec t =
-    let c,args = decomp_app t in
-    match kind_of_term c with
-      | Ind (ind_sp,args as ity) -> 
-	  ((ity, path_of_inductive_path ind_sp, t), t)
-      | Cast (c,_) when args = [] ->
-	  elimrec c
-      | Prod (n,ty,t') when args = [] ->
-	  let (ind, t) = elimrec t' in (ind, mkProd (n,ty,t))
-      | LetIn (n,c,ty,t') when args = [] ->
-	  let (ind, t) = elimrec t' in (ind, mkLetIn (n,c,ty,t))
-      | _ when Instantiate.isEvalRef c ->
-	  elimrec (pf_nf_betaiota gl (pf_one_step_reduce gl t))
-      | _ -> error "Not an inductive product"
-  in 
-  elimrec t
-*)
-
-let reduce_to_ind_goal = pf_reduce_to_quantified_ind
-
-let case_sign ity i = 
-  let rec analrec acc = function 
-    | [] -> acc 
-    | (c::rest) -> analrec (false::acc) rest
-  in
-  let (mib,mip) = Global.lookup_inductive ity in
-  let recarg = mip.mind_listrec in 
-  analrec [] recarg.(i-1)
-
-let elim_sign ity i =
-  let (_,j) = ity in
-  let rec analrec acc = function 
-    | (Param(_)::rest) -> analrec (false::acc) rest
-    | (Norec::rest)    -> analrec (false::acc) rest
-    | (Imbr(_)::rest)  -> analrec (false::acc) rest
-    | (Mrec k::rest)   -> analrec ((j=k)::acc) rest
-    | []               -> List.rev acc 
+let compute_construtor_signatures isrec (_,k as ity) =
+  let rec analrec c recargs =
+    match kind_of_term c, recargs with 
+    | Prod (_,_,c), recarg::rest ->
+	  (match dest_recarg recarg with
+	    | Norec   -> false :: (analrec c rest)
+	    | Imbr _  -> false :: (analrec c rest)
+	    | Mrec j  -> (isrec & j=k) :: (analrec c rest))
+    | LetIn (_,_,_,c), rest -> false :: (analrec c rest)
+    | _, [] -> []
+    | _ -> anomaly "compute_construtor_signatures"
   in 
   let (mib,mip) = Global.lookup_inductive ity in
-  let recarg = mip.mind_listrec in 
-  analrec [] recarg.(i-1)
+  let n = mip.mind_nparams in
+  let lc =
+    Array.map (fun c -> snd (decompose_prod_n_assum n c)) mip.mind_nf_lc in
+  let lrecargs = dest_subterms mip.mind_recargs in
+  array_map2 analrec lc lrecargs
+
+let case_sign = compute_construtor_signatures false
+let elim_sign = compute_construtor_signatures true
 
 let elimination_sort_of_goal gl = 
   match kind_of_term (hnf_type_of gl (pf_concl gl)) with 
@@ -321,6 +289,15 @@ let elimination_sort_of_goal gl =
 	   | Type _ -> InType)
     | _        -> anomaly "goal should be a type"
 
+let elimination_sort_of_hyp id gl = 
+  match kind_of_term (hnf_type_of gl (pf_get_hyp_typ gl id)) with 
+    | Sort s ->
+	(match s with
+	   | Prop Null -> InProp
+	   | Prop Pos -> InSet
+	   | Type _ -> InType)
+    | _        -> anomaly "goal should be a type"
+
 
 (* Find the right elimination suffix corresponding to the sort of the goal *)
 (* c should be of type A1->.. An->B with B an inductive definition *)
@@ -331,7 +308,7 @@ let last_arg c = match kind_of_term c with
 
 let general_elim_then_using 
   elim elim_sign_fun tac predicate (indbindings,elimbindings) c gl =
-  let (ity,t) = reduce_to_ind_goal gl (pf_type_of gl c) in
+  let (ity,t) = pf_reduce_to_quantified_ind gl (pf_type_of gl c) in
   (* applying elimination_scheme just a little modified *)
   let (wc,kONT)  = startWalk gl in
   let indclause  = mk_clenv_from wc (c,t) in
@@ -357,13 +334,14 @@ let general_elim_then_using
   in
   let elimclause' = clenv_fchain indmv elimclause indclause' in
   let elimclause' = clenv_constrain_with_bindings elimbindings elimclause' in
+  let branchsigns = elim_sign_fun ity in
   let after_tac ce i gl =
     let (hd,largs) = decompose_app (clenv_template_type ce).rebus in
-    let branchsign =  elim_sign_fun ity i in
-    let ba = { branchsign = branchsign;
+    let ba = { branchsign = branchsigns.(i-1);
                nassums = 
 		 List.fold_left 
-                   (fun acc b -> if b then acc+2 else acc+1) 0 branchsign;
+                   (fun acc b -> if b then acc+2 else acc+1)
+                   0 branchsigns.(i-1);
                branchnum = i;
                ity = ity;
                largs = List.map (clenv_instance_term ce) largs;
@@ -380,7 +358,7 @@ let general_elim_then_using
 
 
 let elimination_then_using tac predicate (indbindings,elimbindings) c gl = 
-  let (ind,t) = reduce_to_ind_goal gl (pf_type_of gl c) in
+  let (ind,t) = pf_reduce_to_quantified_ind gl (pf_type_of gl c) in
   let elim =
     Indrec.lookup_eliminator ind (elimination_sort_of_goal gl) in
   general_elim_then_using
@@ -392,16 +370,16 @@ let simple_elimination_then tac = elimination_then tac ([],[])
 
 let case_then_using tac predicate (indbindings,elimbindings) c gl =
   (* finding the case combinator *)
-  let (ity,t) = reduce_to_ind_goal gl (pf_type_of gl c) in
+  let (ity,t) = pf_reduce_to_quantified_ind gl (pf_type_of gl c) in
   let sigma = project gl in 
   let sort  = elimination_sort_of_goal gl  in
-  let elim  = Indrec.make_case_gen (pf_env gl) sigma ity sort in  
+  let elim  = Indrec.make_case_dep (pf_env gl) sigma ity sort in  
   general_elim_then_using 
     elim case_sign tac predicate (indbindings,elimbindings) c gl
 
 let case_nodep_then_using tac predicate (indbindings,elimbindings) c gl =
   (* finding the case combinator *)
-  let (ity,t) = reduce_to_ind_goal gl (pf_type_of gl c) in
+  let (ity,t) = pf_reduce_to_quantified_ind gl (pf_type_of gl c) in
   let sigma = project gl in 
   let sort  = elimination_sort_of_goal gl  in
   let elim  = Indrec.make_case_nodep (pf_env gl) sigma ity sort in  
@@ -419,22 +397,22 @@ let make_elim_branch_assumptions ba gl =
             constargs = constargs;
             recargs   = recargs;
             indargs   = indargs}
-      | ((true::tl), (recarg::indarg::idtl)) ->
+      | ((true::tl), ((idrec,_,_ as recarg)::(idind,_,_ as indarg)::idtl)) ->
 	  makerec (recarg::indarg::assums,
-		   recarg::cargs,
-		   recarg::recargs,
+		   idrec::cargs,
+		   idrec::recargs,
 		   constargs,
-		   indarg::indargs) tl idtl
-      | ((false::tl), (constarg::idtl))      ->
+		   idind::indargs) tl idtl
+      | ((false::tl), ((id,_,_ as constarg)::idtl))      ->
 	  makerec (constarg::assums,
-		   constarg::cargs,
-		   constarg::constargs,
+		   id::cargs,
+		   id::constargs,
 		   recargs,
 		   indargs) tl idtl
       | (_, _) -> error "make_elim_branch_assumptions"
   in 
   makerec ([],[],[],[],[]) ba.branchsign
-    (try list_firstn ba.nassums (pf_ids_of_hyps gl)
+    (try list_firstn ba.nassums (pf_hyps gl)
      with Failure _ -> anomaly "make_elim_branch_assumptions")
 
 let elim_on_ba tac ba gl = tac (make_elim_branch_assumptions ba gl) gl
@@ -449,20 +427,20 @@ let make_case_branch_assumptions ba gl =
             constargs = constargs;
             recargs   = recargs;
             indargs   = []}
-      | ((true::tl), (recarg::idtl)) ->
+      | ((true::tl), ((idrec,_,_ as recarg)::idtl)) ->
 	  makerec (recarg::assums,
-		   recarg::cargs,
-		   recarg::recargs,
+		   idrec::cargs,
+		   idrec::recargs,
 		   constargs) tl idtl
-      | ((false::tl), (constarg::idtl)) ->
+      | ((false::tl), ((id,_,_ as constarg)::idtl)) ->
 	  makerec (constarg::assums,
-		   constarg::cargs,
+		   id::cargs,
 		   recargs,
-		   constarg::constargs) tl idtl
+		   id::constargs) tl idtl
       | (_, _) -> error "make_case_branch_assumptions"
   in 
   makerec ([],[],[],[]) ba.branchsign
-    (try list_firstn ba.nassums (pf_ids_of_hyps gl)
+    (try list_firstn ba.nassums (pf_hyps gl)
      with Failure _ -> anomaly "make_case_branch_assumptions")
 
 let case_on_ba tac ba gl = tac (make_case_branch_assumptions ba gl) gl
