@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: tacticals.mli,v 1.23 2001/09/19 16:53:52 herbelin Exp $ i*)
+(*i $Id: tacticals.mli,v 1.28 2002/03/20 16:57:48 courant Exp $ i*)
 
 (*i*)
 open Names
@@ -29,8 +29,10 @@ val tclTHEN_i        : tactic -> (int -> tactic) -> tactic
 val tclTHENL         : tactic -> tactic -> tactic
 val tclTHENS         : tactic -> tactic list -> tactic
 val tclTHENSi        : tactic -> tactic list -> (int -> tactic) -> tactic
+val tclTHENSEQ       : tactic list -> tactic
 val tclREPEAT        : tactic -> tactic
 val tclFIRST         : tactic list -> tactic
+val tclSOLVE         : tactic list -> tactic
 val tclTRY           : tactic -> tactic
 val tclINFO          : tactic -> tactic
 val tclCOMPLETE      : tactic -> tactic
@@ -65,9 +67,9 @@ val pf_matches : goal sigma -> constr_pattern -> constr -> (int * constr) list
 val pf_is_matching : goal sigma -> constr_pattern -> constr -> bool
 
 val allHyps    : goal sigma -> identifier list
-val afterHyp   : identifier -> goal sigma -> identifier list
+val afterHyp   : identifier -> goal sigma -> named_context
 val lastHyp    : goal sigma -> identifier
-val nLastHyps  : int -> goal sigma -> identifier list
+val nLastHyps  : int -> goal sigma -> named_context
 
 val allClauses : goal sigma -> clause list
 
@@ -87,10 +89,10 @@ val ifOnHyp        :
   (identifier * types -> bool) ->
     (identifier -> tactic) -> (identifier -> tactic) -> identifier -> tactic
 
-val onHyps         : (goal sigma -> identifier list) -> 
-                     (identifier list -> tactic) -> tactic
+val onHyps         : (goal sigma -> named_context) -> 
+                     (named_context -> tactic) -> tactic
 val tryAllHyps     : (identifier -> tactic) -> tactic
-val onNLastHyps    : int -> (identifier -> tactic) -> tactic
+val onNLastHyps    : int -> (named_declaration -> tactic) -> tactic
 val onLastHyp      : (identifier -> tactic) -> tactic
 
 (* [ConclPattern concl pat tacast]:
@@ -113,17 +115,17 @@ type branch_args = {
 
 type branch_assumptions = {
   ba        : branch_args;     (* the branch args *)
-  assums    : identifier list; (* the list of assumptions introduced *)
+  assums    : named_context;   (* the list of assumptions introduced *)
   cargs     : identifier list; (* the constructor arguments *)
   constargs : identifier list; (* the CONSTANT constructor arguments *)
   recargs   : identifier list; (* the RECURSIVE constructor arguments *)
   indargs   : identifier list} (* the inductive arguments *)
 
 val elimination_sort_of_goal : goal sigma -> sorts_family
-(*i val suff              : goal sigma -> constr -> string i*)
+val elimination_sort_of_hyp  : identifier -> goal sigma -> sorts_family
 
 val general_elim_then_using :
-  constr ->  (inductive -> int -> bool list) -> 
+  constr ->  (inductive -> bool list array) -> 
     (branch_args -> tactic) -> constr option -> 
       (arg_bindings * arg_bindings) -> constr -> tactic
 	  
