@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: tactics.ml,v 1.100 2001/12/13 09:03:11 filliatr Exp $ *)
+(* $Id: tactics.ml,v 1.113.2.1 2002/07/30 17:28:24 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -101,10 +101,11 @@ let convert_concl   = Tacmach.convert_concl
 let convert_hyp     = Tacmach.convert_hyp
 let thin            = Tacmach.thin 
 let thin_body       = Tacmach.thin_body 
-let move_hyp        = Tacmach.move_hyp 
+let move_hyp        = Tacmach.move_hyp
+let rename_hyp      = Tacmach.rename_hyp
 
 let mutual_fix   = Tacmach.mutual_fix
-let fix f n      = mutual_fix [f] [n] []
+let fix f n      = mutual_fix f n []
 
 let fix_noname n =  
   let id = Pfedit.get_current_proof_name () in
@@ -115,18 +116,18 @@ let dyn_mutual_fix argsl gl =
     | [Integer n]                          -> fix_noname n gl
     | [Identifier id;Integer n]            -> fix id n gl
     | ((Identifier id)::(Integer n)::lfix) -> 
-	let rec decomp lid ln lar = function
+	let rec decomp lar = function
           | (Fixexp (id,n,ar)::rest) -> 
-	      decomp (id::lid) (n::ln) (ar::lar) rest
-          | [] -> (List.rev lid,List.rev ln,List.rev lar)
+	      decomp ((id,n,pf_interp_constr gl ar)::lar) rest
+          | [] -> (List.rev lar)
 	  | _  -> bad_tactic_args "mutual_fix" argsl
 	in
-	let (lid,ln,lar) = decomp [] [] [] lfix in
-	mutual_fix (id::lid) (n::ln) (List.map (pf_interp_constr gl) lar) gl
+	let lar = decomp [] lfix in
+	mutual_fix id n lar gl
     | l -> bad_tactic_args "mutual_fix" l 
 
 let mutual_cofix = Tacmach.mutual_cofix
-let cofix f      =  mutual_cofix [f] []
+let cofix f      =  mutual_cofix f []
 
 let cofix_noname n =  
   let id = Pfedit.get_current_proof_name () in
@@ -137,14 +138,14 @@ let dyn_mutual_cofix argsl gl =
     | []                       -> cofix_noname gl
     | [(Identifier id)]        -> cofix id gl
     | ((Identifier id)::lcofix) -> 
-	let rec decomp lid lar = function 
+	let rec decomp lar = function 
           | (Cofixexp (id,ar)::rest) -> 
-              decomp (id::lid) (ar::lar) rest
-          | [] -> (List.rev lid,List.rev lar)
+              decomp ((id,pf_interp_constr gl ar)::lar) rest
+          | [] -> List.rev lar
 	  |  _ -> bad_tactic_args "mutual_cofix" argsl
 	in
-	let (lid,lar) = decomp [] [] lcofix in
-        mutual_cofix (id::lid) (List.map (pf_interp_constr gl) lar) gl
+	let lar = decomp [] lcofix in
+        mutual_cofix id lar gl
    | l -> bad_tactic_args "mutual_cofix" l 
 
 
@@ -160,7 +161,7 @@ type tactic_reduction = env -> evar_map -> constr -> constr
 
 let reduct_in_concl redfun gl = 
   convert_concl (pf_reduce redfun gl (pf_concl gl)) gl
-    
+
 let reduct_in_hyp redfun idref gl =
   let inhyp,id = match idref with
     | InHyp id -> true, id
@@ -168,12 +169,12 @@ let reduct_in_hyp redfun idref gl =
   let (_,c, ty) = pf_get_hyp gl id in
   let redfun' = under_casts (pf_reduce redfun gl) in
   match c with
-    | None -> convert_hyp id (redfun' ty) gl
+    | None -> convert_hyp (id,None,redfun' ty) gl
     | Some b ->
 	if inhyp then (* Default for defs: reduce in body *)
-	  convert_defbody id (redfun' b) gl
+	  convert_hyp (id,Some (redfun' b),ty) gl
 	else
-	  convert_deftype id (redfun' ty) gl
+	  convert_hyp (id,Some b,redfun' ty) gl
 
 let reduct_option redfun = function
   | Some id -> reduct_in_hyp   redfun id 
@@ -327,7 +328,7 @@ let rec intro_gen name_flag move_flag force_flag gl =
     else
       raise e
 
-let intro_mustbe id = intro_gen (IntroMustBe id) None false
+let intro_mustbe_force id = intro_gen (IntroMustBe id) None true
 let intro_using id = intro_gen (IntroBasedOn (id,[])) None false
 let intro_force force_flag = intro_gen (IntroAvoid []) None force_flag
 let intro = intro_force false
@@ -371,7 +372,7 @@ let dyn_intro_move = function
   | l -> bad_tactic_args "intro_move" l
 
 let rec intros_until s g =
-  match pf_lookup_name_as_renamed (pf_hyps g) (pf_concl g) s with
+  match pf_lookup_name_as_renamed (pf_env g) (pf_concl g) s with
     | Some depth -> tclDO depth intro g
     | None -> 
 	try
@@ -382,7 +383,7 @@ let rec intros_until s g =
 	      str " even after head-reduction")
 
 let rec intros_until_n_gen red n g =
-  match pf_lookup_index_as_renamed (pf_concl g) n with
+  match pf_lookup_index_as_renamed (pf_env g) (pf_concl g) n with
     | Some depth -> tclDO depth intro g
     | None ->
       if red then
@@ -497,13 +498,13 @@ let apply_type hdcty argl gl =
 let apply_term hdc argl gl =
   refine (applist (hdc,argl)) gl
 
-let bring_hyps ids gl = 
-  let newcl = 
-    List.fold_right 
-      (fun id cl' -> mkNamedProd id (pf_type_of gl (mkVar id)) cl')
-      ids (pf_concl gl)
-  in 
-  apply_type newcl (List.map mkVar ids) gl
+let bring_hyps hyps = 
+  if hyps = [] then Refiner.tclIDTAC
+  else
+    (fun gl ->
+      let newcl = List.fold_right mkNamedProd_or_LetIn hyps (pf_concl gl) in
+      let f = mkCast (mkMeta (new_meta()),newcl) in
+      refine (mkApp (f, instance_from_named_context hyps)) gl)
 
 (* Resolution with missing arguments *)
 
@@ -595,7 +596,7 @@ let true_cut_anon c gl =
   match kind_of_term (hnf_type_of gl c) with
     | Sort s -> 
         let d = match s with Prop _ -> "H" | Type _ -> "X" in
-        let id = next_name_away_with_default d Anonymous (pf_ids_of_hyps gl) in
+        let id = fresh_id [] (id_of_string d) gl in
         internal_cut id c gl
     | _  -> error "Not a proposition or a type"
 
@@ -738,13 +739,35 @@ let quantify lconstr =
 let letin_abstract id c (occ_ccl,occ_hyps) gl =
   let everywhere = (occ_ccl = None) & (occ_hyps = []) in
   let env = pf_env gl in
+  let compute_dependency _ (hyp,_,_ as d) ctxt =
+    let d' =
+      try
+        let occ = if everywhere then [] else List.assoc hyp occ_hyps in
+        let newdecl = subst_term_occ_decl env occ c d in
+        if d = newdecl then
+	  if not everywhere then raise (RefinerError (DoesNotOccurIn (c,hyp)))
+	  else raise Not_found
+        else 
+	  (subst1_decl (mkVar id) newdecl, true)
+      with Not_found -> 
+      (d,List.exists (fun((id,_,_),dep) -> dep && occur_var_in_decl env id d) ctxt)
+    in d'::ctxt
+  in 
+  let ctxt' = fold_named_context compute_dependency env ~init:[] in
+  let compute_marks ((depdecls,marks as accu),lhyp) ((hyp,_,_) as d,b) =
+    if b then ((d::depdecls,(hyp,lhyp)::marks), lhyp)
+    else (accu, Some hyp) in
+  let (depdecls,marks),_ = List.fold_left compute_marks (([],[]),None) ctxt' in
+(*
   let abstract ((depdecls,marks as accu),lhyp) (hyp,_,_ as d) =
     try
       let occ = if everywhere then [] else List.assoc hyp occ_hyps in
-      let newdecl = subst_term_occ_decl occ c d in
+      let newdecl = subst_term_occ_decl env occ c d in
       if d = newdecl then
 	if not everywhere then raise (RefinerError (DoesNotOccurIn (c,hyp)))
-	else (accu, Some hyp)
+	else
+          if List.exists (fun (id,_,_) -> occur_var id d) 
+          (accu, Some hyp)
       else 
 	let newdecl = subst1_decl (mkVar id) newdecl in
 	((newdecl::depdecls,(hyp,lhyp)::marks), lhyp)
@@ -752,21 +775,20 @@ let letin_abstract id c (occ_ccl,occ_hyps) gl =
       (accu, Some hyp) 
   in
   let (depdecls,marks),_ =
-    fold_named_context_reverse abstract ~init:(([],[]),None) env in
+    fold_named_context_reverse abstract ~init:(([],[]),None) env in(*  *)
+*)
   let occ_ccl = if everywhere then Some [] else occ_ccl in
   let ccl = match occ_ccl with
     | None -> pf_concl gl
-    | Some occ -> subst1 (mkVar id) (subst_term_occ occ c (pf_concl gl))
+    | Some occ -> subst1 (mkVar id) (subst_term_occ env occ c (pf_concl gl))
   in
   (depdecls,marks,ccl)
 
 let letin_tac with_eq name c occs gl =
   let x = id_of_name_using_hdchar (Global.env()) (pf_type_of gl c) name in
-  let env = pf_env gl in
-  let used_ids = ids_of_context env in
   let id =
-    if name = Anonymous then next_ident_away x used_ids else
-      if not (mem_named_context x (named_context env)) then x else
+    if name = Anonymous then fresh_id [] x gl else
+      if not (mem_named_context x (pf_hyps gl)) then x else
 	error ("The variable "^(string_of_id x)^" is already declared") in
   let (depdecls,marks,ccl)= letin_abstract id c occs gl in 
   let ctxt =
@@ -776,14 +798,8 @@ let letin_tac with_eq name c occs gl =
       depdecls in
   let t = pf_type_of gl c in
   let tmpcl = List.fold_right mkNamedProd_or_LetIn depdecls ccl in
-  let args = List.map (fun (id,_,_) -> mkVar id) depdecls in
+  let args = Array.to_list (instance_from_named_context depdecls) in
   let newcl = mkNamedLetIn id c t tmpcl in
-(*
-    if with_eq then
-    else (* To fix : add c to args, or use LetIn and clear the body *)
-      mkNamed id t tmpcl
-  in
-*)
   let lastlhyp = if marks=[] then None else snd (List.hd marks) in
   tclTHENLIST
     [ apply_type newcl args;
@@ -812,11 +828,17 @@ let dyn_lettac args gl = match args with
       letin_tac true (Name id) c (o,l) gl
   | l -> bad_tactic_args "letin" l
 
+let nowhere = (Some [],[])
+
 let dyn_forward args gl = match args with
-  | [Command com; Identifier id] ->
-      letin_tac false (Name id) (pf_interp_constr gl com) (None,[]) gl
-  | [Constr c; Identifier id]    ->
-      letin_tac false (Name id) c (None,[]) gl
+  | [Quoted_string s; Command com; Identifier id] ->
+      letin_tac (s="KeepBody") (Name id) (pf_interp_constr gl com) nowhere gl
+  | [Quoted_string s; Constr c; Identifier id]    ->
+      letin_tac (s="KeepBody") (Name id) c nowhere gl
+  | [Quoted_string s; Constr c] ->
+      letin_tac (s="KeepBody") Anonymous c nowhere gl
+  | [Quoted_string s; Command c] ->
+      letin_tac (s="KeepBody") Anonymous (pf_interp_constr gl c) nowhere gl
   | l -> bad_tactic_args "forward" l
 
 (********************************************************************)
@@ -876,11 +898,12 @@ let dyn_assumption = function
  * goal. *)                                                               
 
 let clear ids gl    = thin ids gl
-let clear_one id gl = clear [id] gl
 let dyn_clear = function
   | [Clause ids] ->
-      let out = function InHyp id -> id | _ -> assert false in
-      clear (List.map out ids)
+      if ids=[] then tclIDTAC
+      else
+        let out = function InHyp id -> id | _ -> assert false in
+        clear (List.map out ids)
   | l -> bad_tactic_args "clear" l
 
 let clear_body = thin_body
@@ -890,20 +913,6 @@ let dyn_clear_body = function
       clear_body (List.map out ids)
   | l -> bad_tactic_args "clear_body" l
 
-(* Clears a list of identifiers clauses form the context *)
-(*
-let clear_clauses clsl =
-  clear (List.map 
-           (function 
-	      | Some id -> id
-              | None    -> error "ThinClauses") clsl)
-*)
-let clear_clauses = clear
-
-(* Clears one identifier clause from the context *)
-
-let clear_clause cls = clear_clauses [cls]
-
 
 (*   Takes a list of booleans, and introduces all the variables 
  *  quantified in the goal which are associated with a value
@@ -913,15 +922,15 @@ let rec intros_clearing = function
   | []          -> tclIDTAC
   | (false::tl) -> tclTHEN intro (intros_clearing tl)
   | (true::tl)  ->
-      tclTHENLIST [ intro; onLastHyp clear_clause; intros_clearing tl]
+      tclTHENLIST
+        [ intro; onLastHyp (fun id -> clear [id]); intros_clearing tl]
 
 (* Adding new hypotheses  *)
 
-let new_hyp mopt c blist g =
+let new_hyp mopt c lbind g =
   let (wc,kONT) = startWalk g in
-  let clause  = mk_clenv_printable_type_of wc c in
-  let clause' = clenv_match_args blist clause in
-  let (thd,tstack) = whd_stack (clenv_instance_template clause') in
+  let clause  = make_clenv_binding wc (c,w_type_of wc c) lbind in
+  let (thd,tstack) = whd_stack (clenv_instance_template clause) in
   let nargs = List.length tstack in
   let cut_pf = 
     applist(thd, 
@@ -955,17 +964,23 @@ let dyn_new_hyp argsl gl =
 
 let dyn_move = function
   | [Identifier idfrom; Identifier idto] -> move_hyp false idfrom idto
-  | _ -> assert false
+  | l -> bad_tactic_args "move" l 
 
 let dyn_move_dep = function
   | [Identifier idfrom; Identifier idto] -> move_hyp true idfrom idto
-  | _ -> assert false
+  | l -> bad_tactic_args "move_dep" l 
+
+(* Renaming hypotheses *)
+
+let dyn_rename = function
+  | [Identifier idfrom; Identifier idto] -> rename_hyp idfrom idto
+  | l -> bad_tactic_args "rename" l 
 
 (************************)
 (* Introduction tactics *)
 (************************)
 
-let constructor_checking_bound  boundopt i lbind gl =
+let constructor_tac boundopt i lbind gl =
   let cl = pf_concl gl in 
   let (mind,redcl) = pf_reduce_to_quantified_ind gl cl in 
   let nconstr =
@@ -983,7 +998,7 @@ let constructor_checking_bound  boundopt i lbind gl =
   let apply_tac = apply_with_bindings (cons,lbind) in
   (tclTHENLIST [convert_concl redcl; intros; apply_tac]) gl
 
-let one_constructor i = (constructor_checking_bound None i)
+let one_constructor i = constructor_tac None i
 
 let any_constructor gl = 
   let cl = pf_concl gl in 
@@ -995,13 +1010,30 @@ let any_constructor gl =
   tclFIRST (List.map (fun i -> one_constructor i []) 
               (interval 1 nconstr)) gl
 
+(* Try to apply the constructor of the inductive definition followed by 
+   a tactic t given as an argument.
+   Should be generalize in Constructor (Fun c : I -> tactic)
+ *)
+
+let tclConstrThen t gl = 
+  let mind = fst (pf_reduce_to_quantified_ind gl (pf_concl gl))
+  in let lconstr =
+       (snd (Global.lookup_inductive mind)).mind_consnames
+  in let nconstr = Array.length lconstr 
+  in
+  if nconstr = 0 then error "The type has no constructors";
+  tclFIRST (List.map (fun i -> (tclTHEN (one_constructor i []) t))
+              (interval 1 nconstr)) gl
+
 let dyn_constructor = function 
   | [Integer i; Bindings binds]  -> tactic_bind_list (one_constructor i) binds
   | [Integer i; Cbindings binds] -> (one_constructor i) binds
+  | [Tac (tac,_)] -> tclConstrThen tac
   | []                           -> any_constructor 
   | l                            -> bad_tactic_args "constructor" l
-                       
-let left           = (constructor_checking_bound (Some 2) 1)
+
+
+let left           = constructor_tac (Some 2) 1
 let simplest_left  = left  []
 
 let dyn_left = function 
@@ -1009,7 +1041,7 @@ let dyn_left = function
   | [Bindings binds]  -> tactic_bind_list left binds 
   | l                 -> bad_tactic_args "left" l
 
-let right          = (constructor_checking_bound (Some 2) 2)
+let right          = constructor_tac (Some 2) 2
 let simplest_right = right []
 
 let dyn_right  = function 
@@ -1018,7 +1050,7 @@ let dyn_right  = function
   | l                  -> bad_tactic_args "right" l
 
 
-let split          = (constructor_checking_bound (Some 1) 1)
+let split          = constructor_tac (Some 1) 1
 let simplest_split = split []
 
 let dyn_split  = function 
@@ -1042,7 +1074,7 @@ let last_arg c = match kind_of_term c with
   | App (f,cl) ->  array_last cl
   | _ -> anomaly "last_arg"
 	
-let elimination_clause_scheme kONT wc elimclause indclause gl = 
+let elimination_clause_scheme kONT elimclause indclause gl = 
   let indmv = 
     (match kind_of_term (last_arg (clenv_template elimclause).rebus) with
        | Meta mv -> mv
@@ -1073,7 +1105,7 @@ let general_elim (c,lbindc) (elimc,lbindelimc) gl =
   let indclause  = make_clenv_binding wc (c,t) lbindc  in
   let elimt      = w_type_of wc elimc in
   let elimclause = make_clenv_binding wc (elimc,elimt) lbindelimc in 
-  elimination_clause_scheme kONT wc elimclause indclause gl
+  elimination_clause_scheme kONT elimclause indclause gl
 
 (* Elimination tactic with bindings but using the default elimination 
  * constant associated with the type. *)
@@ -1090,6 +1122,43 @@ let default_elim (c,lbindc)  gl =
 
 let simplest_elim c = default_elim (c,[])
 
+(* Elimination in hypothesis *)
+
+let elimination_in_clause_scheme kONT id elimclause indclause =
+  let (hypmv,indmv) = 
+    match clenv_independent elimclause with
+        [k1;k2] -> (k1,k2)
+      | _  -> errorlabstrm "elimination_clause"
+          (str "The type of elimination clause is not well-formed") in
+  let elimclause'  = clenv_fchain indmv elimclause indclause in 
+  let hyp = mkVar id in
+  let hyp_typ = clenv_type_of elimclause' hyp in
+  let hypclause =
+    mk_clenv_from_n elimclause'.hook (Some 0) (hyp, hyp_typ) in
+  let elimclause'' = clenv_fchain hypmv elimclause' hypclause in  
+  let new_hyp_prf  = clenv_instance_template elimclause'' in
+  let new_hyp_typ  = clenv_instance_template_type elimclause'' in
+  if eq_constr hyp_typ new_hyp_typ then
+    errorlabstrm "general_rewrite_in" 
+      (str "Nothing to rewrite in " ++ pr_id id);
+  tclTHEN
+    (kONT elimclause''.hook)
+    (tclTHENS
+      (cut new_hyp_typ)
+      [ (* Try to insert the new hyp at the same place *)
+        tclORELSE (intro_replacing id)
+          (tclTHEN (clear [id]) (introduction id));
+        refine new_hyp_prf])
+
+let general_elim_in id (c,lbindc) (elimc,lbindelimc) gl = 
+  let (wc,kONT)  = startWalk gl in
+  let ct = pf_type_of gl c in
+  let t = try snd (pf_reduce_to_quantified_ind gl ct) with UserError _ -> ct in
+  let indclause  = make_clenv_binding wc (c,t) lbindc  in
+  let elimt      = w_type_of wc elimc in
+  let elimclause = make_clenv_binding wc (elimc,elimt) lbindelimc in 
+  elimination_in_clause_scheme kONT id elimclause indclause gl
+
 (*
  * A "natural" induction tactic
  * 
@@ -1129,8 +1198,7 @@ let rec is_rec_arg env sigma indpath t =
   try
     let (ind_sp,_) = find_mrectype env sigma t in
     path_of_inductive env ind_sp = indpath
-  with Induc -> 
-    false
+  with Not_found -> false
 
 let rec recargs indpath env sigma t =
   match kind_of_term (whd_betadeltaiota env sigma t) with
@@ -1406,7 +1474,7 @@ let induction_tac varname typ (elimc,elimt) gl =
   let (wc,kONT)  = startWalk gl                    in
   let indclause  = make_clenv_binding wc (c,typ) []  in
   let elimclause = make_clenv_binding wc (mkCast (elimc,elimt),elimt) [] in
-  elimination_clause_scheme kONT wc elimclause indclause gl
+  elimination_clause_scheme kONT elimclause indclause gl
 
 let is_indhyp p n t =
   let c,_ = decompose_app t in 
@@ -1418,19 +1486,18 @@ let is_indhyp p n t =
 (* eliminator _ind, _rec or _rect, or eliminator built by Scheme) *)
 let compute_elim_signature_and_roughly_check elimt mind =
   let (mib,mip) = Global.lookup_inductive mind in
-  let lra = mip.mind_listrec in
+  let lra = dest_subterms mip.mind_recargs in
   let nconstr = Array.length mip.mind_consnames in
   let _,elimt2 = decompose_prod_n mip.mind_nparams elimt in
   let n = nb_prod elimt2 in
   let npred = n - nconstr - mip.mind_nrealargs - 1 in
   let rec check_branch p c ra = match kind_of_term c, ra with
-    | Prod (_,_,c), Declarations.Mrec i :: ra' ->
-	(match kind_of_term c with
-	   | Prod (_,t,c) when is_indhyp (p+1) npred t ->
+    | Prod (_,_,c), r :: ra' ->
+	(match dest_recarg r, kind_of_term c with
+	   | Mrec i, Prod (_,t,c) when is_indhyp (p+1) npred t ->
 	       true::(check_branch (p+2) c ra')
 	   | _ -> false::(check_branch (p+1) c ra'))
     | LetIn (_,_,_,c), ra' -> false::(check_branch (p+1) c ra)
-    | Prod (_,_,c), _ :: ra -> false::(check_branch (p+1) c ra)
     | _, [] -> []
     | _ ->
 	error"Not a recursive eliminator: some constructor argument is lacking"
@@ -1568,7 +1635,9 @@ let general_case_analysis (c,lbindc) gl =
   let (mind,_) = pf_reduce_to_quantified_ind gl (pf_type_of gl c) in
   let sigma    = project gl in 
   let sort     = elimination_sort_of_goal gl in
-  let elim     = Indrec.make_case_gen env sigma mind sort in 
+  let case = if occur_term c (pf_concl gl) then Indrec.make_case_dep
+  else Indrec.make_case_gen in
+  let elim     = case env sigma mind sort in 
   general_elim (c,lbindc) (elim,[]) gl
     
 let simplest_case c = general_case_analysis (c,[])
@@ -1615,7 +1684,7 @@ let elim_scheme_type elim t gl =
     | Meta mv ->
         let clause' =
 	  (* t is inductive, then CUMUL or CONV is irrelevant *)
-	  clenv_unify CUMUL t (clenv_instance_type clause mv) clause in
+	  clenv_unify true CUMUL t (clenv_instance_type clause mv) clause in
 	elim_res_pf kONT clause' gl
     | _ -> anomaly "elim_scheme_type"
 
