@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: tactics.mli,v 1.31 2001/11/12 15:56:10 clrenard Exp $ i*)
+(*i $Id: tactics.mli,v 1.40 2002/04/11 13:25:07 herbelin Exp $ i*)
 
 (*i*)
 open Names
@@ -43,11 +43,12 @@ exception Bound
 val introduction    : identifier -> tactic
 val refine          : constr -> tactic
 val convert_concl   : constr -> tactic
-val convert_hyp     : identifier -> constr -> tactic
+val convert_hyp     : named_declaration -> tactic
 val thin            : identifier list -> tactic
-val mutual_fix      : identifier list -> int list -> constr list -> tactic
+val mutual_fix      :
+  identifier -> int -> (identifier * int * constr) list -> tactic
 val fix             : identifier -> int -> tactic
-val mutual_cofix    : identifier list -> constr list -> tactic
+val mutual_cofix    : identifier -> (identifier * constr) list -> tactic
 val cofix           : identifier -> tactic
 
 val dyn_mutual_fix   : tactic_arg list -> tactic
@@ -67,7 +68,7 @@ val dyn_intro_move       : tactic_arg list -> tactic
 
 val intro_replacing      : identifier -> tactic
 val intro_using          : identifier -> tactic
-val intro_mustbe         : identifier -> tactic
+val intro_mustbe_force   : identifier -> tactic
 val intros_using         : identifier list -> tactic
 val intro_erasing        : identifier -> tactic
 val intros_replacing     : identifier list -> tactic
@@ -152,26 +153,24 @@ val pattern_option    :
 (*s Modification of the local context. *)
 
 val clear         : identifier list -> tactic
-val clear_one     : identifier -> tactic
 val dyn_clear     : tactic_arg list -> tactic
 
 val clear_body    : identifier list -> tactic
 val dyn_clear_body : tactic_arg list -> tactic
 
-val clear_clauses : identifier list -> tactic
-val clear_clause  : identifier -> tactic
-
 val new_hyp       : int option ->constr -> constr substitution -> tactic
 val dyn_new_hyp   : tactic_arg list -> tactic
 
 val dyn_move      : tactic_arg list -> tactic
 val dyn_move_dep  : tactic_arg list -> tactic
 
+val dyn_rename    : tactic_arg list -> tactic
+
 (*s Resolution tactics. *)
 
 val apply_type : constr -> constr list -> tactic
 val apply_term : constr -> constr list -> tactic
-val bring_hyps : identifier list -> tactic
+val bring_hyps : named_context -> tactic
 
 val apply                 : constr      -> tactic
 val apply_without_reduce  : constr      -> tactic
@@ -190,6 +189,9 @@ val default_elim   : constr * constr substitution -> tactic
 val simplest_elim  : constr -> tactic
 val dyn_elim       : tactic_arg list -> tactic
 
+val general_elim_in : identifier -> constr * constr substitution ->
+                      constr * constr substitution -> tactic
+
 val old_induct       : identifier -> tactic
 val old_induct_nodep : int        -> tactic
 val dyn_old_induct : tactic_arg list -> tactic
@@ -227,10 +229,11 @@ val dorE : bool -> clause ->tactic
 
 (*s Introduction tactics. *)
 
-val constructor_checking_bound : int option -> int -> 
+val constructor_tac            : int option -> int -> 
                                  constr substitution  -> tactic
 val one_constructor            : int -> constr substitution  -> tactic
 val any_constructor            : tactic
+val tclConstrThen              : tactic -> tactic
 val left                       : constr substitution -> tactic
 val simplest_left              : tactic
 val right                      : constr substitution -> tactic
@@ -268,6 +271,8 @@ val cut                         : constr -> tactic
 val cut_intro                   : constr -> tactic
 val cut_replacing               : identifier -> constr -> tactic
 val cut_in_parallel             : constr list -> tactic
+val true_cut                    : identifier -> constr -> tactic
+val true_cut_anon               : constr -> tactic
 val dyn_cut                     : tactic_arg list -> tactic
 val dyn_true_cut                : tactic_arg list -> tactic
 val dyn_lettac                  : tactic_arg list -> tactic
