@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: command.ml,v 1.71 2001/12/13 09:03:12 filliatr Exp $ *)
+(* $Id: command.ml,v 1.73 2002/02/15 18:02:05 barras Exp $ *)
 
 open Pp
 open Util
@@ -204,8 +204,7 @@ let interp_mutual lparams lnamearconstrs finite =
 	   List.map 
 	     (interp_type_with_implicits sigma ind_env_params ind_impls) bodies
 	 in
-	 { mind_entry_nparams = nparams;
-	   mind_entry_params = params';
+	 { mind_entry_params = params';
 	   mind_entry_typename = name;
 	   mind_entry_arity = ar;
 	   mind_entry_consnames = constrnames;
@@ -277,7 +276,7 @@ let build_recursive lnameargsardef =
            let arity = interp_type sigma env0 raw_arity in
            let _ = declare_variable recname
 	     (Lib.cwd(),SectionLocalAssum arity, NeverDischarge) in
-           (Environ.push_named_decl (recname,None,arity) env, (arity::arl)))
+           (Environ.push_named (recname,None,arity) env, (arity::arl)))
         (env0,[]) lnameargsardef
     with e ->
       States.unfreeze fs; raise e in
@@ -341,7 +340,7 @@ let build_corecursive lnameardef =
 	   let arity = arj.utj_val in
            let _ = declare_variable recname
 	     (Lib.cwd(),SectionLocalAssum arj.utj_val,NeverDischarge) in
-           (Environ.push_named_decl (recname,None,arity) env, (arity::arl)))
+           (Environ.push_named (recname,None,arity) env, (arity::arl)))
         (env0,[]) lnameardef
     with e -> 
       States.unfreeze fs; raise e in 
