@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: command.mli,v 1.23 2001/11/06 13:44:29 clrenard Exp $ i*)
+(*i $Id: command.mli,v 1.24 2002/03/17 04:57:32 delahaye Exp $ i*)
 
 (*i*)
 open Names
@@ -21,6 +21,14 @@ open Nametab
    functions of [Declare]; they return an absolute reference to the
    defined object *)
 
+val constant_entry_of_com :
+  Coqast.t * Coqast.t option * bool -> Safe_typing.constant_entry
+
+val declare_global_definition :
+  Names.identifier ->
+  Safe_typing.constant_entry ->
+  Declare.strength -> bool -> Nametab.global_reference
+
 val definition_body : identifier -> bool * strength -> 
   Coqast.t -> Coqast.t option -> global_reference
 
