@@ -6,13 +6,13 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: discharge.mli,v 1.5 2001/03/15 13:38:58 filliatr Exp $ i*)
+(*i $Id: discharge.mli,v 1.6 2001/08/10 14:42:22 herbelin Exp $ i*)
+
+open Names
 
 (* This module implements the discharge mechanism. It provides a function to
    close the last opened section. That function calls [Lib.close_section] and
    then re-introduce all the discharged versions of the objects that were
    defined in the section. *)
 
-val close_section : bool -> string -> unit
-
-val save_module_to : string -> string -> unit
+val close_section : bool -> identifier -> unit
