@@ -6,8 +6,6 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: recordobj.mli,v 1.2 2001/03/15 13:38:59 filliatr Exp $ *)
+(* $Id: recordobj.mli,v 1.4 2001/11/05 16:48:30 barras Exp $ *)
 
-open Term
-
-val objdef_declare : global_reference -> unit
+val objdef_declare : Nametab.global_reference -> unit
