@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: usage.ml,v 1.10 2001/09/18 15:33:06 herbelin Exp $ *)
+(* $Id: usage.ml,v 1.13 2002/04/10 15:49:50 barras Exp $ *)
 
 let version () =
   Printf.printf "The Coq Proof Assistant, version %s (%s)\n"
@@ -33,14 +33,14 @@ let print_usage_channel co command =
   -load-ml-object f      load ML object file f 
   -load-ml-source f      load ML file f 
   -load-vernac-source f  load Coq file f.v (Load f.)
+  -l f                   (idem) 
   -load-vernac-object f  load Coq object file f.vo
   -require f             load Coq object file f.vo and import it (Require f.)
   -compile f             compile Coq file f.v (implies -batch)
   -compile-verbose f     verbosely compile Coq file f.v (implies -batch)
 
   -opt                   run the native-code version of Coq or Coq_SearchIsos
-  -bindir dir            specify an alternative directory for the binaries
-  -libdir dir            specify an alternative directory for the library
+  -byte                  run the bytecode version of Coq or Coq_SearchIsos
 
   -where                 print Coq's standard library location and exit
   -v                     print Coq version and exit
@@ -51,6 +51,7 @@ let print_usage_channel co command =
   -batch                 batch mode (exits just after arguments parsing)
   -boot                  boot mode (implies -q and -batch)
   -emacs                 tells Coq it is executed under Emacs
+  -dump-glob f           dump globalizations in file f (to be used by coqdoc)
 "
 
 (* print the usage on standard error *)
