@@ -6,12 +6,13 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: vernac.ml,v 1.19 2001/12/13 09:03:13 filliatr Exp $ *)
+(* $Id: vernac.ml,v 1.20 2002/02/14 14:39:07 filliatr Exp $ *)
 
 (* Parsing of vernacular. *)
 
 open Pp
 open Util
+open Options
 open System
 open Coqast
 open Vernacinterp
@@ -165,6 +166,7 @@ let compile verbosely f =
     let ldir = Nameops.extend_dirpath ldir0 m in
     Termops.set_module ldir; (* Just for universe naming *)
     Lib.start_module ldir;
+    if !dump then dump_string ("F" ^ Names.string_of_dirpath ldir ^ "\n");
     let _ = load_vernac verbosely longf in
     let mid = Lib.end_module m in
     assert (mid = ldir);
