@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: vernac.mli,v 1.4 2001/03/15 13:38:59 filliatr Exp $ i*)
+(*i $Id: vernac.mli,v 1.5 2001/09/18 15:33:07 herbelin Exp $ i*)
 
 (* Parsing of vernacular. *)
 
@@ -33,3 +33,8 @@ val raw_do_vernac : Pcoq.Gram.parsable -> unit
    and location *)
 
 val load_vernac : bool -> string -> unit
+
+
+(* Compile a vernac file, verbosely or not (f is assumed without .v suffix) *)
+
+val compile : bool -> string -> unit
